Package: rmoo
Title: Multi-Objective Optimization in R
Version: 0.3.2
Date: 2026-05-03
Authors@R: 
    c(person(given = "Francisco",
             family = "Benitez",
             role = c("aut", "cre"),
             email = "benitezfj94@gmail.com"),
      person(given = "Diego P.",
             family = "Pinto-Roa",
             role = "aut",
             email = "dpinto@pol.una.py",
             comment = c(ORCID = "0000-0003-2479-9876")))
Description: The 'rmoo' package is a framework for multi- and many-objective
        optimization, which allows researchers and users versatility
        in parameter configuration, as well as tools for analysis, replication
        and visualization of results. The 'rmoo' package was built as a fork of
        the 'GA' package by Luca Scrucca(2017) <DOI:10.32614/RJ-2017-008> and
        implementing the Non-Dominated Sorting Genetic Algorithms proposed
        by K. Deb's.
License: GPL (>= 2)
Encoding: UTF-8
Language: en
LazyData: true
RoxygenNote: 7.3.3
Collate: 'AllClasses.R' 'associate.R' 'crowding_distance.R' 'data.R'
        'generate_reference_points.R' 'geneticoperator.R'
        'get_fixed_rowsum_integer_matrix.R' 'miscfun.R' 'AllGenerics.R'
        'niching.R' 'non_dominated_fronts.R' 'parallel.R' 'utils.R'
        'nsga.R' 'nsga2.R' 'nsga3.R' 'rnsga2.R'
        'modified_crowding_distance.R' 'rmooControl.R'
        'performance_metrics.R' 'reference_point_multi_layer.R'
        'rmoo.R' 'rmoo_main.R' 'sharing.R' 'update_points.R' 'zzz.R'
Imports: stats, utils, graphics, methods, foreach, GA, grDevices,
        plotly, ggplot2, BBmisc
URL: https://github.com/Evolutionary-Optimization-Laboratory/rmoo/
BugReports: https://github.com/Evolutionary-Optimization-Laboratory/rmoo/issues/
Suggests: testthat, covr, rgl, ecr, emoa, cdata, dplyr, reshape2,
        parallel, doParallel, doRNG (>= 1.6)
Depends: R (>= 2.10)
NeedsCompilation: no
Packaged: 2026-05-03 21:27:26 UTC; Maria
Author: Francisco Benitez [aut, cre],
  Diego P. Pinto-Roa [aut] (ORCID:
    <https://orcid.org/0000-0003-2479-9876>)
Maintainer: Francisco Benitez <benitezfj94@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 00:00:02 UTC
Built: R 4.4.3; ; 2026-05-04 02:43:28 UTC; unix

Package: rmoo
Title: Multi-Objective Optimization in R
Version: 0.3.2
Date: 2026-05-03
Authors@R: 
    c(person(given = "Francisco",
             family = "Benitez",
             role = c("aut", "cre"),
             email = "benitezfj94@gmail.com"),
      person(given = "Diego P.",
             family = "Pinto-Roa",
             role = "aut",
             email = "dpinto@pol.una.py",
             comment = c(ORCID = "0000-0003-2479-9876")))
Description: The 'rmoo' package is a framework for multi- and many-objective
        optimization, which allows researchers and users versatility
        in parameter configuration, as well as tools for analysis, replication
        and visualization of results. The 'rmoo' package was built as a fork of
        the 'GA' package by Luca Scrucca(2017) <DOI:10.32614/RJ-2017-008> and
        implementing the Non-Dominated Sorting Genetic Algorithms proposed
        by K. Deb's.
License: GPL (>= 2)
Encoding: UTF-8
Language: en
LazyData: true
RoxygenNote: 7.3.3
Collate: 'AllClasses.R' 'associate.R' 'crowding_distance.R' 'data.R'
        'generate_reference_points.R' 'geneticoperator.R'
        'get_fixed_rowsum_integer_matrix.R' 'miscfun.R' 'AllGenerics.R'
        'niching.R' 'non_dominated_fronts.R' 'parallel.R' 'utils.R'
        'nsga.R' 'nsga2.R' 'nsga3.R' 'rnsga2.R'
        'modified_crowding_distance.R' 'rmooControl.R'
        'performance_metrics.R' 'reference_point_multi_layer.R'
        'rmoo.R' 'rmoo_main.R' 'sharing.R' 'update_points.R' 'zzz.R'
Imports: stats, utils, graphics, methods, foreach, GA, grDevices,
        plotly, ggplot2, BBmisc
URL: https://github.com/Evolutionary-Optimization-Laboratory/rmoo/
BugReports: https://github.com/Evolutionary-Optimization-Laboratory/rmoo/issues/
Suggests: testthat, covr, rgl, ecr, emoa, cdata, dplyr, reshape2,
        parallel, doParallel, doRNG (>= 1.6)
Depends: R (>= 2.10)
NeedsCompilation: no
Packaged: 2026-05-03 21:27:26 UTC; Maria
Author: Francisco Benitez [aut, cre],
  Diego P. Pinto-Roa [aut] (ORCID:
    <https://orcid.org/0000-0003-2479-9876>)
Maintainer: Francisco Benitez <benitezfj94@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 00:00:02 UTC
Built: R 4.6.0; ; 2026-05-04 03:32:51 UTC; unix

# Generated by roxygen2: do not edit by hand

export(PerformScalarizing)
export(UpdateIdealPoint)
export(UpdateWorstPoint)
export(associate_to_niches)
export(calc_norm_pref_distance)
export(compute_niche_count)
export(compute_perpendicular_distance)
export(crowding_distance)
export(generate_reference_points)
export(generational_distance)
export(getCrowdingDistance)
export(getDummyFitness)
export(getFitness)
export(getMetrics)
export(getPopulation)
export(get_fixed_rowsum_integer_matrix)
export(get_nadir_point)
export(modifiedCrowdingDistance)
export(niching)
export(non_dominated_fronts)
export(nsga)
export(nsga2)
export(nsga3)
export(plot)
export(print)
export(progress)
export(reference_point_multi_layer)
export(rmoo)
export(rmooControl)
export(rmooMonitor)
export(rmoo_huxCrossover)
export(rmoo_lrSelection)
export(rmoo_spCrossover)
export(rmoo_tourSelection)
export(rmoo_uxCrossover)
export(rmoo_uxMutation)
export(rmoobin_Population)
export(rmoobin_huxCrossover)
export(rmoobin_lrSelection)
export(rmoobin_raMutation)
export(rmoobin_spCrossover)
export(rmoobin_tourSelection)
export(rmoobin_uxCrossover)
export(rmoobin_uxMutation)
export(rmooint_Population)
export(rmooint_huxCrossover)
export(rmooint_spCrossover)
export(rmooint_uxCrossover)
export(rmooint_uxMutation)
export(rmooperm_Population)
export(rmooperm_lrSelection)
export(rmooperm_oxCrossover)
export(rmooperm_simMutation)
export(rmooperm_tourSelection)
export(rmooreal_Population)
export(rmooreal_lrSelection)
export(rmooreal_polMutation)
export(rmooreal_raMutation)
export(rmooreal_sbxCrossover)
export(rmooreal_spCrossover)
export(rmooreal_tourSelection)
export(rnsga2)
export(scale_reference_directions)
export(sharing)
export(startParallel)
export(stopParallel)
export(summary)
exportClasses(algorithm)
exportClasses(nsga)
exportClasses(nsga1)
exportClasses(nsga2)
exportClasses(nsga3)
exportClasses(numberOrNAOrMatrix)
exportClasses(rnsga2)
import(foreach)
import(graphics)
import(methods)
import(stats)
import(utils)
importFrom(BBmisc,which.first)
importFrom(GA,ga)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,label_value)
importFrom(grDevices,rainbow)
importFrom(plotly,"%>%")
importFrom(plotly,add_markers)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# rmoo - R Multi-Objective Optimization <img src="man/figures/logo.png" align="right" width="150px" />

<!-- badges: start -->

[![R build
status](https://github.com/Evolutionary-Optimization-Laboratory/rmoo/workflows/R-CMD-check/badge.svg/)](https://github.com/Evolutionary-Optimization-Laboratory/rmoo/actions/)
[![Codecov test
coverage](https://codecov.io/gh/Evolutionary-Optimization-Laboratory/rmoo/branch/master/graph/badge.svg?token=QK4Z2yVUSw/)](https://app.codecov.io/gh/Evolutionary-Optimization-Laboratory/rmoo?branch=master)
[![CRAN
status](https://www.r-pkg.org/badges/version/rmoo)](https://CRAN.R-project.org/package=rmoo/)
[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable/)
<!-- badges: end -->

## Overview

`rmoo` is a non-dominated sorting based multi-objective optimization
package built upon the [‘GA’
package](https://CRAN.R-project.org/package=GA). It provides a
comprehensive, flexible, and modular framework for multi/many-objective
optimization. Users have a wide range of configuration options at their
disposal, including the representation of real-values, permutations, and
binaries.

The algorithms available in `rmoo` include GA, NSGA-I, NSGA-II,
R-NSGA-II, and NSGA-III.

## Installation

You can install the **stable** version on [R
CRAN](https://cran.r-project.org/package=rmoo):

``` r
install.packages("rmoo")
```

Or you can install the **development** version from
[GitHub](https://github.com/Evolutionary-Optimization-Laboratory/rmoo):

``` r
# install.packages("devtools")
devtools::install_github("Evolutionary-Optimization-Laboratory/rmoo")
```

## Usage

All the algorithms implemented in `rmoo` are called throught rmoo
function. It will receive all the necessary parameters for execution. In
this simple example, we solve the DTLZ1 problem, using the NSGA-III:

``` r
library(rmoo)

DTLZ1 <- function (x, nobj = 3, ...) 
{
    if (is.null(dim(x))) {
        x <- matrix(x, 1)
    }
    n <- ncol(x)
    y <- matrix(x[, 1:(nobj - 1)], nrow(x))
    z <- matrix(x[, nobj:n], nrow(x))
    g <- 100 * (n - nobj + 1 + rowSums((z - 0.5)^2 - cos(20 * 
        pi * (z - 0.5))))
    tmp <- t(apply(y, 1, cumprod))
    tmp <- cbind(t(apply(tmp, 1, rev)), 1)
    tmp2 <- cbind(1, t(apply(1 - y, 1, rev)))
    f <- tmp * tmp2 * 0.5 * (1 + g)
    return(f)
}

ref_points <- generate_reference_points(3,12)

result <- rmoo(fitness = DTLZ1,
               type = "real-valued",
               algorithm = "NSGA-III",
               lower = c(0,0,0),
               upper = c(1,1,1),
               monitor = FALSE,
               summary = FALSE,
               parallel = FALSE,
               nObj = 3,
               reference_dirs = ref_points,
               popSize = 92,
               maxiter = 300)
```

The rmoo package has a set of S4 method that can help the user to
visualize, analysis, and interpret the results.

We are going to show how use the `plot` method, since depending on the
parameter it will display difference plots, e.g. passing “pcp” as
parameter for type, the method displays the Parallel Coordinate Plot:

``` r
plot(result, type="pcp")
```

![](https://github.com/Evolutionary-Optimization-Laboratory/rmoo/blob/master/man/figures/README-example-1.png)<!-- -->

Another example of plot method is when no parameters are passed, the
method by default display the scatter plot. It evaluates the result
dimensionality and depending on this, it will display 2-D, 3-D or
Pairwise Scatter Plot.

``` r
#Scatter without optimal points
plot(result)
```

![](https://github.com/Evolutionary-Optimization-Laboratory/rmoo/blob/master/man/figures/README-example-2.png)<!-- -->

When displaying the scatter plot, reference points can be passed as
parameters of the optimal argument, allowing the results to be compared
to them:

``` r
#Scatter with optimal points (Using reference points as optimal points)
plot(result, optimal = result@reference_points)
```

![](https://github.com/Evolutionary-Optimization-Laboratory/rmoo/blob/master/man/figures/README-example-3.png)<!-- -->

Other plots available in `rmoo` are heatmap and polar coordinate, both
allow the user to view specific solutions.

``` r
#Polar Coordinates
plot(result, type="polar", individual = c(1:3))
```

![](https://github.com/Evolutionary-Optimization-Laboratory/rmoo/blob/master/man/figures/README-example-4.png)<!-- -->

``` r
#Heatmap Plot
plot(result, type="heatmap", individual = c(1:3))
```

![](https://github.com/Evolutionary-Optimization-Laboratory/rmoo/blob/master/man/figures/README-example-5.png)<!-- -->

Other methods available in `rmoo` that may be of interest are
`getFitness()`, `getPopulation()`, `getMetrics()`, `print()`,
`summary()` and others. We do not show the functionality of all these
functions since they will be detailed in depth in the future article and
vignettes.

## Citation

Until the submission of the formal article that introduces **rmoo**,
please cite using:

> Benitez F., Pinto-Roa Diego P. (2023). *rmoo: Multi-Objective
> Optimization in R*. R package version 0.2.3
> <https://CRAN.R-project.org/package=rmoo/>.

BibTeX entries for LaTeX users:

    @Manual{,
        title = {{rmoo}: Multi-Objective Optimization in R},
        author = {Francisco Benitez and Diego P. {Pinto-Roa}},
        year = {2023},
        note = {R package version 0.2.3},
        url = {https://CRAN.R-project.org/package=rmoo/},
    }

or

    @INPROCEEDINGS{10418638,
        author={Francisco, J. Benítez and Diego, P. Pinto-Roa and Miguel, García-Torres and Parameshachari, B. D.},
        booktitle={2023 IEEE CHILEAN Conference on Electrical, Electronics Engineering, Information and Communication Technologies (CHILECON)}, 
        title={A Hybrid Approach for Many-Objective Feature Selection in Intrusion Detection on Windows Operating Systems}, 
        year={2023},
        volume={},
        number={},
        pages={1-6},
        doi={10.1109/CHILECON60335.2023.10418638/}}

<!-- README.md is generated from README.Rmd. Please edit that file -->

# rmoo - R Multi-Objective Optimization <img src="man/figures/logo.png" align="right" width="150px" />

<!-- badges: start -->

[![R build
status](https://github.com/Evolutionary-Optimization-Laboratory/rmoo/workflows/R-CMD-check/badge.svg/)](https://github.com/Evolutionary-Optimization-Laboratory/rmoo/actions/)
[![Codecov test
coverage](https://codecov.io/gh/Evolutionary-Optimization-Laboratory/rmoo/branch/master/graph/badge.svg?token=QK4Z2yVUSw/)](https://app.codecov.io/gh/Evolutionary-Optimization-Laboratory/rmoo?branch=master)
[![CRAN
status](https://www.r-pkg.org/badges/version/rmoo)](https://CRAN.R-project.org/package=rmoo/)
[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable/)
<!-- badges: end -->

## Overview

A Non-Dominated Sorting based Multi-Objective Optimization package,
built upon the [‘GA’ package](https://CRAN.R-project.org/package=GA).

`rmoo` provides a complete, flexible and modular framework for
optimizing multiple supplied objectives. You will have at your disposal
a wide range of configuration options for the NSGA, NSGA-II and NSGA-III
algorithms, as well as representation of real numbers, permutations and
binaries.

## Installation

You can install the **stable** version on [R
CRAN](https://cran.r-project.org/package=rmoo):

``` r
install.packages("rmoo")
```

Or you can install the **development** version from
[GitHub](https://github.com/Evolutionary-Optimization-Laboratory/rmoo):

``` r
# install.packages("devtools")
devtools::install_github("Evolutionary-Optimization-Laboratory/rmoo")
```

## Usage

All the algorithms implemented in `rmoo` are called throught rmoo
function. It will receive all the necessary parameters for execution. In
this simple example, we solve the DTLZ1 problem, using the NSGA-III:

``` r
library(rmoo)

DTLZ1 <- function (x, nobj = 3, ...) 
{
    if (is.null(dim(x))) {
        x <- matrix(x, 1)
    }
    n <- ncol(x)
    y <- matrix(x[, 1:(nobj - 1)], nrow(x))
    z <- matrix(x[, nobj:n], nrow(x))
    g <- 100 * (n - nobj + 1 + rowSums((z - 0.5)^2 - cos(20 * 
        pi * (z - 0.5))))
    tmp <- t(apply(y, 1, cumprod))
    tmp <- cbind(t(apply(tmp, 1, rev)), 1)
    tmp2 <- cbind(1, t(apply(1 - y, 1, rev)))
    f <- tmp * tmp2 * 0.5 * (1 + g)
    return(f)
}

result <- rmoo(fitness = DTLZ1,
                type = "real-valued",
                strategy = "NSGA-III",
                lower = c(0,0,0),
                upper = c(1,1,1),
                monitor = FALSE,
                summary = FALSE,
                popSize = 92,
                n_partitions = 12,
                maxiter = 300)
```

The rmoo package has a set of S4 method that can help the user to
visualize, analysis, and interpret the results.

We are going to show how use the `plot` method, since depending on the
parameter it will display difference plots, e.g. passing “pcp” as
parameter for type, the method displays the Parallel Coordinate Plot:

``` r
plot(result, type="pcp")
```

![](https://github.com/Evolutionary-Optimization-Laboratory/rmoo/blob/master/man/figures/README-example-1.png)<!-- -->

Another example of plot method is when no parameters are passed, the
method by default display the scatter plot. It evaluates the result
dimensionality and depending on this, it will display 2-D, 3-D or
Pairwise Scatter Plot.

``` r
#Scatter without optimal points
plot(result)
```

![](https://github.com/Evolutionary-Optimization-Laboratory/rmoo/blob/master/man/figures/README-example-2.png)<!-- -->

When displaying the scatter plot, reference points can be passed as
parameters of the optimal argument, allowing the results to be compared
to them:

``` r
#Scatter with optimal points (Using reference points as optimal points)
plot(result, optimal = result@reference_points)
```

![](https://github.com/Evolutionary-Optimization-Laboratory/rmoo/blob/master/man/figures/README-example-3.png)<!-- -->

Other plots available in `rmoo` are heatmap and polar coordinate, both
allow the user to view specific solutions.

``` r
#Polar Coordinates
plot(result, type="polar", individual = c(1:3))
```

![](https://github.com/Evolutionary-Optimization-Laboratory/rmoo/blob/master/man/figures/README-example-4.png)<!-- -->

``` r
#Heatmap Plot
plot(result, type="heatmap", individual = c(1:3))
```

![](https://github.com/Evolutionary-Optimization-Laboratory/rmoo/blob/master/man/figures/README-example-5.png)<!-- -->

Other methods available in `rmoo` that may be of interest are
`getFitness()`, `getPopulation()`, `getMetrics()`, `print()`,
`summary()` and others. We do not show the functionality of all these
functions since they will be detailed in depth in the future article and
vignettes.

#' Calculation of Dummy Fitness
#'
#' Calculate of sharing distance and dummy fitness
#'
#' The sharing distance operator guides the selection process at the various
#' stages of the algorithm toward a uniformly spread-out Pareto-optimal front
#'
#' @param object An object of class 'nsga', usually resulting from a call to
#' function nsga. Fitness Function Objective Numbers.
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @references N. Srinivas and K. Deb, 'Multiobjective Optimization Using
#' Nondominated Sorting in Genetic Algorithms,' in Evolutionary Computation,
#' vol. 2, no. 3, pp. 221-248, Sept. 1994, doi: 10.1162/evco.1994.2.3.221.
#'
#' @seealso [non_dominated_fronts()]
#'
#' @return A vector with the dummy fitness.
#' @export
sharing <- function(object) {
    front_count <- 1
    dShare <- object@dShare
    delta_dum <- object@deltaDummy
    nro_front <- length(object@f)
    while (front_count <= nro_front) {
        if (front_count == 1) {
            for (i in object@f[[front_count]]) {
                ind <- object@f[[front_count]]
                object@dumFitness[i, ] <- max(object@fitness)
                nichecount <- 1
                for (j in object@f[[front_count]]) {
                  if (i != j & i < j) {
                    d <- rbind(object@fitness[i, ], object@fitness[j, ])
                    distance <- dist(d, method = "euclidean")
                    if (distance <= 0.5) {
                      nichecount <- nichecount + 1
                    } else if (distance < dShare) {
                      nichecount <- nichecount + (1 - (distance/dShare)) * (1 - (distance/dShare))
                    }
                  }
                }
                object@dumFitness[i, ] <- object@dumFitness[i, ]/nichecount
            }
            minimum_dum <- min(object@dumFitness[ind, ])
        } else {
            for (i in object@f[[front_count]]) {
                ind <- object@f[[front_count]]
                object@dumFitness[i, ] <- minimum_dum - delta_dum
                for (j in object@f[[front_count]]) {
                  if (i != j) {
                    d <- rbind(object@fitness[i, ], object@fitness[j, ])
                    distance <- dist(d, method = "euclidean")
                    if (distance <= 0.5) {
                      nichecount <- nichecount + 1
                    } else if (distance < dShare) {
                      nichecount <- nichecount + (1 - (distance/dShare)) * (1 - (distance/dShare))
                    }
                  }
                }
                object@dumFitness[i, ] <- object@dumFitness[i, ]/nichecount
            }
            minimum_dum <- min(object@dumFitness[ind, ])
        }
        front_count <- front_count + 1
    }
    return(object@dumFitness)
}

#' Non-Dominated Sorting in Genetic Algorithms
#'
#' Minimization of a fitness function using Non-Dominated Genetic algorithms
#' (NSGA). Local search using general-purpose optimisation algorithms can be
#' applied stochastically to exploit interesting regions.
#'
#' The Non-dominated genetic algorithms is a meta-heuristic proposed by
#' N. Srinivas and K. Deb in 1994. The purpose of the algorithms is to find an
#' efficient way to optimize multi-objectives functions (two or more).
#'
#' @param type the type of genetic algorithm to be run depending on the nature
#' of decision variables. Possible values are:
#' \describe{
#' 	\item{\code{"binary"}}{for binary representations of decision variables.}
#'	\item{\code{"real-valued"}}{for optimization problems where the decision
#'	variables are floating-point representations of real numbers.}
#' 	\item{\code{"permutation"}}{for problems that involves reordering of a list
#' 	of objects.}
#' }
#'
#' @param fitness the fitness function, any allowable R function which takes as
#' input an individual string representing a potential solution, and returns a
#' numerical value describing its “fitness”.
#' @param ... additional arguments to be passed to the fitness function. This
#' allows to write fitness functions that keep some variables fixed during the
#' search.
#' @param lower a vector of length equal to the decision variables providing the
#' lower bounds of the search space in case of real-valued or permutation
#' encoded optimizations.
#' @param upper a vector of length equal to the decision variables providing the
#' upper bounds of the search space in case of real-valued or permutation
#' encoded optimizations.
#' @param nBits a value specifying the number of bits to be used in binary
#' encoded optimizations.
#' @param population an R function for randomly generating an initial population.
#' See [rmoo_Population()] for available functions.
#' @param selection an R function performing selection, i.e. a function which
#' generates a new population of individuals from the current population
#' probabilistically according to individual fitness.
#' @param crossover an R function performing crossover, i.e. a function which
#' forms offsprings by combining part of the genetic information from
#' their parents. See [rmoo_Crossover()] for available functions.
#' @param mutation an R function performing mutation, i.e. a function which
#' randomly alters the values of some genes in a parent chromosome.
#' See [rmoo_Mutation()] for available functions.
#' @param popSize the population size.
#' @param nObj number of objective in the fitness function.
#' @param dshare the maximun phenotypic distance allowed between any two
#' individuals to become members of a niche.
#' @param pcrossover the probability of crossover between pairs of chromosomes.
#' Typically this is a large value and by default is set to 0.8.
#' @param pmutation the probability of mutation in a parent chromosome. Usually
#' mutation occurs with a small probability, and by default is set to 0.1.
#' @param maxiter the maximum number of iterations to run before the NSGA search
#' is halted.
#' @param run the number of consecutive generations without any improvement in
#' the best fitness value before the NSGA is stopped.
#' @param maxFitness the upper bound on the fitness function after that the NSGA
#' search is interrupted.
#' @param names a vector of character strings providing the names of decision
#' variables.
#' @param suggestions a matrix of solutions strings to be included in the
#' initial population. If provided the number of columns must match the number
#' of decision variables.
#' @param monitor a logical or an R function which takes as input the current
#' state of the nsga-class object and show the evolution of the search. By
#' default, for interactive sessions the function rmooMonitor prints the average
#' and best fitness values at each iteration. If set to plot these information
#' are plotted on a graphical device. Other functions can be written by the user
#' and supplied as argument. In non interactive sessions, by default
#' monitor = FALSE so any output is suppressed.
#' @param summary If there will be a summary generation after generation.
#' @param seed an integer value containing the random number generator state.
#' This argument can be used to replicate the results of a NSGA search. Note
#' that if parallel computing is required, the doRNG package must be installed.
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @references N. Srinivas and K. Deb, "Multiobjective Optimization Using
#' Nondominated Sorting in Genetic Algorithms, in Evolutionary Computation,
#' vol. 2, no. 3, pp. 221-248, Sept. 1994, doi: 10.1162/evco.1994.2.3.221.
#'
#' Scrucca, L. (2017) On some extensions to 'GA' package: hybrid optimisation,
#' parallelisation and islands evolution. The R Journal, 9/1, 187-206.
#' doi: 10.32614/RJ-2017-008
#'
#' @seealso [nsga2()], [nsga3()]
#'
#' @return Returns an object of class nsga1-class. See [nsga1-class] for a
#' description of available slots information.
#'
#' @examples
#' #Example
#' #Two Objectives - Real Valued
#' zdt1 <- function (x) {
#'  if (is.null(dim(x))) {
#'    x <- matrix(x, nrow = 1)
#'  }
#'  n <- ncol(x)
#'  g <- 1 + rowSums(x[, 2:n, drop = FALSE]) * 9/(n - 1)
#'  return(cbind(x[, 1], g * (1 - sqrt(x[, 1]/g))))
#' }
#'
#' #Not run:
#' \dontrun{
#' result <- nsga(type = "real-valued",
#'                fitness = zdt1,
#'                lower = c(0,0),
#'                upper = c(1,1),
#'                popSize = 100,
#'                nObj = 2,
#'                dshare = 1,
#'                monitor = FALSE,
#'                maxiter = 500)
#' }
#'
#' @export
nsga <- function (type = c("binary", "real-valued", "permutation"),
    fitness, ...,
    lower, upper, nBits,
    population = rmooControl(type)$population,
    selection = rmooControl(type)$selection,
    crossover = rmooControl(type)$crossover,
    mutation = rmooControl(type)$mutation,
    popSize = 50,
    nObj = NULL,
    dshare,
    pcrossover = 0.8,
    pmutation = 0.1,
    maxiter = 100,
    run = maxiter,
    maxFitness = Inf,
    names = NULL,
    suggestions = NULL,
    monitor = if (interactive()) rmooMonitor else FALSE,
    summary = FALSE,
    seed = NULL)
{
    call <- match.call()

    type <- match.arg(type, choices = eval(formals(nsga)$type))

    callArgs <- list(...)

    callArgs$strategy <- NULL

    if (!is.function(population))
      population <- get(population)
    if (!is.function(selection))
      selection <- get(selection)
    if (!is.function(crossover))
      crossover <- get(crossover)
    if (!is.function(mutation))
      mutation <- get(mutation)

    if (is.null(nObj)) {
      stop("Please, define the objective number (nObj)")
    } else {
      if (!is.numeric(nObj) | (nObj%%1!=0)) {
        stop("Objective number (nObj) is a character or is not an integer.")
      }
    }

    if (missing(fitness)) {
      stop("A fitness function must be provided")
    }
    if (!is.function(fitness)) {
      stop("A fitness function must be provided")
    }
    if (popSize < 10) {
      warning("The population size is less than 10.")
    }
    if (maxiter < 1) {
      stop("The maximum number of iterations must be at least 1.")
    }
    if (pcrossover < 0 | pcrossover > 1) {
      stop("Probability of crossover must be between 0 and 1.")
    }
    if (is.numeric(pmutation)) {
      if (pmutation < 0 | pmutation > 1) {
        stop("If numeric probability of mutation must be between 0 and 1.")
      }
      else if (!is.function(population)) {
        stop("pmutation must be a numeric value in (0,1) or a function.")
      }
    }

    if (missing(lower) & missing(upper) & missing(nBits)) {
      stop("A lower and upper range of values (for 'real-valued' or 'permutation') or nBits (for 'binary') must be provided!")
    }

    # if (is.null(nObj)) {
    #   nObj <- ncol(fitness(matrix(10000, ncol = 100, nrow = 100)))
    # }

    dum_Fitness <- matrix(NA, nrow = popSize, ncol = nObj);
    initialDummy <- popSize
    delta_dum <- 0.1 * initialDummy

    switch(type,
      binary = {
        nBits <- as.vector(nBits)[1]
        lower <- upper <- NA
        nvars <- nBits
        if (is.null(names)) names <- paste0("x", 1:nvars)
      },
      `real-valued` = {
        lnames <- names(lower)
        unames <- names(upper)
        lower <- as.vector(lower)
        upper <- as.vector(upper)
        nBits <- NA
        if (length(lower) != length(upper))
          stop("lower and upper must be vector of the same length")
        #if ((length(lower) != nObj) & (length(upper) != nObj))
        #  stop("The lower and upper limits must be vector of the same number of objectives")
        nvars <- length(upper)
        if (is.null(names) & !is.null(lnames))
          names <- lnames
        if (is.null(names) & !is.null(unames))
          names <- unames
        if (is.null(names))
          names <- paste0("x", 1:nvars)
      },
      permutation = {
        lower <- as.vector(lower)[1]
        upper <- as.vector(upper)[1]
        nBits <- NA
        nvars <- length(seq.int(lower, upper))
        if (is.null(names))
          names <- paste0("x", 1:nvars)
      }
    )

    if (is.null(suggestions)) {
      suggestions <- matrix(nrow = 0, ncol = nvars)
    } else {
      if (is.vector(suggestions)) {
        if (nvars > 1)
          suggestions <- matrix(suggestions, nrow = 1)
        else suggestions <- matrix(suggestions, ncol = 1)
      }
      else {
        suggestions <- as.matrix(suggestions)
      }
      if (nvars != ncol(suggestions))
        stop("Provided suggestions (ncol) matrix do not match number of variables of the problem!")
    }

    # check monitor arg
    if (is.logical(monitor)) {
      if (monitor)
        monitor <- rmooMonitor
    }
    if (is.null(monitor)) monitor <- FALSE

    # set seed for reproducibility
    if (!is.null(seed))
      set.seed(seed)

    i. <- NULL #dummy to trick R CMD check

    Fitness <- matrix(NA, nrow = popSize, ncol = nObj)

    fitnessSummary <- vector("list", maxiter)

    #Creacion del objetivo tipo nsga
    object <- new("nsga1",
        call = call,
        type = type,
        lower = lower,
        upper = upper,
        nBits = nBits,
        names = if (is.null(names))
          character()
        else names,
        popSize = popSize,
        front = matrix(),
        f = list(),
        iter = 0,
        run = 1,
        maxiter = maxiter,
        suggestions = suggestions,
        population = matrix(),
        pcrossover = pcrossover,
        pmutation = if (is.numeric(pmutation))
          pmutation
        else NA,
        dumFitness = dum_Fitness,
        dShare = dshare,
        deltaDummy = delta_dum,
        fitness = Fitness,
        summary = fitnessSummary)

    #Generate initial population
    if (maxiter == 0)
      return(object)

    switch(type,
      binary = {
        Pop <- matrix(NA_real_, nrow = popSize, ncol = nBits)
      },
      `real-valued` = {
        Pop <- matrix(NA_real_, nrow = popSize, ncol = nvars)
      },
      permutation = {
        Pop <- matrix(NA_real_, nrow = popSize, ncol = nvars)
      }
    )

    ng <- min(nrow(suggestions), popSize)

    if (ng > 0) {
      Pop[1:ng, ] <- suggestions
    }
    if (popSize > ng) {
      Pop[(ng + 1):popSize, ] <- population(object)[1:(popSize - ng), ]
    }
    object@population <- Pop

    for (i in seq_len(popSize)) {
      if (is.na(Fitness[i])) {
        fit <- do.call(fitness, c(list(Pop[i, ]), callArgs))
        Fitness[i, ] <- fit
      }
    }

    object@population <- Pop
    object@fitness <- Fitness

    #First Non-dominated Ranking
    out <- non_dominated_fronts(object)
    object@f <- out$fit
    object@front <- matrix(unlist(out$fronts), ncol = 1, byrow = TRUE)
    object@dumFitness <- sharing(object)

    for (iter in seq_len(maxiter)) {
      object@iter <- iter

      #Selection Operator
      if (is.function(selection)) {
        sel <- selection(object, nObj)
        Pop <- sel$population
        Fitness <- sel$fitness
      } else {
        sel <- sample(1:popSize, size = popSize, replace = TRUE)
        Pop <- object@population[sel, ]
        Fitness <- object@fitness[sel, ]
      }
      object@population <- Pop
      object@fitness <- Fitness

      #Cross Operator
      if (is.function(crossover) & pcrossover > 0) {
        nmating <- floor(popSize / 2)
        mating <- matrix(sample(1:(2 * nmating), size = (2 * nmating)), ncol = 2)
        for (i in seq_len(nmating)) {
          if (pcrossover > runif(1)) {
            parents <- mating[i, ]
            Crossover <- crossover(object, parents)
            Pop[parents, ] <- Crossover$children
            Fitness[parents, ] <- Crossover$fitness
          }
        }
      }
      object@population <- Pop
      object@fitness <- Fitness

      #Mutation Operator
      pm <- if (is.function(pmutation)) {
        pmutation(object)
      } else {pmutation}
      if (is.function(mutation) & pm > 0) {
        for (i in seq_len(popSize)) {
          if (pm > runif(1)) {
            Mutation <- mutation(object, i)
            Pop[i, ] <- Mutation
            Fitness[i,] <- NA
          }
        }
      }
      object@population <- Pop
      object@fitness <- Fitness

      #Evaluate Fitness
      for (i in seq_len(popSize)) {
        if (is.na(Fitness[i])) {
          fit <- do.call(fitness, c(list(Pop[i, ]), callArgs))
          Fitness[i,] <- fit
        }
      }

      object@population <- Pop
      object@fitness <- Fitness

      out <- non_dominated_fronts(object)
      object@f <- out$fit
      object@front <- matrix(unlist(out$fronts), ncol = 1, byrow = TRUE)
      object@dumFitness <- sharing(object)
      rm(out)

      if (summary == TRUE) {
        fitnessSummary[[iter]] <- progress(object, callArgs)
        object@summary <- fitnessSummary
      } else {
        object@summary <- list(NULL)
      }

      #Plot front non-dominated by iteration
      if (is.function(monitor)) {
        monitor(object = object, callArgs)
        # monitor(object = object, number_objective = nObj)
      }

      if (max(Fitness, na.rm = TRUE) >= maxFitness)
        break
      if (object@iter == maxiter)
        break
    }

    solution <- object

    return(solution)
}

# @export
#' @rdname plot-methods
#' @aliases plot,nsga1-method
setMethod("plot", signature(x="nsga1", y="missing"), .get.plotting)

# @export
#' @rdname progress-methods
#' @aliases progress,nsga1-method
setMethod("progress", "nsga1", .nsga1.progress)

# @export
#' @rdname getDummyFitness-methods
#' @aliases getDummyFitness,nsga1-method
setMethod("getDummyFitness", "nsga1",
          function(obj) {
            cat("NSGA-I Dummy Fitness: \n")
            cat("\n#========================================#\n")
            print(obj@dumFitness)
            n_dum <- ncol(obj@dumFitness)
            dum_Fitness <- data.frame(obj@dumFitness)
            colnames(dum_Fitness) <- sprintf("FitDummy_%s",seq(n_dum))
            return(invisible(dum_Fitness))
          }
)

# @export
#' @rdname print-methods
#' @aliases print,nsga1-method
setMethod("print", "nsga1",
          function(x, ...) {
            algorithm <- class(x)[1]
            # Print
            cat("\nSlots Configuration:\n")
            print((slotNames(x)))
            cat("\n#========================================#\n")
            cat("\nTotal iterations: ", x@iter)
            cat("\nRepresentation Type: ", x@type)
            cat("\nPopulation size: ", x@popSize)
            if (x@type == "binary") {
              cat("\nNumber of Bits: ", x@nBits)
            } else{
              cat("\nLower Bounds: ", x@lower)
              cat("\nLower Bounds: ", x@upper)
            }
            cat("\nDelta Distance (dShare):  ", x@dShare)
            cat("\nDistance of sharing function: ", x@deltaDummy)
            cat("\nNumber of Nondominated Front: ", length(x@f[[1]]))
            cat("\n#========================================#\n")

          }
)


# @export
#' @rdname summary-methods
#' @aliases summary,nsga1-method
setMethod("summary", "nsga1",
          function(object, ...){
            callArgs <- list(...)
            nullRP <- is.null(callArgs$reference_dirs)

            # Calculate information for summary

            first <- object@f[[1]]
            first_front_fit <-
              first_front_pop <- object@population[first, ]
            nadir_point <- apply(object@fitness[first, ], 2, max)

            #first_dum <- object@dumFitness[first, ] for nsga1 summary method

            if("ecr" %in% rownames(utils::installed.packages())){
              if (nullRP) {
                cat("Warning! \nReference points not provided:\n
                      value necessary to evaluate GD and IGD.")

              } else{
                gd <- ecr::computeGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
                igd <- ecr::computeInvertedGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
              }
            }

            if("emoa" %in% rownames(utils::installed.packages())){
              if(nullRP) {
                cat("\nUsing the maximum in each dimension to evaluate Hypervolumen")
                reference_point <- nadir_point
              } else {reference_point <- apply(callArgs$reference_dirs, 2, max)}
              hv <- emoa::dominated_hypervolume(points = t(object@fitness[first, ]), ref = reference_point)
            }

            cat("\nSummary of NSGA-I run")
            cat("\n#====================================")
            cat("\nNumber of Objectives evaluated: ", ncol(object@fitness))
            cat("\nTotal iterations: ", object@iter)
            cat("\nPopulation size: ", object@popSize)
            #cat("\nFeasible points found: ", nfeas,paste0("(", signif(100 * nfeas / npts, 3), "%"),"of total)")
            cat("\nNondominated points found: ", length(first),
                paste0("(", signif(100 * length(first) / object@popSize, 3), "%"),
                "of total)")
            cat("\nShare Distance: ", object@dShare)
            cat("\nSharing Values calculated: ", object@deltaDummy)
            #cat("\nEstimated nadir point: ", round(object@nadir_point, 3))
            cat("\nMutation Probability: ",
                paste0(signif(100 * object@pmutation, 3), "%"))
            cat("\nCrossover Probability: ",
                paste0(signif(100 * object@pcrossover, 3), "%"))
            if("ecr" %in% rownames(utils::installed.packages())){
              if(!nullRP) cat("\nEstimated IGD: ", igd)
              if(!nullRP) cat("\nEstimated GD: ", gd)
            } else cat("\n\nPlease install package 'ecr' to calculate IGD and GD.")
            if("emoa" %in% rownames(utils::installed.packages())) {
              cat("\nEstimated HV: ", hv)
              cat("\nRef point used for HV: ", reference_point)
            } else cat("\n\nPlease install package 'emoa' to calculate hypervolume.")
            cat("\n#====================================")
          }
)

context("Executing nsga")

test_that("nsga() return the value suggestions", {
  testfunction <- function (x){
    return(x)
  }

  x <- matrix(rep(1,30), ncol = 3)


  out <- nsga(type = "real-valued", fitness = testfunction,
              lower = c(0,0,0), upper = c(1,1,1),
              selection = rmoo::rmoo_lrSelection,
              popSize = 10, suggestions = x,
              pcrossover = 0, pmutation = 0,
              seed = 1, dshare = 1, monitor = FALSE,
              summary = FALSE,
              maxiter = 1, nObj = 3)

  expect_identical(out@fitness, x)
})

#' @keywords internal
#' @aliases rmoo-package
"_PACKAGE"

#' @import stats utils graphics methods foreach
#' @importFrom GA ga
#' @importFrom grDevices rainbow
#' @importFrom ggplot2 aes
#' @importFrom ggplot2 geom_line
#' @importFrom ggplot2 label_value
#' @importFrom ggplot2 element_text
#' @importFrom plotly %>%
#' @importFrom plotly add_markers
#' @importFrom BBmisc which.first
NULL

generational_distance <- function(fitness, reference_points) {
    popSize <- nrow(fitness)
    distances <- rep(Inf, popSize)
    aux <- c()
    for (i in 1:popSize) {
        for (j in 1:popSize) {
            aux <- dist(rbind(fitness[i, ], reference_points[j, ]),
                        method = "euclidean")
            if (aux < distances[i]) {
                distances[i] <- aux
            }
        }
        aux <- c()
    }
    distances <- sum(distances) / nrow(reference_points)

    return(distances)
}



# front and true_pareto_front can be:
#   N×m matrix where N is the number of points and m is the number of objectives.
#   State
#   Array{xFgh_indiv} (usually `State.population`)
# true_pareto_front: is a M×m matrix.
# p_ is the power in for the ‖⋅‖_p
# plus: if true then computes the GD+
# inverted: if true then computes IGD

#' @export
generational_distance <- function(front, true_pareto_front, p = 1, inverted = FALSE, plus = FALSE) {
  if (inverted) {
    tmp <- true_pareto_front
    true_pareto_front <- front
    front <- tmp
  }

  distances <- rep(0, nrow(true_pareto_front))

  s <- 0.0
  for (i in 1:nrow(front)) {
    x <- front[i,]
    for (j in 1:nrow(true_pareto_front)) {
      y <- true_pareto_front[j,]
      # IGD Plus
      if (plus && inverted) {
        distances[j] <- norm(pmax(y - x, 0), type = "2")
        # GD Plus
      } else if (plus && !inverted) {
        distances[j] <- norm(pmax(x - y, 0), type = "2")
        # GD and IGD
      } else {
        distances[j] <- norm(y - x, type = "2")
      }
    }

    s <- s + min(distances)^p
  }

  return ((s^(1/p)) / nrow(front))
}

gd_plus <- function(front, true_pareto_front, p = 1) {
  generational_distance(front, true_pareto_front, p = p, plus = TRUE)
}
# generational_distance_cpp <- cppFunction("
#   NumericVector generational_distance_cpp(NumericMatrix front, NumericMatrix true_pareto_front,
#                                           double p = 1, bool inverted = false, bool plus = false) {
#     int n_front = front.nrow();
#     int n_true_pareto = true_pareto_front.nrow();
#     NumericVector distances(n_true_pareto);
#     double s = 0.0;
#
#     for (int i = 0; i < n_front; ++i) {
#       NumericVector x = front(i, _);
#       for (int j = 0; j < n_true_pareto; ++j) {
#         NumericVector y = true_pareto_front(j, _);
#         if (plus && inverted) {
#           for (int k = 0; k < x.length(); ++k) {
#             distances[j] += pow(std::max(y[k] - x[k], 0.0), 2);
#           }
#         } else if (plus && !inverted) {
#           for (int k = 0; k < x.length(); ++k) {
#             distances[j] += pow(std::max(x[k] - y[k], 0.0), 2);
#           }
#         } else {
#           for (int k = 0; k < x.length(); ++k) {
#             distances[j] += pow(y[k] - x[k], 2);
#           }
#         }
#       }
#       s += pow(*std::min_element(distances.begin(), distances.end()), p);
#     }
#
#     return pow(s, 1/p) / n_front;
#   }
# ")
#
# gd_plus_cpp <- function(front, true_pareto_front, p = 1) {
#   generational_distance_cpp(front, true_pareto_front, p = p, plus = TRUE)
# }



# #include <Rcpp.h>
# using namespace Rcpp;
#
# // [[Rcpp::export]]
# double generational_distance(NumericMatrix front, NumericMatrix true_pareto_front,
#                              double p = 1, bool inverted = false, bool plus = false) {
#   if (inverted) {
#     NumericMatrix tmp = true_pareto_front;
#     true_pareto_front = front;
#     front = tmp;
#   }
#
#   int nrow_true_pareto_front = true_pareto_front.nrow();
#   NumericVector distances(nrow_true_pareto_front);
#
#   double s = 0.0;
#   for (int i = 0; i < front.nrow(); i++) {
#     NumericVector x = front(i, _);
#     for (int j = 0; j < nrow_true_pareto_front; j++) {
#       NumericVector y = true_pareto_front(j, _);
#       // IGD Plus
#       if (plus && inverted) {
#         distances[j] = sqrt(sum(pow(pmax(y - x, 0), 2)));
#         // GD Plus
#       } else if (plus && !inverted) {
#         distances[j] = sqrt(sum(pow(pmax(x - y, 0), 2)));
#         // GD and IGD
#       } else {
#         distances[j] = sqrt(sum(pow(y - x, 2)));
#       }
#     }
#
#     s += pow(min(distances), p);
#   }
#
#   return pow(s, 1 / p) / front.nrow();
# }
#
# // [[Rcpp::export]]
# double gd_plus(NumericMatrix front, NumericMatrix true_pareto_front, double p = 1) {
#   return generational_distance(front, true_pareto_front, p, false, true);
# }
#
# library(Rcpp)
# sourceCpp("path/to/cpp/code.cpp")

context("Executing performance metrics")

test_that("generational_distance() return the value suggestions", {
  x <- generate_reference_points(3,3)

  out <- generational_distance(x,x)

  expect_identical(out, 0)
})

.onLoad <- function(lib, pkg) {
    op <- options()
    op.rmoo <- list(rmoo.path = "~/R-dev",
                  rmoo.install.args = "",
                  rmoo.name = "Francisco Jose Benitez Rios",
                  rmoo.desc.author = "person(\"Francisco Jose\", \"Benitez Rios\",
                                             \"benitezfj94@gmail.com\",
                                             role = c(\"aut\", \"cre\"))",
                  rmoo.desc.license = "Licence",
                  rmoo.desc.suggests = NULL, rmoo.desc = list())
    toset <- !(names(op.rmoo) %in% names(op))
    if (any(toset))
        options(op.rmoo[toset])
    invisible()
}

rmooStartupMessage <- function() {
    msg <- paste0("
       ____  __    __  _____  _____
      |  _ | | \\  /  ||  _  ||  _  | R
      |  __| | |\\/ / || | | || | | | Multi-/Many-
      | |\\\\  | | \\/| || |_| || |_| | Objective
      |_| \\\\ |_|   |_||_____||_____| Optimization Package ",
        packageVersion("rmoo"))
    return(msg)
}

.onAttach <- function(lib, pkg) {
    # unlock .rmoo.default variable allowing its modification
    unlockBinding(".rmoo.default", asNamespace("rmoo"))
    # startup message
    msg <- rmooStartupMessage()
    if (!interactive())
        msg[1] <- paste("Package 'rmoo' version", packageVersion("rmoo"))
    packageStartupMessage(msg)
    invisible()
}

options(rmoo.description = list(Title = "Multi-Objective Optimization in R",
          `Authors@R` = "c(person(\"Francisco\", \"Benitez\",
                                email = \"benitezfj94@gmail.com\",
                                role = c(\"aut\", \"cre\")),
                             person(\"Diego P.\", \"Pinto-Roa\",
                                email = \"dpinto@pol.una.py\",
                                role = c(\"aut\"),
                                comment = c(ORCID = \"0000-0003-2479-9876\")))",
        Description = "The 'rmoo' package is a framework for omulti- and many-
        objective optimization, which allows researchers and users versatility
        in parameter configuration, as well as tools for analysis, replication
        and visualization of results. The 'rmoo' package was built as a fork of
        the 'GA' package by Luca Scrucca(2017) <DOI:10.32614/RJ-2017-008> and
        implementing the Non-Dominated Sorting Genetic Algorithms proposed
        by K. Deb's.",
        License = "GPL (>= 2)",
        Language = "es"))

#' Start Parallel Backend for rmoo Package
#'
#' This function sets up parallel computing using the `parallel` and `doParallel` packages.
#' It supports both "snow" (PSOCK) and "multicore" backends depending on the OS.
#'
#' @param parallel Logical, numeric, character, or a cluster object. If `TRUE`, uses all detected cores.
#'   If numeric, it specifies the number of cores. If character, should be `"snow"` or `"multicore"`.
#'   If a cluster object, it will use that cluster.
#' @param ... Additional arguments (currently unused).
#'
#' @return An object of class `logical` with attributes:
#'   - `type`: cluster type ("snow" or "multicore")
#'   - `cores`: number of cores used
#'   - `cluster`: the cluster object created or passed
#'
#' @examples
#' \dontrun{
#'   cl <- startParallel(TRUE)
#'   stopParallel(attr(cl, "cluster"))
#' }
#'
#' @export
startParallel <- function(parallel = TRUE, ...)
{
  # Start parallel computing for rmoo package

  # check availability of parallel and doParallel (their dependencies, i.e.
  # foreach and iterators, are specified as Depends on package DESCRIPTION file)
  if(!all(requireNamespace("parallel", quietly = TRUE),
          requireNamespace("doParallel", quietly = TRUE)))
    stop("packages 'parallel' and 'doParallel' required for parallelization!")

  # if a cluster is provided as input argument use that cluster and exit
  if(any(class(parallel) == "cluster")) {
    cl <- parallel
    parallel <- TRUE
    attr(parallel, "type") <- foreach::getDoParName()
    attr(parallel, "cores") <- foreach::getDoParWorkers()
    attr(parallel, "cluster") <- cl
    return(parallel)
  }

  # set default parallel functionality depending on system OS:
  # - snow functionality on Windows OS
  # - multicore functionality on Unix-like systems (Unix/Linux & Mac OSX)
  parallelType <- if (.Platform$OS.type == "windows") "snow" else "multicore"

  # get the current number of cores available
  numCores <- parallel::detectCores()

  # set parameters for parallelization
  if (is.numeric(parallel)) {
    numCores <- as.integer(parallel)
    parallel <- TRUE
  } else if (is.character(parallel)) {
    parallelType <- parallel
    parallel <- TRUE
  } else if (!is.logical(parallel)) {
    parallel <- FALSE
  }

  attr(parallel, "type") <- parallelType
  attr(parallel, "cores") <- numCores

  # start "parallel backend" if needed
  if(parallel) {
    if (parallelType == "snow") {
      # snow functionality on Unix-like systems & Windows
      cl <- parallel::makeCluster(numCores, type = "PSOCK")
      attr(parallel, "cluster") <- cl
      # export parent environment

      # varlist <- ls(envir = parent.frame(), all.names = TRUE)
      # varlist <- varlist[varlist != "..."]
      # parallel::clusterExport(cl, varlist = varlist, envir = parent.frame()) # envir = parent.env(environment())

      parallel::clusterExport(cl, varlist = setdiff(ls(parent.frame(), all.names = TRUE), "..."), envir = parent.frame())
      # export global environment (workspace)
      parallel::clusterExport(cl, varlist = ls(envir = globalenv(), all.names = TRUE), envir = globalenv())
      # load current packages in workers
      # pkgs <- .packages()
      lapply(.packages(), function(pkg)
        parallel::clusterCall(cl, library, package = pkg, character.only = TRUE))
      #
      doParallel::registerDoParallel(cl, cores = numCores)
    }
    else if (parallelType == "multicore") {
      # multicore functionality on Unix-like systems
      cl <- parallel::makeCluster(numCores, type = "FORK")
      doParallel::registerDoParallel(cl, cores = numCores)
      attr(parallel, "cluster") <- cl
    }
    else
    { stop("Only 'snow' and 'multicore' clusters allowed!") }
  }

  return(parallel)
}


#' Stop Parallel Backend
#'
#' Stops the parallel backend and reverts to sequential execution.
#'
#' @param cluster A cluster object, typically retrieved from `attr(parallel, "cluster")`
#' @param ... Additional arguments (currently unused).
#'
#' @return `invisible(NULL)`, used for side effects.
#'
#' @examples
#' \dontrun{
#'   cl <- startParallel()
#'   stopParallel(attr(cl, "cluster"))
#' }
#'
#' @export
stopParallel <- function(cluster, ...) {
  if (!inherits(cluster, "cluster")) {
    warning("No valid cluster provided to stopParallel().")
    return(invisible(NULL))
  }
  # Stop parallel computing
  parallel::stopCluster(cluster)
  foreach::registerDoSEQ()
  invisible(NULL)
}

#' @export
rmooControl <- function(...) {
    current <- .rmoo.default
    if (nargs() == 0)
        return(current)
    args <- list(...)
    if (length(args) == 1 && is.null(names(args))) {
        arg <- args[[1]]
        switch(mode(arg),
          list = args <- arg,
          character = return(.rmoo.default[[arg]]),
          stop("invalid argument: ", dQuote(arg))
        )
    }

    if (length(args) == 0)
        return(current)
    nargs <- names(args)
    if (is.null(nargs))
        stop("options must be given by name")

    if (is.list(args)) {
        changed <- current[nargs]
        for (i in 1:length(nargs)) {
            if (is.list(args[[i]])) {
                what <- names(args[[i]])
                changed[[i]][what] <- args[[i]][what]
            } else {
                changed[i] <- args[[i]]
            }
        }
        current[nargs] <- changed
    } else {
        changed <- current[nargs]
        current[nargs] <- args
    }

    if (sys.parent() == 0)
        env <- asNamespace("rmoo") else env <- parent.frame()
    assign(".rmoo.default", current, envir = env)
    invisible(current)
}


.rmoo.default <- list(binary = list(population = "rmoobin_Population",
                                    selection = "rmoobin_tourSelection",
                                    crossover = "rmoobin_spCrossover",
                                    mutation = "rmoobin_raMutation"),
                     `real-valued` = list(population = "rmooreal_Population",
                                          selection = "rmooreal_tourSelection",
                                          crossover = "rmooreal_sbxCrossover",
                                          mutation = "rmooreal_polMutation"),
                     permutation = list(population = "rmooperm_Population",
                                        selection = "rmooperm_tourSelection",
                                        crossover = "rmooperm_oxCrossover",
                                        mutation = "rmooperm_simMutation"),
                     discrete = list(population = "rmooint_Population",
                                        selection = "rmooint_tourSelection",
                                        crossover = "rmooint_uxCrossover",
                                        mutation = "rmooint_uxMutation"),
  eps = sqrt(.Machine$double.eps),
  useRcpp = FALSE)

#' Calculate of Non-Dominated Front
#'
#' A fast approach for calculate Non-Dominated Fronts.
#'
#' Function to determine the non-dominated fronts of a population and the
#' aptitude value.
#'
#' @param object An object of class 'nsga', usually resulting from a call to
#' function nsga, nsga2 and nsga3.
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @references K. Deb, A. Pratap, S. Agarwal and T. Meyarivan, 'A fast and
#' elitist multiobjective genetic algorithm: NSGA-II,' in IEEE Transactions on
#' Evolutionary Computation, vol. 6, no. 2, pp. 182-197, April 2002,
#' doi: 10.1109/4235.996017.
#'
#' @seealso [nsga()], [nsga2()] and [nsga3()]
#'
#' @return A list with 'non-dominated fronts' and 'occupied positions' on the fronts.
#' @export
non_dominated_fronts <- function(object) {
  nondom.layers <- ecr::doNondominatedSorting(t(object@fitness))

  max.rank <- max(nondom.layers$ranks)

  idxs.by.rank <- lapply(seq(max.rank), function(r) which(nondom.layers$ranks == r))

  out <- list(fit = idxs.by.rank, fronts = nondom.layers$ranks)

  return(out)
}

# non_dominated_fronts <- function(object) {
#   fitness <- object@fitness
#   popSize <- nrow(fitness)
#   dominated_count <- numeric(popSize)
#   domination_set <- vector("list", popSize)
#   front <- numeric(popSize)
#   front_index <- 1
#
#   for (i in seq_len(popSize - 1)) {
#     for (j in seq(i + 1, popSize)) {
#       if (all(fitness[i, ] <= fitness[j, ]) && any(fitness[i, ] < fitness[j, ])) {
#         domination_set[[i]] <- c(domination_set[[i]], j)
#         dominated_count[j] <- dominated_count[j] + 1
#       } else if (all(fitness[j, ] <= fitness[i, ]) && any(fitness[j, ] < fitness[i, ])) {
#         domination_set[[j]] <- c(domination_set[[j]], i)
#         dominated_count[i] <- dominated_count[i] + 1
#       }
#     }
#   }
#
#   front_index <- 1
#   q <- which(dominated_count == 0)
#   front[q] <- front_index
#
#   while (length(q) > 0) {
#     front_index <- front_index + 1
#     new_q <- integer()
#     for (i in q) {
#       for (j in domination_set[[i]]) {
#         dominated_count[j] <- dominated_count[j] - 1
#         if (dominated_count[j] == 0) {
#           new_q <- c(new_q, j)
#           front[j] <- front_index
#         }
#       }
#     }
#     q <- new_q
#   }
#
#   out <- list(fit = split(seq_len(popSize), front), fronts = front)
#   return(out)
# }





# non_dominated_fronts <- function(object) {
#     pop_count <- 0
#     pop_size <- nrow(object@population)
#     dominated_count <- vector("list", pop_size)
#     domination_set <- vector("list", pop_size)
#     front <- vector("list", pop_size)
#     fitness <- object@fitness
#     q <- c()
#     front_index <- 1
#
#     for (i in seq_len(pop_size)) {
#         dominated_count[[i]] <- 0
#     }
#     for (i in seq_len(pop_size)) {
#         for (j in seq_len(pop_size)) {
#             if (i != j & i < j) {
#                 if (all(fitness[i, ] <= fitness[j, ]) && any(fitness[i, ] < fitness[j, ])) {
#                   domination_set[[i]] <- c(domination_set[[i]], j)
#                   dominated_count[[j]] <- dominated_count[[j]] + 1
#
#                 }
#                 if (all(fitness[j, ] <= fitness[i, ]) && any(fitness[j, ] < fitness[i, ])) {
#                   domination_set[[j]] <- c(domination_set[[j]], i)
#                   dominated_count[[i]] <- dominated_count[[i]] + 1
#                 }
#             }
#         }
#
#         if (dominated_count[[i]] == 0) {
#             q <- c(q, i)
#             front[[i]] <- front_index
#             pop_count <- (pop_count + 1)
#         }
#     }
#     f <- list()
#     f[front_index] <- list(sort(q))
#
#     while (TRUE) {
#         q <- c()
#         for (i in f[[front_index]]) {
#             for (j in domination_set[[i]]) {
#                 dominated_count[[j]] <- dominated_count[[j]] - 1
#                 if (dominated_count[j] == 0) {
#                   q <- c(q, j)
#                   front[[j]] <- front_index + 1
#                   pop_count <- pop_count + 1
#                 }
#             }
#         }
#         if (is.null(q))
#             break
#         front_index <- front_index + 1
#         f[front_index] <- list(sort(q))
#     }
#     out <- list(fit = f, fronts = front)
#     return(out)
# }





# cppFunction('
# List non_dominated_fronts_rcpp(NumericMatrix fitness) {
#   int pop_size = fitness.nrow();
#   NumericVector dominated_count(pop_size, 0.0);
#   List domination_set(pop_size);
#   IntegerVector front(pop_size, 0);
#   int front_index = 1;
#
#   for (int i = 0; i < pop_size - 1; ++i) {
#     for (int j = i + 1; j < pop_size; ++j) {
#       bool i_dominates_j = true;
#       bool j_dominates_i = true;
#
#       for (int k = 0; k < fitness.ncol(); ++k) {
#         if (fitness(i, k) > fitness(j, k)) {
#           i_dominates_j = false;
#         }
#         if (fitness(j, k) > fitness(i, k)) {
#           j_dominates_i = false;
#         }
#       }
#
#       if (i_dominates_j && !j_dominates_i) {
#         IntegerVector dom_set = domination_set[i];
#         dom_set.push_back(j);
#         dominated_count[j]++;
#         domination_set[i] = dom_set;
#       } else if (!i_dominates_j && j_dominates_i) {
#         IntegerVector dom_set = domination_set[j];
#         dom_set.push_back(i);
#         dominated_count[i]++;
#         domination_set[j] = dom_set;
#       }
#     }
#   }
#
#   front_index = 1;
#   IntegerVector q = which(dominated_count == 0);
#   for (int i = 0; i < q.size(); ++i) {
#     front[q[i]] = front_index;
#   }
#
#   while (q.size() > 0) {
#     front_index++;
#     IntegerVector new_q;
#     for (int i = 0; i < q.size(); ++i) {
#       IntegerVector dom_set = domination_set[q[i]];
#       for (int j = 0; j < dom_set.size(); ++j) {
#         dominated_count[dom_set[j]]--;
#         if (dominated_count[dom_set[j]] == 0) {
#           new_q.push_back(dom_set[j]);
#           front[dom_set[j]] = front_index;
#         }
#       }
#     }
#     q = new_q;
#   }
#
#   List out;
#   out["fit"] = split(seq_len(pop_size), front);
#   out["fronts"] = front;
#   return out;
# }')

#' Determination of Reference Points on a Hyper-Plane
#'
#' A implementation of Das and Dennis's Reference Points Generation.
#'
#' The implemented Reference Point Generation is based on the Das and Dennis's
#' systematic approach that places points on a normalized hyper-plane which is
#' equally inclined to all objective axes and has an intercept of one on each axis.
#'
#' @param m,h,scaling Number of reference points 'h' in M-objective problems, and
#' scaling that is the scale on which the points are distributed.
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @references K. Deb and H. Jain, 'An Evolutionary Many-Objective Optimization
#' Algorithm Using Reference-Point-Based Nondominated Sorting Approach, Part I:
#' Solving Problems With Box Constraints,' in IEEE Transactions on Evolutionary
#' Computation, vol. 18, no. 4, pp. 577-601, Aug. 2014,
#' doi: 10.1109/TEVC.2013.2281535.
#'
#' Das, Indraneel & Dennis, J. (2000). Normal-Boundary Intersection: A New
#' Method for Generating the Pareto Surface in Nonlinear Multicriteria
#' Optimization Problems. SIAM Journal on Optimization. 8.
#' 10.1137/S1052623496307510.
#'
#' @seealso [non_dominated_fronts()] and [get_fixed_rowsum_integer_matrix()]
#'
#' @return A matrix with the reference points uniformly distributed.
#' @export
generate_reference_points <- function(m, h, scaling = NULL) {
    zr <- (get_fixed_rowsum_integer_matrix(m, h) / h)
    if (!is.null(scaling)) zr <- scale_reference_directions(zr ,scaling)
    return(zr)
}

#' Scale Reference Points
#'
#' A implementation of Das and Dennis's Reference Points Generation.
#'
#' The implemented Reference Point Generation is based on the Das and Dennis's
#' systematic approach that places points on a normalized hyper-plane which is
#' equally inclined to all objective axes and has an intercept of one on each axis.
#'
#' @param ref_dirs,scaling where 'ref_dirs' are the reference points generated and
#' 'scaling' are the scale on which the points are distributed.
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @references J. Blank and K. Deb, "Pymoo: Multi-Objective Optimization in Python," in
#' IEEE Access, vol. 8, pp. 89497-89509, 2020, doi: 10.1109/ACCESS.2020.2990567.
#'
#' @seealso [generate_reference_points()] and [get_fixed_rowsum_integer_matrix()]
#'
#' @return A matrix with rescaled reference points uniformly distributed.
#' @export
scale_reference_directions <- function(ref_dirs, scaling){
  return(ref_dirs * scaling + ((1 - scaling) / ncol(ref_dirs)))
}

context("Executing nsga3")

test_that("nsga3() return the value suggestions", {
  testfunction <- function (x){
    return(x)
  }

  x <- matrix(runif(30, 0, 1), ncol = 3, nrow = 10)
  ref_points <- generate_reference_points(3,3)

  out <- rmoo(type = "real-valued", algorithm = "NSGA-III", fitness = testfunction,
               lower = c(0,0,0), upper = c(1,1,1),
               popSize = 10, suggestions = x,
               reference_dirs = ref_points,
               pcrossover = 0.1, pmutation = 0.1,
               seed = 1, parallel = FALSE, monitor = FALSE,
               summary = FALSE, maxiter = 1, nObj = 3)

  expect_identical(dim(out@fitness), dim(x))
})

context("test-scatter-.r")

test_that("Scatter Plot for 2-objective works", {
  testfunction <- function (x){
    return(x)
  }

  x <- matrix(runif(20, 0, 1), ncol = 2, nrow = 10)

  out <- rmoo(type = "real-valued", algorithm = "NSGA-II", fitness = testfunction,
              lower = c(0,0), upper = c(1,1),
              popSize = 10, suggestions = x,
              pcrossover = 0, pmutation = 0,
              seed = 1, parallel = FALSE, monitor = FALSE,
              summary = FALSE, maxiter = 1, nObj = 2)

  expect_message(plot(x = out), NA)
})

test_that("Scatter Plot for 3-objective works", {
  testfunction <- function (x){
    return(x)
  }

  x <- matrix(runif(30, 0, 1), ncol = 3, nrow = 10)
  ref_points <- generate_reference_points(3,3)

  out <- rmoo(type = "real-valued", algorithm = "NSGA-III", fitness = testfunction,
	      lower = c(0,0,0), upper = c(1,1,1),
	      popSize = 10, suggestions = x,
	      reference_dirs = ref_points,
	      pcrossover = 0.1, pmutation = 0.1,
	      seed = 1, parallel = FALSE, monitor = FALSE,
	      summary = FALSE, maxiter = 1, nObj = 3)

  expect_message(plot(x = out), NA)
  expect_message(plot(x = out,
                 optimal = out@reference_points), NA)
})

test_that("Scatter Plot for n-objective works", {
  testfunction <- function (x){
    return(x)
  }

  x <- matrix(runif(40, 0, 1), ncol = 4, nrow = 10)
  ref_points <- generate_reference_points(4,2)

  out <- rmoo(type = "real-valued", algorithm = "NSGA-III", fitness = testfunction,
    lower = rep(0,4), upper = rep(1,4),
    popSize = 10, suggestions = x,
    reference_dirs = ref_points,
    pcrossover = 0.1, pmutation = 0.1,
    seed = 1, parallel = FALSE, monitor = FALSE,
    summary = FALSE, maxiter = 1, nObj = 4)

  expect_message(plot(x = out), NA)
})

context("test-pcp-.r")

test_that("Parallel Coordinate Plot for n-objective works", {
  testfunction <- function (x){
    return(x)
  }

  x <- matrix(runif(40, 0, 1), ncol = 4, nrow = 10)
  ref_points <- generate_reference_points(4,2)

  out <- rmoo(type = "real-valued", algorithm = "NSGA-III", fitness = testfunction,
	      lower = rep(0,4), upper = rep(1,4),
	      popSize = 10, suggestions = x,
	      reference_dirs = ref_points,
	      pcrossover = 0.1, pmutation = 0.1,
	      seed = 1, parallel = FALSE, monitor = FALSE,
	      summary = FALSE, maxiter = 1, nObj = 4)

  expect_message(plot(x = out, type = "pcp"), NA)
  expect_message(plot(x = out, type = "heatmap", individual = c(1:5)), NA)
  expect_message(plot(x = out, type = "polar", individual = c(1:5)), NA)
})

#' R Multi-Objective Optimization Main Function
#'
#' Main function of rmoo, based on the parameters it will call the different
#' algorithms implemented in the package. Optimization algorithms will minimize
#' a fitness function. For more details of the algorithms
#' see [nsga2()], [nsga3()], [rnsga2()].
#'
#' Multi- and Many-Optimization of a fitness function using Non-dominated
#' Sorting Genetic Algorithms. The algorithms currently implemented by rmoo
#' are: NSGA-II, NSGA-III and R-NSGA-II
#'
#' The Non-dominated genetic algorithms II (NSGA-II) is a meta-heuristic proposed by
#' K. Deb, A. Pratap, S. Agarwal and T. Meyarivan in 2002. The purpose of the
#' algorithms is to find an efficient way to optimize multi-objectives functions
#' (two or more).
#'
#' The Non-dominated genetic algorithms III (NSGA-III) is a meta-heuristic proposed by
#' K. Deb and H. Jain in 2013.
#' The purpose of the algorithms is to find an efficient way to optimize
#' multi-objectives functions (more than three).
#'
#' The Reference point-based Non-dominated genetic algorithms II (R-NSGA-II) is
#' a meta-heuristic proposed by K. Deb and J. Sundar in 2006. It is a modification
#' of NSGA-II based on reference points in which the decision-maker supplies one
#' or more preference points and a weight vector that will guide the solutions
#' towards regions desired by the user.
#'
#' @name rmoo_main
#'
#' @param type the type of genetic algorithm to be run depending on the nature
#' of decision variables. Possible values are:
#' \describe{
#'     \item{'binary'}{for binary representations of decision variables.}
#'     \item{'real-valued'}{for optimization problems where the decision
#'     variables are floating-point representations of real numbers.}
#'     \item{'permutation'}{for problems that involves reordering of a list of
#'     objects.}
#'     \item{'discrete'}{for discrete representations of decision variables.}
#' }
#'
#' @param algorithm the type of genetic algorithm to be run depending on the nature
#' of decision variables. Possible values are:
#' \describe{
#'     \item{'NSGA-II'}{for .}
#'     \item{'NSGA-III'}{for .}
#'     \item{'R-NSGA-II'}{for .}
#' }
#'
#' @param fitness the fitness function, any allowable R function which takes as
#' input an individual string representing a potential solution, and returns a
#' numerical value describing its 'fitness'.
#' @param ... argument in which all the values necessary for the configuration
#' will be passed as parameters. The user is encouraged to see the documentations.
#' @param lower a vector of length equal to the decision variables providing the
#' lower bounds of the search space in case of real-valued or permutation
#' encoded optimizations.
#' @param upper a vector of length equal to the decision variables providing the
#' upper bounds of the search space in case of real-valued or permutation
#' encoded optimizations.
#' @param nBits a value specifying the number of bits to be used in binary
#' encoded optimizations.
#' @param nvars a value .
#' @param population an R function for randomly generating an initial population.
#' See [rmoo_Population()] for available functions.
#' @param selection an R function performing selection, i.e. a function which
#' generates a new population of individuals from the current population
#' probabilistically according to individual fitness.
#' @param crossover an R function performing crossover, i.e. a function which
#' forms offsprings by combining part of the genetic information from their
#' parents. See [rmoo_Crossover()] for available functions.
#' @param mutation an R function performing mutation, i.e. a function which
#' randomly alters the values of some genes in a parent chromosome.
#' See [rmoo_Mutation()] for available functions.
#' @param pcrossover the probability of crossover between pairs of chromosomes.
#' Typically this is a large value and by default is set to 0.8.
#' @param pmutation the probability of mutation in a parent chromosome. Usually
#' mutation occurs with a small probability, and by default is set to 0.1.
#' @param popSize the population size.
#' @param maxiter the maximum number of iterations to run before the NSGA search
#' is halted.
#' @param nObj number of objective in the fitness function.
#' @param names a vector of character strings providing the names of decision
#' variables.
#' @param suggestions a matrix of solutions strings to be included in the
#' initial population. If provided the number of columns must match the number
#' of decision variables.
#' @param monitor a logical or an R function which takes as input the current
#' state of the nsga-class object and show the evolution of the search. By
#' default, for interactive sessions the function rmooMonitor prints the average
#' and best fitness values at each iteration. If set to plot these information
#' are plotted on a graphical device. Other functions can be written by the user
#' and supplied as argument. In non interactive sessions, by default
#' monitor = FALSE so any output is suppressed.
#' @param parallel An optional argument which allows to specify if the NSGA-II
#' should be run sequentially or in parallel.
#' @param summary If there will be a summary generation after generation.
#' @param seed an integer value containing the random number generator state.
#' This argument can be used to replicate the results of a NSGA search. Note
#' that if parallel computing is required, the doRNG package must be installed.
#' @param reference_dirs Function to generate reference points using Das and
#' Dennis approach or matrix with supplied reference points.
#' @param epsilon controls the extent of obtained solutions by grouping all
#' solutions that have a normalized difference sum in objective values of epsilon or less.
#' @param normalization of the ideal points and nadir. They can be:
#' \describe{
#'     \item{'ever'}{.}
#'     \item{'front'}{.}
#'     \item{'no'}{.}
#' }
#'
#' @param extreme_points_as_ref_dirs flag to use extreme points as reference points.
#' @param weights vector specifies the importance of one objective function over
#' the other, by default all objectives have equal weights.
#' of [nsga2()], [rnsga2()], [nsga3()] in which the necessary parameters for each
#' algorithm are cited, in addition, the chosen strategy to execute must be
#' passed as an argument. This can be seen more clearly in the examples.
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @references Scrucca, L. (2017) On some extensions to 'GA' package: hybrid
#' optimisation, parallelisation and islands evolution. The R Journal, 9/1, 187-206.
#' doi: 10.32614/RJ-2017-008
#'
#' Kalyanmoy Deb and J. Sundar. 2006. Reference point based multi-objective
#' optimization using evolutionary algorithms. In Proceedings of the 8th annual
#' conference on Genetic and evolutionary computation (GECCO '06). Association
#' for Computing Machinery, New York, NY, USA, 635–642. doi: 10.1145/1143997.1144112
#'
#' K. Deb, A. Pratap, S. Agarwal and T. Meyarivan, 'A fast and
#' elitist multiobjective genetic algorithm: NSGA-II,' in IEEE Transactions on
#' Evolutionary Computation, vol. 6, no. 2, pp. 182-197, April 2002,
#' doi: 10.1109/4235.996017.
#'
#' K. Deb and H. Jain, "An Evolutionary Many-Objective Optimization
#' Algorithm Using Reference-Point-Based Nondominated Sorting Approach, Part I:
#' Solving Problems With Box Constraints," in IEEE Transactions on Evolutionary
#' Computation, vol. 18, no. 4, pp. 577-601, Aug. 2014,
#' doi: 10.1109/TEVC.2013.2281535.
#'
#' @seealso [nsga2()], [rnsga2()], [nsga3()]
#'
#' @return Returns an object of class nsga2-class, rnsga2-class or nsga3-class.
#' See [nsga2-class], [rnsga2-class], [nsga3-class] for a description of
#' available slots information.
#'
#' @examples
#' #Example 1
#' #Two Objectives - Real Valued
#' zdt1 <- function (x) {
#'  if (is.null(dim(x))) {
#'    x <- matrix(x, nrow = 1)
#'  }
#'  n <- ncol(x)
#'  g <- 1 + rowSums(x[, 2:n, drop = FALSE]) * 9/(n - 1)
#'  return(cbind(x[, 1], g * (1 - sqrt(x[, 1]/g))))
#' }
#'
#' #Not run:
#' \dontrun{
#' result <- rmoo(type = "real-valued",
#'                fitness = zdt1,
#'                algorithm = "NSGA-II",
#'                lower = c(0,0),
#'                upper = c(1,1),
#'                popSize = 100,
#'                nObj = 2,
#'                monitor = FALSE,
#'                maxiter = 500)
#'
#' }
#'
#' #Example 2
#' #Three Objectives - Real Valued
#' dtlz1 <- function (x, nobj = 3){
#'     if (is.null(dim(x))) {
#'         x <- matrix(x, 1)
#'     }
#'     n <- ncol(x)
#'     y <- matrix(x[, 1:(nobj - 1)], nrow(x))
#'     z <- matrix(x[, nobj:n], nrow(x))
#'     g <- 100 * (n - nobj + 1 + rowSums((z - 0.5)^2 - cos(20 * pi * (z - 0.5))))
#'     tmp <- t(apply(y, 1, cumprod))
#'     tmp <- cbind(t(apply(tmp, 1, rev)), 1)
#'     tmp2 <- cbind(1, t(apply(1 - y, 1, rev)))
#'     f <- tmp * tmp2 * 0.5 * (1 + g)
#'     return(f)
#' }
#'
#' #Define uniformly distributed reference points.
#' ref_points <- generate_reference_points(3,12)
#'
#' #Not Run
#' \dontrun{
#' result <- rmoo(type = "real-valued",
#'                 fitness = dtlz1,
#'                 algorithm = "NSGA-III",
#'                 lower = c(0,0,0),
#'                 upper = c(1,1,1),
#'                 popSize = 92,
#'                 nObj = 3,
#'                 reference_dirs = ref_points,
#'                 monitor = FALSE,
#'                 maxiter = 500)
#' }
#'
#' #Example 3
#' #Two Objectives - Real Valued with Preference-guided
#' zdt2 <- function (x)
#' {
#'   if (is.null(dim(x))) {
#'     x <- matrix(x, nrow = 1)
#'   }
#'   n <- ncol(x)
#'   g <- 1 + rowSums(x[, 2:n, drop = FALSE]) * 9/(n - 1)
#'   return(cbind(x[, 1], g * (1 - (x[, 1]/g)^2)))
#' }
#'
#' #Define uniformly distributed reference points.
#' ref_points <- rbind(c(1.0, 0.0), c(0.0, 1.0), c(0.5, 0.5))
#'
#' #Not run
#' \dontrun{
#' result <- rmoo(type = "real-valued",
#'                fitness = zdt2,
#'                algorithm = "R-NSGA-II",
#'                lower = c(0,0),
#'                upper = c(1,1),
#'                reference_dirs = ref_points,
#'                popSize = 92,
#'                nObj = 2,
#'                monitor = FALSE,
#'                maxiter = 500)
#'
#' }
#'
#' @export
#' @aliases rmoo,rmoo-main,rmoo-function
#' @rdname rmoo
rmoo <- function(type = c("binary", "real-valued", "permutation", "discrete"),
                 algorithm = c("NSGA-II", "NSGA-III", "R-NSGA-II"),
                 fitness, ...,
                 lower, upper, nBits, nvars,
                 population = rmooControl(type)$population,
                 selection = rmooControl(type)$selection,
                 crossover = rmooControl(type)$crossover,
                 mutation = rmooControl(type)$mutation,
                 pcrossover = 0.8,
                 pmutation = 0.1,
                 popSize = 50,
                 maxiter = 100,
                 nObj = NULL,
                 names = NULL,
                 suggestions = NULL,
                 monitor = if (interactive()) rmooMonitor else FALSE,
                 parallel = FALSE,
                 summary = FALSE,
                 seed = NULL,
                 reference_dirs = NULL,
                 epsilon = 0.001,
                 normalization = NULL,
                 extreme_points_as_ref_dirs = FALSE,
                 weights = NULL)
{
  start_time <- Sys.time()
  call       <- match.call()
  type       <- match.arg(type, choices = eval(formals(rmoo)$type))
  algorithm  <- match.arg(algorithm, choices = eval(formals(rmoo)$algorithm))
  callArgs   <- list(...)

  if (!is.function(population)) population <- get(population)
  if (!is.function(selection)) selection <- get(selection)
  if (!is.function(crossover)) crossover <- get(crossover)
  if (!is.function(mutation)) mutation <- get(mutation)

  # checkmate::assertCount(nObj)
  # checkmate::assertCount(maxiter)
  check_numeric_arg(nObj, "Objective number (nObj)", check_negative = TRUE)
  check_numeric_arg(maxiter, "Maximum number of iterations (maxiter)", check_negative = TRUE)
  check_function_arg(fitness, "Fitness function")
  check_probability_arg(pcrossover, "Probability of crossover (pcrossover)")
  check_probability_arg(pmutation, "Probability of mutation (pmutation)")
  check_algorithm_arg(nObj, algorithm, normalization, reference_dirs)
  if (popSize < 10) warning("The population size is less than 10.")

  switch(type,
         binary = {
           # checkmate::assertCount(nBits)
           check_numeric_arg(nBits, "Number of bits (nBits)", check_negative = TRUE)
           nBits <- as.vector(nBits)[1]
           lower <- upper <- NA
           nvars <- nBits
           if (is.null(names))
             names <- paste0("x", 1:nvars)
         },
         `real-valued` = {
           lnames <- names(lower)
           unames <- names(upper)
           lower <- as.vector(lower)
           upper <- as.vector(upper)

           check_numeric_arg(length(lower), "Length of lower")
           check_numeric_arg(length(upper), "Length of upper")
           nBits <- NA
           nvars <- length(upper)
           if (is.null(names) & !is.null(lnames)) names <- lnames
           if (is.null(names) & !is.null(unames)) names <- unames
           if (is.null(names))
             names <- paste0("x", 1:nvars)
         },
         permutation = {
           check_numeric_arg(as.vector(lower), "Lower bounds (lower)")
           check_numeric_arg(as.vector(upper), "Upper bounds (upper)")
           lower <- as.vector(lower)[1]
           upper <- as.vector(upper)[1]
           nBits <- NA
           nvars <- length(seq.int(lower, upper))
           if (is.null(names))
             names <- paste0("x", 1:nvars)
         },
         discrete = {
           check_numeric_arg(as.vector(lower), "Lower bounds (lower)")
           check_numeric_arg(as.vector(upper), "Upper bounds (upper)")
           check_numeric_arg(nvars, "Number of decision variables (nvars)")
           lower <- as.vector(lower)[1]
           upper <- as.vector(upper)[1]
           nBits <- NA
           if (is.null(names))
             names <- paste0("x", 1:nvars)
         }
  )

  if (is.null(suggestions)) {
    suggestions <- matrix(nrow = 0, ncol = nvars)
  } else {
    if (is.vector(suggestions)) {
      if (nvars > 1)
        suggestions <- matrix(suggestions, nrow = 1)
      else
        suggestions <- matrix(suggestions, ncol = 1)
    } else {
      suggestions <- as.matrix(suggestions)
    }
    if (nvars != ncol(suggestions))
      stop("Provided suggestions (ncol) matrix do not match number of variables of the problem")
  }

  # check monitor arg
  if (is.logical(monitor)) {
    if (monitor) monitor <- rmooMonitor
  }
  if (is.null(monitor)) monitor <- FALSE

  # Start parallel computing (if needed)
  if(is.logical(parallel)){
    if(parallel) {
      parallel <- startParallel(parallel)
      stopCluster <- TRUE
    } else {
      parallel <- stopCluster <- FALSE
    }
  }else {
    stopCluster <- if(inherits(parallel, "cluster")) FALSE else TRUE
    parallel <- startParallel(parallel)
  }
  on.exit(if(parallel & stopCluster)
    stopParallel(attr(parallel, "cluster")))
  # define operator to use depending on parallel being TRUE or FALSE
  `%DO%` <- if(parallel && requireNamespace("doRNG", quietly = TRUE)){
    doRNG::`%dorng%` } else if (parallel){ foreach::`%dopar%` } else { foreach::`%do%` }

  # set seed for reproducibility
  if (!is.null(seed))
    set.seed(seed)

  i. <- NULL #dummy to trick R CMD check

  Fitness <- matrix(NA, nrow = popSize, ncol = nObj)

  fitnessSummary <- vector("list", maxiter)

  p_fit <- q_fit <- matrix(NA_real_, nrow = popSize, ncol = nObj)

  switch(type,
         binary = {
           Pop <- P <- Q <- matrix(NA_real_, nrow = popSize, ncol = nBits)
         },
         `real-valued` = {
           Pop <- P <- Q <- matrix(NA_real_, nrow = popSize, ncol = nvars)
         },
         permutation = {
           Pop <- P <- Q <- matrix(NA_real_, nrow = popSize, ncol = nvars)
         },
         discrete = {
           Pop <- P <- Q <- matrix(NA_integer_, nrow = popSize, ncol = nvars)
         }
  )

  ng <- min(nrow(suggestions), popSize)

  if (ng > 0) {
    Pop[1:ng, ] <- suggestions
  }

  object <- create_object_instance(algorithm, call, type, lower, upper, nBits,
                                   nvars, names, popSize, maxiter, suggestions,
                                   pcrossover, pmutation, Fitness,
                                   fitnessSummary, reference_dirs, nObj)

  if (maxiter == 0)
    return(object)

  if (popSize > ng) {
    Pop[(ng + 1):popSize, ] <- population(object)[1:(popSize - ng), ]
  }

  object@population <- Pop

  # Evaluate Solution Fitness
  Fitness <- evaluate_fitness(parallel, popSize, Fitness, fitness, Pop, `%DO%`, callArgs)

  object@population <- P <- Pop
  object@fitness <- p_fit <- Fitness

  # First Non-dominated Ranking
  out <- non_dominated_fronts(object)
  object@f <- out$fit
  object@front <- matrix(unlist(out$fronts), ncol = 1, byrow = TRUE)


  for (iter in seq_len(maxiter)) {
    # initialize the iteration counter
    object@iter <- iter

    # Selection operator
    if (is.function(selection)) {
      sel <- selection(object, nObj)
      Pop <- sel$population
      Fitness <- sel$fitness
    } else {
      sel <- sample(1:popSize, size = popSize, replace = TRUE)
      Pop <- object@population[sel, ]
      Fitness <- object@fitness[sel, ]
    }
    object@population <- Pop
    object@fitness <- Fitness

    # Crossover operator
    if (is.function(crossover) & pcrossover > 0) {
      nmating <- floor(popSize / 2)
      mating <- matrix(sample(1:(2 * nmating), size = (2 * nmating)), ncol = 2)
      for (i in seq_len(nmating)) {
        if (pcrossover > runif(1)) {
          parents <- mating[i, ]
          Crossover <- crossover(object, parents)
          Pop[parents, ] <- Crossover$children
          Fitness[parents, ] <- Crossover$fitness
        }
      }
    }
    object@population <- Pop
    object@fitness <- Fitness

    # Mutation operator
    pm <- if (is.function(pmutation)) {
      pmutation(object)
    } else {pmutation}
    if (is.function(mutation) & pm > 0) {
      for (i in seq_len(popSize)) {
        if (pm > runif(1)) {
          Mutation <- mutation(object, i)
          Pop[i, ] <- Mutation
          Fitness[i, ] <- NA
        }
      }
    }
    object@population <- Q <- Pop
    object@fitness <- q_fit <- Fitness

    # Evaluate Solution Fitness
    Fitness <- evaluate_fitness(parallel, popSize, Fitness, fitness, Pop, `%DO%`, callArgs)

    object@population <- Q <- Pop
    object@fitness <- q_fit <- Fitness

    # R = P U Q
    object@population <- Pop <- rbind(P, Q)
    object@fitness <- rbind(p_fit, q_fit)

    # Fast Non Dominated Sorting
    out <- non_dominated_fronts(object)
    object@f <- out$fit
    object@front <- matrix(unlist(out$fronts), ncol = 1, byrow = TRUE)

    out <- optimization_process(object, algorithm, nObj, epsilon, weights, normalization, extreme_points_as_ref_dirs)

    object <- out$object
    Pop <- out$p_pop
    p_fit <- out$p_fit

    object@population <- P <- Pop
    object@fitness <- p_fit

    # Fast Non Dominated Sorting
    out <- non_dominated_fronts(object)
    object@f <- out$fit
    object@front <- matrix(unlist(out$fronts), ncol = 1, byrow = TRUE)

    if (summary) {
      fitnessSummary[[iter]] <- progress(object, callArgs)
      object@summary <- fitnessSummary
    } else {
      object@summary <- list(NULL)
    }

    if (is.function(monitor)) {
      # monitor(object = object, number_objective = nObj)
      monitor(object = object, callArgs)
    }

    # if (max(Fitness, na.rm = TRUE) >= maxFitness)
    #   break
    if (object@iter == maxiter)
      break
  }
  object@execution_time <- as.numeric(difftime(Sys.time() , start_time, units = "secs"))
  # solution <- object

  return(object)
}



# setClass("Person", representation(name = "character", age = "numeric", greet = "function"))
#
# setValidity("Person", function(object) {
#   if (nchar(object@name) == 0) {
#     return("Name must be non-empty")
#   }
#   TRUE
# })
#
# setGeneric("age", function(object, birthdate) {
#   standardGeneric("age")
# })
#
#
# setMethod("age", "Person", function(object, birthdate) {
#   today <- Sys.Date()
#   as.numeric(difftime(today, as.Date(birthdate), units = "weeks")) %/% 52
# })
#
#
# greetFunction <- function(person) {
#   cat(paste0("Hello, my name is ", person@name, " and I am ", person@age, " years old.\n"))
# }
#
# hadley <- new("Person", name = "Hadley", age = 31, greet = greetFunction)
#
# hadley$age("1979-07-30")
#




# r_nsga_iii <- function(){
#   get_ref_dirs_from_points <- function(ref_point, ref_dirs, mu = 0.1) {
#     n_obj <- ncol(ref_point)
#
#     val <- list()
#
#     n_vector <- rep(1 / sqrt(n_obj), n_obj)  # Normal vector of Das Dennis plane
#
#
#     point_on_plane <- diag(n_obj)[1, ]  # Point on Das-Dennis
#
#     for (i in 1:nrow(ref_point)) {
#       point <- ref_point[i,]
#       ref_dir_for_aspiration_point <- ref_dirs * mu # Copy of computed reference directions
#
#       cent <- colMeans(ref_dir_for_aspiration_point)  # Find centroid of shrunken reference points
#
#       # Project shrunken Das-Dennis points back onto original Das-Dennis hyperplane
#       intercept <- line_plane_intersection(rep(0, n_obj), point, point_on_plane, n_vector)
#       shift <- intercept - cent  # shift vector
#
#       ref_dir_for_aspiration_point <- sweep(ref_dir_for_aspiration_point, 2, shift, "+")
#
#       # If reference directions are located outside of first octant, redefine points onto the border
#       if (!all(ref_dir_for_aspiration_point > 0)) {
#         ref_dir_for_aspiration_point[ref_dir_for_aspiration_point < 0] <- 0
#         ref_dir_for_aspiration_point <- sweep(ref_dir_for_aspiration_point, 1, rowSums(ref_dir_for_aspiration_point), "/")
#       }
#       val <- c(val, list(ref_dir_for_aspiration_point))
#       #val <- c(val, ref_dir_for_aspiration_point)
#     }
#
#     val <- c(val, list(diag(n_obj)))  # Add extreme points
#     # val <- c(val, diag(n_obj))  # Add extreme points
#     return(do.call(rbind, val))
#     # return(array(unlist(val), dim = c(length(val), n_obj)))
#   }
#
#
#   line_plane_intersection <- function(l_zero, l_one, p_zero, p_no, epsilon = 1e-6) {
#
#     l <- l_one - l_zero
#     dot <- sum(l * p_no)
#
#     if (abs(dot) > epsilon) {
#       w <- p_zero - l_zero
#       d <- sum(w * p_no) / dot
#       l <- l * d
#       return(l_zero + l)
#     } else {
#       ref_proj <- l_one - sum((l_one - p_zero) * p_no) * p_no
#
#       return(ref_proj)
#     }
#   }
# }

# ref_point <- rbind(c(1.0, 0.5, 0.2),c(0.3, 0.2, 0.6))
# ref_dirs <- generate_reference_points(3,12)
# pop_per_ref_point=50
# mu <- 0.1

# get_ref_dirs_from_points(ref_point, ref_dirs, mu = 0.1)

context("Executing nsga2")

test_that("nsga2() return the value suggestions", {
  testfunction <- function (x){
    return(x)
  }

  x <- matrix(rep(1,30), ncol = 3)

  out <- rmoo(type = "real-valued", algorithm = "NSGA-II", fitness = testfunction,
               lower = c(0,0,0), upper = c(1,1,1),
               popSize = 10, suggestions = x,
               pcrossover = 0, pmutation = 0,
               seed = 1, parallel = FALSE, monitor = FALSE,
               summary = FALSE, maxiter = 1, nObj = 3)

  expect_identical(out@fitness, x)
})

# Update Ideal Point
#
# Calculates the ideal point (minimum value per objective).
#
# @param object An object with a fitness matrix and an ideal_point vector.
# @param nObj Number of objectives.
# @return A numeric vector representing the updated ideal point.
##Ideal Point
#' @export
UpdateIdealPoint <- function(object, nObj) {
  cost <- as.matrix(object@fitness)
  if (anyNA(object@ideal_point)) {
    ideal_point <- apply(cost, 2, min)
  } else {
    ideal_point <- apply(rbind(object@ideal_point, cost), 2, min)
  }
  return(ideal_point)
}

# Update Worst Point
#
# Calculates the worst point (maximum value per objective).
#
# @param object An object with a fitness matrix and a worst_point vector.
# @param nObj Number of objectives.
# @return A numeric vector representing the updated worst point.
## Worst points
#' @export
UpdateWorstPoint <- function(object, nObj) {
  cost <- as.matrix(object@fitness)
  if (anyNA(object@worst_point)) {
    worst_point <- apply(cost, 2, max)
  } else {
    worst_point <- apply(rbind(object@worst_point, cost), 2, max)
  }
  return(worst_point)
}

# Perform Scalarizing to Find Extreme Points
#
# Calculates extreme points using the Achievement Scalarizing Function (ASF).
#
# @param population Population matrix.
# @param fitness Fitness matrix.
# @param smin Current minimum scalar values.
# @param extreme_points Current extreme points matrix.
# @param ideal_point Ideal point vector.
# @return A list with updated extreme points and smin index.
## Extreme points
#' @export
PerformScalarizing <- function(population, fitness, smin, extreme_points, ideal_point) {
  nObj <- ncol(fitness)

  if (anyNA(smin)) {
    extreme_points <- matrix(0, nObj, nObj)
    smin <- rep(Inf, nObj)
    F <- fitness
  } else {
    F <- rbind(extreme_points, fitness)
  }

  fp <- sweep(F, 2, ideal_point, FUN = "-")
  w <- diag(1, nObj)
  w[w == 0] <- 1e-6
  # w[which(w == 0)] <- 1e-06

  sminj <- apply(fp, 1, function(x) max(x/w))

  for (j in 1:nObj) {
    sminj_j <- min(sminj, na.rm = TRUE)
    ind <- which(sminj == sminj_j)

    if (length(ind) > 1)
      ind <- sample(ind, 1)

    if (sminj_j < smin[j]) {
      extreme_points[j, ] <- F[ind, ]
      smin[j] <- sminj_j
    }
  }
  out <- list(extremepoint = extreme_points, indexmin = smin)
  return(out)
}
# PerformScalarizing <- function(population, fitness, smin, extreme_points, ideal_point) {
#     nPop <- nrow(population)
#     nObj <- ncol(fitness)
#     if (!anyNA(smin)) {
#         F <- rbind(extreme_points, fitness)
#     } else {
#         extreme_points <- matrix(0, nObj, nObj)
#         smin <- rep(Inf, nObj)
#         F <- fitness
#     }
#     fp <- sweep(F, 2, ideal_point)
#     w <- diag(1, nObj)
#     w[which(w == 0)] <- 1e-06
#     for (j in 1:nObj) {
#         s <- rep(0, nPop)
#         for (i in 1:nPop) {
#             s[i] <- max(fp[i, ]/w[j, ])
#         }
#         sminj <- min(s)
#         ind <- which(s == sminj)
#
#         if (length(ind) > 1)
#             ind <- sample(ind, 1)
#
#         if (sminj < smin[j]) {
#             extreme_points[j, ] <- F[ind, ]
#             smin[j] <- sminj
#         }
#     }
#     out <- list(extremepoint = extreme_points, indexmin = smin)
#     return(out)
# }

# Calculate Nadir Point
#
# Computes the Nadir point from the extreme and ideal points.
#
# @param object An object containing extreme_points, ideal_point, worst_point, worst_of_front, worst_of_population.
# @return A numeric vector representing the nadir point.
## Nadir Point
#' @export
get_nadir_point <- function(object) {
  # Extract relevant data from object
  extreme_point <- object@extreme_points
  ideal_point <- object@ideal_point
  worst_point <- object@worst_point
  worst_of_front <- object@worst_of_front
  worst_of_population <- object@worst_of_population

  # Calculate nadir point
  M <- sweep(extreme_point, 2, ideal_point)
  b <- rep(1, ncol(object@fitness))

  # Check if M is a singular matrix
  if (abs(det(M)) < 1e-10) {
    nadir_point <- worst_of_front
  } else {
    plane <- solve(M, b)
    intercepts <- 1/plane
    nadir_point <- ideal_point + intercepts

    # Check for errors and handle them
    if (!all.equal(as.vector(M %*% plane), b) || any(intercepts <= 1e-05) || any(nadir_point > worst_point)) {
      nadir_point <- worst_of_front
    }
  }

  # Update nadir point with worst of population if necessary
  b <- (nadir_point - ideal_point) <= 1e-06
  nadir_point[b] <- worst_of_population[b]

  return(nadir_point)
}
# get_nadir_point <- function(object) {
#     extreme_point <- object@extreme_points
#     ideal_point <- object@ideal_point
#     worst_point <- object@worst_point
#     nObj <- ncol(object@fitness)
#     worst_of_front <- object@worst_of_front
#     worst_of_population <- object@worst_of_population
#     out <- tryCatch({
#         M <- sweep(extreme_point, 2, ideal_point)
#         b <- rep(1, nObj)
#         plane <- solve(M, b)
#         intercepts <- 1/plane
#         nadir_point <- ideal_point + intercepts
#         if (!all.equal(as.vector(M %*% plane), b) || any(intercepts <= 1e-05) || any(nadir_point > worst_point)) {
#             stop()
#         }
#         nadir_point
#     }, error = function(e) {
#         nadir_point <- worst_of_front
#         return(nadir_point)
#     })
#     b <- (out - ideal_point) <= 1e-06
#     out[b] <- worst_of_population[b]
#     return(out)
# }

# @export
check_numeric_arg <- function(arg=NULL, name, check_negative = FALSE) {
  if (is.null(arg)) stop(paste("Please, define the", name))
  if (!is.numeric(arg))
    stop(paste(name, "must be a numeric value."))
  # checkmate::assertCount(arg)
  if ((arg %% 1 != 0))
    stop(paste(name, "must be a non-negative integer."))
  if (!check_negative && arg < 0)
    stop(paste(name, "must not be negative."))
}

# @export
check_probability_arg <- function(arg, name) {
  if (!is.numeric(arg))
    stop(paste(name, "must be a numeric value."))
  if (arg < 0 || arg > 1)
    stop(paste(name, "must be a numeric value in [0, 1]."))
}

# @export
check_function_arg <- function(arg=NULL, name) {
  if (is.null(arg)) stop(paste("Please, define the", name))
  if (!is.function(arg))
    stop(paste(name, "must be a function."))
}

# @export
check_matrix_arg <- function(arg, name) {
  if (is.null(arg)) stop(paste("Please, define the", name))
  if (!is.matrix(arg))
    stop(paste(name, "must be a matrix."))
}

# @export
check_algorithm_arg <- function(nObj, algorithm, normalization=NULL, reference_dirs=NULL){
  if (algorithm == "NSGA-III"){
    check_matrix_arg(reference_dirs, "Reference points")
    if (ncol(reference_dirs) != nObj) {
      stop("Dimensionality of reference points must be equal to the number of objectives")
    }
  }
  if (algorithm == "R-NSGA-II"){
    check_matrix_arg(reference_dirs, "Reference points")
    if (ncol(reference_dirs) != nObj) {
      stop("Dimensionality of reference points must be equal to the number of objectives")
    }
    if (is.null(normalization)) stop(paste("Please, define the Normalization type ('ever', 'front', 'no')"))

    normalization_values <- c("ever", "front", "no")
    if (!(normalization %in% normalization_values)) {
      stop("Normalization of the crowding distance must be 'ever', 'front' or 'no'.")
    }
  }
}

# @export
create_object_instance <- function(algorithm, call, type, lower,
                                   upper, nBits, nvars, names, popSize,
                                   maxiter, suggestions, pcrossover,
                                   pmutation, Fitness, fitnessSummary,
                                   reference_dirs, nObj){
  if (algorithm == "NSGA-II") {
    object <- new("nsga2",
                  call = call, type = type, lower = lower, upper = upper,
                  nBits = nBits, nvars = nvars,
                  names = if (is.null(names))
                    character()
                  else names,
                  popSize = popSize, front = matrix(), f = list(), iter = 0,
                  run = 1, maxiter = maxiter, suggestions = suggestions,
                  population = matrix(), pcrossover = pcrossover,
                  pmutation = if (is.numeric(pmutation))
                    pmutation
                  else NA,
                  crowdingDistance = matrix(NA_real_, nrow = popSize),
                  fitness = Fitness, summary = fitnessSummary)
  } else if (algorithm == "R-NSGA-II") {
    object <- new("rnsga2",
                  call = call, type = type, lower = lower, upper = upper,
                  nBits = nBits, nvars = nvars,
                  names = if (is.null(names))
                    character()
                  else names,
                  popSize = popSize, front = matrix(), f = list(), iter = 0,
                  run = 1, maxiter = maxiter, suggestions = suggestions,
                  population = matrix(), pcrossover = pcrossover,
                  pmutation = if (is.numeric(pmutation))
                    pmutation
                  else NA,
                  crowdingDistance = c(), fitness = Fitness,
                  reference_points = reference_dirs, extreme_points = matrix(),
                  smin =  rep(NA, nObj), summary = fitnessSummary)
  } else if (algorithm == "NSGA-III") {
    object <- new("nsga3",
                  call = call, type = type, lower = lower, upper = upper,
                  nBits = nBits, nvars = nvars,
                  names = if (is.null(names))
                    character()
                  else names,
                  popSize = popSize, front = matrix(), f = list(), iter = 0,
                  run = 1, maxiter = maxiter, suggestions = suggestions,
                  population = matrix(), ideal_point = NA, worst_point = NA,
                  smin = rep(NA, nObj), extreme_points = matrix(),
                  worst_of_population = rep(NA, nObj), worst_of_front = rep(NA, nObj),
                  nadir_point = rep(NA, nObj), pcrossover = pcrossover,
                  pmutation = if (is.numeric(pmutation))
                    pmutation
                  else NA,
                  reference_points = reference_dirs, fitness = Fitness, summary = fitnessSummary)
  } else {
    stop("Invalid algorithm specified. Supported algorithms: 'NSGA-II', 'R-NSGA-II', 'NSGA-III'")
  }

  return(object)
}

# @export
evaluate_fitness <- function(parallel, popSize, Fitness, fitness, Pop, `%DO%`, callArgs) {
  if (!parallel) {
    for (i in seq_len(popSize)) {
      if (is.na(Fitness[i])) {
        fit <- do.call(fitness, c(list(Pop[i, ]), callArgs))
        Fitness[i, ] <- fit
      }
    }
  } else {
    Fitness <- foreach(i. = seq_len(popSize), .combine = "rbind") %DO%
      {
        if(is.na(Fitness[i.])) do.call(fitness, c(list(Pop[i.,]), callArgs))
        else Fitness[i.,]
      }
  }
  return(Fitness)
}

# @export
optimization_process <- function(object, algorithm, nObj, epsilon, weights, normalization, extreme_points_as_ref_dirs) {
  if (algorithm == "NSGA-II") {
    out <- nsga_ii(object, nObj)
  } else if (algorithm == "R-NSGA-II") {
    out <- r_nsga_ii(object, epsilon, weights, normalization, extreme_points_as_ref_dirs)
  } else if (algorithm == "NSGA-III") {
    out <- nsga_iii(object, nObj)
  } else {
    stop("Invalid algorithm specified. Supported algorithms: 'NSGA-II', 'R-NSGA-II', 'NSGA-III'")
  }
  return(out)
}



get_relation <- function(a, b, cva = NULL, cvb = NULL) {
  if (!is.null(cva) && !is.null(cvb)) {
    if (cva < cvb) {
      return(1)
    } else if (cvb < cva) {
      return(-1)
    }
  }

  val <- 0
  for (i in 1:length(a)) {
    if (a[i] < b[i]) {
      if (val == -1) {
        return(0)
      }
      val <- 1
    } else if (b[i] < a[i]) {
      if (val == 1) {
        return(0)
      }
      val <- -1
    }
  }
  return(val)
}


compare <- function(a, a_val, b, b_val, method, return_random_if_equal = TRUE) {
  if (method == 'larger_is_better') {
    if (a_val > b_val) {
      return(a)
    } else if (a_val < b_val) {
      return(b)
    } else {
      if (return_random_if_equal) {
        return(sample(c(a, b), 1))
      } else {
        return(NULL)
      }
    }
  } else if (method == 'smaller_is_better') {
    if (a_val < b_val) {
      return(a)
    } else if (a_val > b_val) {
      return(b)
    } else {
      if (return_random_if_equal) {
        return(sample(c(a, b), 1))
      } else {
        return(NULL)
      }
    }
  }
}

random_permutations <- function(n, l, concat = TRUE) {
  P <- list()
  for (i in 1:n) {
    P[[i]] <- sample(1:l, l)
  }
  if (concat) {
    P <- unlist(P)
  }
  return(P)
}

context("Running rmoo check_validation")

test_that("check_numeric_arg() return the value suggestions", {
  expect_error(check_numeric_arg(arg=NULL, "test input", check_negative = FALSE))
  expect_error(check_numeric_arg(arg="3", "test input", check_negative = FALSE))
  expect_error(check_numeric_arg(arg=3.5, "test input", check_negative = FALSE))
  expect_error(check_numeric_arg(arg=-3, "test input", check_negative = FALSE))
})


test_that("check_probability_arg() return the value suggestions", {
  expect_error(check_probability_arg(arg="0.1", "test input"))
  expect_error(check_probability_arg(arg=-0.1, "test input"))
  expect_error(check_probability_arg(arg=1.1, "test input"))
  # expect_identical(colSums(out), 0)
})

test_that("check_function_arg() return the value suggestions", {
  expect_error(check_function_arg(arg=NULL, "test input"))
  expect_error(check_function_arg(arg=3, "test input"))
})

test_that("check_matrix_arg() return the value suggestions", {
  expect_error(check_matrix_arg(arg=NULL, "test input"))
  expect_error(check_matrix_arg(arg=3, "test input"))
  expect_error(check_matrix_arg(arg="3", "test input"))
})

test_that("check_algorithm_arg() return the value suggestions", {
  expect_error(check_algorithm_arg(nObj=3, algorithm="R-NSGA-II"))
  expect_error(check_algorithm_arg(nObj=3, algorithm="NSGA-III"))
})

#' Niche-Preservation Operation
#'
#' Generation of niche, by associating reference points to population members
#'
#' Niching procesure is a algorithms proposed by K. Deb and H. Jain in 2013.
#'
#'
#' @param pop Last Front Population
#' @param n_remaining Number of points to choose
#' @param niche_count Niche count of individuals with the reference point
#' @param niche_of_individuals Count of the closest reference point to the last
#' front objective values
#' @param dist_to_niche Distance between closest reference point to last front
#' objective values
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @references K. Deb and H. Jain, 'An Evolutionary Many-Objective Optimization
#' Algorithm Using Reference-Point-Based Nondominated Sorting Approach, Part I:
#' Solving Problems With Box Constraints,' in IEEE Transactions on Evolutionary
#' Computation, vol. 18, no. 4, pp. 577-601, Aug. 2014,
#' doi: 10.1109/TEVC.2013.2281535.
#'
#' Scrucca, L. (2017) On some extensions to 'GA' package: hybrid optimisation,
#' parallelisation and islands evolution. The R Journal, 9/1, 187-206.
#' doi: 10.32614/RJ-2017-008
#'
#' Felix-Antoine Fortin, Francois-Michel De Rainville, Marc-André Gardner
#' Gardner, Marc Parizeau, and Christian Gagne. 2012. DEAP: evolutionary
#' algorithms made easy. J. Mach. Learn. Res. 13, 1 (January 2012), 2171–2175.
#'
#'
#' @seealso [associate_to_niches()], [PerformScalarizing()]
#'
#' @return Returns the association of reference points to each individual in the population.
#' @export
niching <- function(pop, n_remaining, niche_count, niche_of_individuals, dist_to_niche) {
    survivors <- c()
    mask <- rep(TRUE, nrow(pop))
    while (length(survivors) < n_remaining) {
        n <- n_remaining - length(survivors)

        available_niches <- rep(FALSE, length(niche_count))
        available_niches[unique(niche_of_individuals[mask])] <- TRUE

        min_count <- min(niche_count[available_niches])

        selected_niches <- which((niche_count == min_count) & available_niches)

        if (length(selected_niches) > 1) {
            selected_niches <- sample(selected_niches)
        }
        selected_niches <- as.vector(na.omit(selected_niches[1:n]))

        for (i in selected_niches) {
            niche_of_individual <- which((niche_of_individuals == i) & mask)
            if (length(niche_of_individual) > 1) {
                niche_of_individual <- sample(niche_of_individual)
            }

            if (niche_count[i] == 0) {
                s <- niche_of_individual[which.min(dist_to_niche[niche_of_individual])]
            } else {
                s <- niche_of_individual[1]
            }

            mask[s] <- FALSE
            niche_count[i] <- niche_count[i] + 1
            survivors <- c(survivors, s)

        }
    }
    return(survivors)
}


# niching <- function(pop, n_remaining, niche_count, niche_of_individuals, dist_to_niche) {
#   survivors <- c()
#   mask <- rep(TRUE, nrow(pop))
#
#   available_niches <- unique(niche_of_individuals)
#   n_available_niches <- length(available_niches)
#
#   while (length(survivors) < n_remaining) {
#     n <- n_remaining - length(survivors)
#
#     available_niches_mask <- available_niches[niche_count[available_niches] > 0]
#     available_niches_min_count <- min(niche_count[available_niches_mask])
#     selected_niches <- available_niches_mask[niche_count[available_niches_mask] == available_niches_min_count]
#
#     if (length(selected_niches) > n) {
#       selected_niches <- sample(selected_niches, n)
#     }
#
#     for (i in selected_niches) {
#       niche_of_individual <- which(niche_of_individuals == i & mask)
#
#       if (length(niche_of_individual) > 1) {
#         niche_of_individual <- sample(niche_of_individual)
#       }
#
#       s <- niche_of_individual[which.min(dist_to_niche[niche_of_individual])]
#
#       mask[s] <- FALSE
#       niche_count[i] <- niche_count[i] + 1
#       survivors <- c(survivors, s)
#     }
#   }
#
#   return(survivors)
# }

#' Non-Dominated Sorting in Genetic Algorithms II
#'
#' Minimization of a fitness function using non-dominated sorting genetic
#' algorithms - II (NSGA-IIs). Multiobjective evolutionary algorithms
#'
#' The Non-dominated genetic algorithms II is a meta-heuristic proposed by
#' K. Deb, A. Pratap, S. Agarwal and T. Meyarivan in 2002. The purpose of the
#' algorithms is to find an efficient way to optimize multi-objectives functions
#' (two or more).
#'
#' @param type the type of genetic algorithm to be run depending on the nature
#' of decision variables. Possible values are:
#' \describe{
#'     \item{'binary'}{for binary representations of decision variables.}
#'     \item{'real-valued'}{for optimization problems where the decision
#'     variables are floating-point representations of real numbers.}
#'     \item{'permutation'}{for problems that involves reordering of a list of
#'     objects.}
#' }
#'
#' @param fitness the fitness function, any allowable R function which takes as
#' input an individual string representing a potential solution, and returns a
#' numerical value describing its 'fitness'.
#' @param ... additional arguments to be passed to the fitness function. This
#' allows to write fitness functions that keep some variables fixed during the
#' search
#' @param lower a vector of length equal to the decision variables providing the
#' lower bounds of the search space in case of real-valued or permutation
#' encoded optimizations.
#' @param upper a vector of length equal to the decision variables providing the
#' upper bounds of the search space in case of real-valued or permutation
#' encoded optimizations.
#' @param nBits a value specifying the number of bits to be used in binary
#' encoded optimizations
#' @param population an R function for randomly generating an initial population.
#' See [rmoo_Population()] for available functions.
#' @param selection an R function performing selection, i.e. a function which
#' generates a new population of individuals from the current population
#' probabilistically according to individual fitness.
#' @param crossover an R function performing crossover, i.e. a function which
#' forms offsprings by combining part of the genetic information from their
#' parents. See [rmoo_Crossover()] for available functions.
#' @param mutation an R function performing mutation, i.e. a function which
#' randomly alters the values of some genes in a parent chromosome.
#' See [rmoo_Mutation()] for available functions.
#' @param popSize the population size.
#' @param nObj number of objective in the fitness function.
#' @param pcrossover the probability of crossover between pairs of chromosomes.
#' Typically this is a large value and by default is set to 0.8.
#' @param pmutation the probability of mutation in a parent chromosome. Usually
#' mutation occurs with a small probability, and by default is set to 0.1.
#' @param maxiter the maximum number of iterations to run before the NSGA search
#' is halted.
#' @param run the number of consecutive generations without any improvement in
#' the best fitness value before the NSGA is stopped
#' @param maxFitness the upper bound on the fitness function after that the NSGA
#' search is interrupted.
#' @param names a vector of character strings providing the names of decision
#' variables.
#' @param suggestions a matrix of solutions strings to be included in the
#' initial population. If provided the number of columns must match the number
#' of decision variables.
#' @param parallel An optional argument which allows to specify if the NSGA-II
#' should be run sequentially or in parallel.
#' @param monitor a logical or an R function which takes as input the current
#' state of the nsga-class object and show the evolution of the search. By
#' default, for interactive sessions the function rmooMonitor prints the average
#' and best fitness values at each iteration. If set to plot these information
#' are plotted on a graphical device. Other functions can be written by the user
#' and supplied as argument. In non interactive sessions, by default
#' monitor = FALSE so any output is suppressed.
#' @param summary If there will be a summary generation after generation.
#' @param seed an integer value containing the random number generator state.
#' This argument can be used to replicate the results of a NSGA search. Note
#' that if parallel computing is required, the doRNG package must be installed.
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @references K. Deb, A. Pratap, S. Agarwal and T. Meyarivan, 'A fast and
#' elitist multiobjective genetic algorithm: NSGA-II,' in IEEE Transactions on
#' Evolutionary Computation, vol. 6, no. 2, pp. 182-197, April 2002,
#' doi: 10.1109/4235.996017.
#'
#' Scrucca, L. (2017) On some extensions to 'GA' package: hybrid optimisation,
#' parallelisation and islands evolution. The R Journal, 9/1, 187-206.
#' doi: 10.32614/RJ-2017-008
#'
#' @seealso [nsga()], [nsga3()]
#'
#' @return Returns an object of class nsga2-class. See [nsga2-class] for a
#' description of available slots information.
#'
#' @examples
#' #Example
#' #Two Objectives - Real Valued
#' zdt1 <- function (x) {
#'  if (is.null(dim(x))) {
#'    x <- matrix(x, nrow = 1)
#'  }
#'  n <- ncol(x)
#'  g <- 1 + rowSums(x[, 2:n, drop = FALSE]) * 9/(n - 1)
#'  return(cbind(x[, 1], g * (1 - sqrt(x[, 1]/g))))
#' }
#'
#' #Not run:
#' \dontrun{
#' result <- nsga2(type = "real-valued",
#'                 fitness = zdt1,
#'                 lower = c(0,0),
#'                 upper = c(1,1),
#'                 popSize = 100,
#'                 nObj = 2,
#'                 monitor = FALSE,
#'                 maxiter = 500)
#' }
#'
#' #Example 2
#' #Three Objectives - Real Valued
#' dtlz1 <- function (x, nobj = 3){
#'     if (is.null(dim(x))) {
#'         x <- matrix(x, 1)
#'     }
#'     n <- ncol(x)
#'     y <- matrix(x[, 1:(nobj - 1)], nrow(x))
#'     z <- matrix(x[, nobj:n], nrow(x))
#'     g <- 100 * (n - nobj + 1 + rowSums((z - 0.5)^2 - cos(20 * pi * (z - 0.5))))
#'     tmp <- t(apply(y, 1, cumprod))
#'     tmp <- cbind(t(apply(tmp, 1, rev)), 1)
#'     tmp2 <- cbind(1, t(apply(1 - y, 1, rev)))
#'     f <- tmp * tmp2 * 0.5 * (1 + g)
#'     return(f)
#' }
#'
#' #Not run:
#' \dontrun{
#' result <- nsga2(type = "real-valued",
#'                 fitness = dtlz1,
#'                 lower = c(0,0,0),
#'                 upper = c(1,1,1),
#'                 popSize = 92,
#'                 nObj = 3,
#'                 monitor = FALSE,
#'                 maxiter = 500)
#' }
#'
#' @export
nsga2 <- function(type = c("binary", "real-valued", "permutation"),
    fitness, ...,
    lower, upper, nBits,
    population = rmooControl(type)$population,
    selection = rmooControl(type)$selection,
    crossover = rmooControl(type)$crossover,
    mutation = rmooControl(type)$mutation,
    popSize = 50,
    nObj = NULL,
    pcrossover = 0.8,
    pmutation = 0.1,
    maxiter = 100,
    run = maxiter,
    maxFitness = Inf,
    names = NULL,
    suggestions = NULL,
    parallel = FALSE,
    monitor = if (interactive()) rmooMonitor else FALSE,
    summary = FALSE,
    seed = NULL)
{
    call <- match.call()

    type <- match.arg(type, choices = eval(formals(nsga2)$type))

    callArgs <- list(...)

    callArgs$strategy <- NULL

    if (!is.function(population))
        population <- get(population)
    if (!is.function(selection))
        selection <- get(selection)
    if (!is.function(crossover))
        crossover <- get(crossover)
    if (!is.function(mutation))
        mutation <- get(mutation)

    if (is.null(nObj)) {
      stop("Please, define the objective number (nObj)")
    } else {
      if (!is.numeric(nObj) | (nObj%%1!=0)) {
        stop("Objective number (nObj) is a character or is not an integer.")
      }
    }

    if (missing(fitness)) {
        stop("A fitness function must be provided")
    }
    if (!is.function(fitness)) {
        stop("A fitness function must be provided")
    }
    if (popSize < 10) {
        warning("The population size is less than 10.")
    }
    if (maxiter < 1) {
        stop("The maximum number of iterations must be at least 1.")
    }
    if (pcrossover < 0 | pcrossover > 1) {
        stop("Probability of crossover must be between 0 and 1.")
    }
    if (is.numeric(pmutation)) {
        if (pmutation < 0 | pmutation > 1) {
            stop("If numeric probability of mutation must be between 0 and 1.")
        } else if (!is.function(population)) {
            stop("pmutation must be a numeric value in (0,1) or a function.")
        }
    }

    if (missing(lower) & missing(upper) & missing(nBits)) {
        stop("A lower and upper range of values (for 'real-valued' or 'permutation') or nBits (for 'binary') must be provided!")
    }

    switch(type, binary = {
        nBits <- as.vector(nBits)[1]
        lower <- upper <- NA
        nvars <- nBits
        if (is.null(names)) names <- paste0("x", 1:nvars)
    }, `real-valued` = {
        lnames <- names(lower)
        unames <- names(upper)
        lower <- as.vector(lower)
        upper <- as.vector(upper)
        nBits <- NA
        if (length(lower) != length(upper))
          stop("lower and upper must be vector of the same length")
        #if ((length(lower) != nObj) & (length(upper) != nObj))
        # stop("The lower and upper limits must be vector of the same number of objectives")
        nvars <- length(upper)
        if (is.null(names) & !is.null(lnames)) names <- lnames
        if (is.null(names) & !is.null(unames)) names <- unames
        if (is.null(names))
          names <- paste0("x", 1:nvars)
    }, permutation = {
        lower <- as.vector(lower)[1]
        upper <- as.vector(upper)[1]
        nBits <- NA
        nvars <- length(seq.int(lower, upper))
        if (is.null(names))
          names <- paste0("x", 1:nvars)
    })

    if (is.null(suggestions)) {
        suggestions <- matrix(nrow = 0, ncol = nvars)
    } else {
        if (is.vector(suggestions)) {
            if (nvars > 1)
              suggestions <- matrix(suggestions, nrow = 1)
            else
              suggestions <- matrix(suggestions, ncol = 1)
        } else {
            suggestions <- as.matrix(suggestions)
        }
        if (nvars != ncol(suggestions))
            stop("Provided suggestions (ncol) matrix do not match number of variables of the problem")
    }

    # check monitor arg
    if (is.logical(monitor)) {
        if (monitor)
            monitor <- rmooMonitor
    }
    if (is.null(monitor))
        monitor <- FALSE

    # Start parallel computing (if needed)
    if(is.logical(parallel)){
      if(parallel) {
        parallel <- startParallel(parallel)
        stopCluster <- TRUE
      } else {
        parallel <- stopCluster <- FALSE
      }
    }else {
      stopCluster <- if(inherits(parallel, "cluster")) FALSE else TRUE
      parallel <- startParallel(parallel)
    }
    on.exit(if(parallel & stopCluster)
      stopParallel(attr(parallel, "cluster")))
    # define operator to use depending on parallel being TRUE or FALSE
    `%DO%` <- if(parallel && requireNamespace("doRNG", quietly = TRUE)){
      doRNG::`%dorng%` } else if (parallel){ foreach::`%dopar%` } else { foreach::`%do%` }

    # set seed for reproducibility
    if (!is.null(seed))
        set.seed(seed)

    i. <- NULL  #dummy to trick R CMD check

    Fitness <- matrix(NA, nrow = popSize, ncol = nObj)

    fitnessSummary <- vector("list", maxiter)

    # Creacion del objetivo tipo nsga
    object <- new("nsga2",
        call = call,
        type = type,
        lower = lower,
        upper = upper,
        nBits = nBits,
        names = if (is.null(names))
                  character()
                else names,
        popSize = popSize,
        front = matrix(),
        f = list(),
        iter = 0,
        run = 1,
        maxiter = maxiter,
        suggestions = suggestions,
        population = matrix(),
        pcrossover = pcrossover,
        pmutation = if (is.numeric(pmutation))
                      pmutation
                    else NA,
        crowdingDistance = matrix(),
        fitness = Fitness,
        summary = fitnessSummary)

    # Generate initial population
    if (maxiter == 0)
        return(object)

    p_fit <- q_fit <- matrix(NA_real_, nrow = popSize, ncol = nObj)
    switch(type, binary = {
        Pop <- P <- Q <- matrix(NA_real_, nrow = popSize, ncol = nBits)
    }, `real-valued` = {
        Pop <- P <- Q <- matrix(NA_real_, nrow = popSize, ncol = nvars)
    }, permutation = {
        Pop <- P <- Q <- matrix(NA_real_, nrow = popSize, ncol = nvars)
    })

    ng <- min(nrow(suggestions), popSize)

    if (ng > 0) {
        Pop[1:ng, ] <- suggestions
    }
    if (popSize > ng) {
        Pop[(ng + 1):popSize, ] <- population(object)[1:(popSize - ng), ]
    }
    object@population <- Pop

    if(!parallel) {
      for (i in seq_len(popSize)) {
        if (is.na(Fitness[i])) {
          fit <- do.call(fitness, c(list(Pop[i, ]), callArgs))
          Fitness[i, ] <- fit
        }
      }
    } else {
      Fitness <- foreach(i. = seq_len(popSize), .combine = "rbind") %DO%
        { if(is.na(Fitness[i.]))
          do.call(fitness, c(list(Pop[i.,]), callArgs))
          else
            Fitness[i.,]
        }
    }

    object@population <- P <- Pop
    object@fitness <- p_fit <- Fitness

    # First Non-dominated Ranking
    out <- non_dominated_fronts(object)
    object@f <- out$fit
    object@front <- matrix(unlist(out$fronts), ncol = 1, byrow = TRUE)
    object@crowdingDistance <- matrix(NA_real_, nrow = popSize)

    for (iter in seq_len(maxiter)) {
        object@iter <- iter

        # Selection Operator
        if (is.function(selection)) {
            sel <- selection(object, nObj)
            Pop <- sel$population
            Fitness <- sel$fitness
        } else {
            sel <- sample(1:popSize, size = popSize, replace = TRUE)
            Pop <- object@population[sel, ]
            Fitness <- object@fitness[sel, ]
        }
        object@population <- Pop
        object@fitness <- Fitness

        # Cross Operator
        if (is.function(crossover) & pcrossover > 0) {
            nmating <- floor(popSize / 2)
            mating <- matrix(sample(1:(2 * nmating), size = (2 * nmating)), ncol = 2)
            for (i in seq_len(nmating)) {
                if (pcrossover > runif(1)) {
                  parents <- mating[i, ]
                  Crossover <- crossover(object, parents)
                  Pop[parents, ] <- Crossover$children
                  Fitness[parents, ] <- Crossover$fitness
                }
            }
        }
        object@population <- Pop
        object@fitness <- Fitness

        # Mutation Operator
        pm <- if (is.function(pmutation)) {
            pmutation(object)
        } else {
            pmutation
        }
        if (is.function(mutation) & pm > 0) {
            for (i in seq_len(popSize)) {
                if (pm > runif(1)) {
                  Mutation <- mutation(object, i)
                  Pop[i, ] <- Mutation
                  Fitness[i, ] <- NA
                }
            }
        }
        object@population <- Q <- Pop
        object@fitness <- q_fit <- Fitness

        # Evaluate Fitness
        if(!parallel) {
          for (i in seq_len(popSize)) {
            if (is.na(Fitness[i])) {
              fit <- do.call(fitness, c(list(Pop[i, ]), callArgs))
              Fitness[i, ] <- fit
            }
          }
        } else {
          Fitness <- foreach(i. = seq_len(popSize), .combine = "rbind") %DO%
            { if(is.na(Fitness[i.]))
              do.call(fitness, c(list(Pop[i.,]), callArgs))
              else
                Fitness[i.,]
            }
        }

        object@population <- Q <- Pop
        object@fitness <- q_fit <- Fitness

        # R = P U Q
        object@population <- Pop <- rbind(P, Q)
        object@fitness <- rbind(p_fit, q_fit)

        out <- non_dominated_fronts(object)
        object@f <- out$fit
        object@front <- matrix(unlist(out$fronts), ncol = 1, byrow = TRUE)

        #NSGA-II Operator
        cd <- crowding_distance(object, nObj)
        object@crowdingDistance <- cd

        # Sorted population, crowding and fitness by front and crowding distance
        populationsorted <- object@population[order(object@front, -object@crowdingDistance), ]
        fitnesssorted <- object@fitness[order(object@front, -object@crowdingDistance), ]

        # Select de first N element
        object@population <- P <- Pop <- populationsorted[1:popSize, ]
        object@fitness <- p_fit <- fitnesssorted[1:popSize, ]

        out <- non_dominated_fronts(object)
        object@f <- out$fit
        object@front <- matrix(unlist(out$fronts), ncol = 1, byrow = TRUE)

        cd <- crowding_distance(object, nObj)
        object@crowdingDistance <- cd

        if (summary == TRUE) {
          fitnessSummary[[iter]] <- progress(object, callArgs)
          object@summary <- fitnessSummary
        } else {
          object@summary <- list(NULL)
        }

        # Plot front non-dominated by iteration
        if (is.function(monitor)) {
          monitor(object = object, callArgs)
            # monitor(object = object, number_objective = nObj)

        }

        if (max(Fitness, na.rm = TRUE) >= maxFitness)
            break
        if (object@iter == maxiter)
            break
    }

    solution <- object

    return(solution)
}

## NSGA-II Bare Process
# @export
nsga_ii <- function(object, nObj) {
  fitness <- t(object@fitness)
  popSize <- object@popSize
  new.pop.idxs <- integer()

  max.rank = max(object@front)

  idxs.by.rank = object@f

  front.len <- sapply(idxs.by.rank, length)
  cum.front.len <- cumsum(front.len)

  front.first.nonfit <- which.first(cum.front.len > popSize)

  if (front.first.nonfit > 1L) {
    new.pop.idxs <- unlist(idxs.by.rank[1:(front.first.nonfit - 1L)])
  }

  n.diff = popSize - length(new.pop.idxs)

  if (n.diff > 0L) {
    idxs.first.nonfit <- idxs.by.rank[[front.first.nonfit]]
    cds <- ecr::computeCrowdingDistance(fitness[, idxs.first.nonfit, drop = FALSE])
    idxs2 <- order(cds, decreasing = TRUE)[1:n.diff]
    new.pop.idxs <- c(new.pop.idxs, idxs.first.nonfit[idxs2])
  }

  object@population <- p_pop <- object@population[new.pop.idxs,]
  object@fitness <- p_fit <- object@fitness[new.pop.idxs,]

  out <- non_dominated_fronts(object)
  object@f <- out$fit
  object@front <- matrix(unlist(out$fronts), ncol = 1, byrow = TRUE)

  object@crowdingDistance <- crowding_distance(object, nObj)
  # object@crowdingDistance <- ecr::computeCrowdingDistance(t(object@fitness))

  out <- list(object = object,
              p_pop = p_pop,
              p_fit = p_fit)

  return(out)
}
# nsga_ii <- function(object, nObj) {
#   cd <- crowding_distance(object, nObj)
#   object@crowdingDistance <- cd
#
#   populationsorted <- object@population[order(object@front, -object@crowdingDistance), ]
#   fitnesssorted <- object@fitness[order(object@front, -object@crowdingDistance), ]
#
#   object@population <- P <- Pop <- populationsorted[1:object@popSize, ]
#   object@fitness <- p_fit <- fitnesssorted[1:object@popSize, ]
#
#   out <- non_dominated_fronts(object)
#   object@f <- out$fit
#   object@front <- matrix(unlist(out$fronts), ncol = 1, byrow = TRUE)
#
#   cd <- crowding_distance(object, nObj)
#   object@crowdingDistance <- cd
#
#   out <- list(object = object,
#               p_pop = Pop,
#               p_fit = p_fit)
#
#   return(out)
# }

# @export
#' @rdname progress-methods
#' @aliases progress,nsga2-method
setMethod("progress", "nsga2", .nsga2.progress)

# @export
#' @rdname plot-methods
#' @aliases plot,nsga2-method
setMethod("plot", signature(x="nsga2", y="missing"), .get.plotting)

# @export
#' @rdname getCrowdingDistance-methods
#' @aliases getCrowdingDistance,nsga2-method
setMethod("getCrowdingDistance", "nsga2", function(obj) print(obj@crowdingDistance))

# @export
#' @rdname summary-methods
#' @aliases summary,nsga2-method
setMethod("summary", "nsga2",
          function(object, ...){
            callArgs <- list(...)
            nullRP <- is.null(callArgs$reference_dirs)

            # Calculate information for summary

            first <- object@f[[1]]
            first_front_fit <-
              first_front_pop <- object@population[first, ]
            nadir_point <- apply(object@fitness[first, ], 2, max)

            #first_dum <- object@dumFitness[first, ] for nsga1 summary method

            if("ecr" %in% rownames(utils::installed.packages())){
              if (nullRP) {
                cat("Warning! \nReference points not provided:\n
                      value necessary to evaluate GD and IGD.")

              } else{
                gd <- ecr::computeGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
                igd <- ecr::computeInvertedGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
              }
            }

            if("emoa" %in% rownames(utils::installed.packages())){
              if(nullRP) {
                cat("\nUsing the maximum in each dimension to evaluate Hypervolumen")
                reference_point <- nadir_point
              } else {reference_point <- apply(callArgs$reference_dirs, 2, max)}
              hv <- emoa::dominated_hypervolume(points = t(object@fitness[first, ]), ref = reference_point)
            }

            cat("\nSummary of NSGA-II run")
            cat("\n#====================================")
            cat("\nNumber of Objectives evaluated: ", ncol(object@fitness))
            cat("\nTotal iterations: ", object@iter)
            cat("\nPopulation size: ", object@popSize)
            #cat("\nFeasible points found: ", nfeas,paste0("(", signif(100 * nfeas / npts, 3), "%"),"of total)")
            cat("\nNondominated points found: ", length(first),
                paste0("(", signif(100 * length(first) / object@popSize, 3), "%"),
                "of total)")
            cat("\nCrowding distance bounds: ", c(max(object@crowdingDistance), min(object@crowdingDistance)))
            #cat("\nEstimated nadir point: ", round(object@nadir_point, 3))
            cat("\nMutation Probability: ",
                paste0(signif(100 * object@pmutation, 3), "%"))
            cat("\nCrossover Probability: ",
                paste0(signif(100 * object@pcrossover, 3), "%"))
            if("ecr" %in% rownames(utils::installed.packages())){
              if(!nullRP) cat("\nEstimated IGD: ", igd)
              if(!nullRP) cat("\nEstimated GD: ", gd)
            } else cat("\n\nPlease install package 'ecr' to calculate IGD and GD.")
            if("emoa" %in% rownames(utils::installed.packages())) {
              cat("\nEstimated HV: ", hv)
              cat("\nRef point used for HV: ", reference_point)
            } else cat("\n\nPlease install package 'emoa' to calculate hypervolume.")
            cat("\n#====================================")
          }
)

#' @export
rmooMonitor <- function(object, ...) {
  if (!requireNamespace("rgl", quietly = TRUE)){
    stop("packages 'rgl' required for Monitor, please install it!")
  }
  # if(!all(requireNamespace("rgl", quietly = TRUE),
  #         requireNamespace("grDevices", quietly = TRUE))){
  #   stop("packages 'rgl' and 'grDevices' required for Monitor, please install it!")
  # }
  number_objectives <- ncol(object@fitness)
  fitness <- object@fitness
  iter <- object@iter
  cl <- grDevices::rainbow(object@popSize)

  if (number_objectives == 3) {
    rgl::plot3d(fitness)
    rgl::bgplot3d({
      plot.new()
      title(main = paste(class(object)[1], "Iter: ", iter), line = 3)
    })
    plot(seq(number_objectives), object@fitness[1, ],
      col = cl[1], type = "l", main = paste(class(object)[1],
        "Iter: ", iter),
      ylim = c(min(object@fitness), max(object@fitness)),
      xlab = "Objective_No", ylab = "Objective_Value")
    for (i in 2:(object@popSize)) {
      lines(seq(number_objectives),
        object@fitness[i, ], col = cl[i], type = "l")
    }

  } else if (number_objectives == 2) {
    first_front <- fitness[object@f[[1]], ]
    opar <- par('mar','xpd')
    on.exit(par(opar))
    par(mar = c(5.1, 4.1, 4.1, 8.1), xpd = TRUE)
    plot(fitness[, 1], fitness[, 2],
      col = "green", pch = 20, main = paste(class(object)[1],
        "Iter: ", iter),
      xlim = range(fitness[, 1]), ylim = range(fitness[, 2]),
      xlab = "f_1", ylab = "f_2")
    lines(first_front[, 1][order(first_front[, 1])],
      first_front[, 2][order(first_front[, 1])],
      xlim = range(first_front[, 1]),
      ylim = range(first_front[, 2]),
      xlab = "f_1", ylab = "f_2",
      col = "red", type = "l", pch = 12, main = "Pareto Front")
    legend("topright", inset = c(-0.8, 0),
      legend = c("Population", "Pareto Optimal"), pch = c(19, NA),
      title = "Values", lwd = c(NA, 2),
      col = c("green", "red"), y.intersp = 1.5)


  } else if (number_objectives > 3) {
    plot(seq(number_objectives), object@fitness[1, ],
      col = cl[1], type = "l", main = paste(class(object)[1],
        "Iter: ", iter),
      ylim = c(min(object@fitness), max(object@fitness)),
      xlab = "Objective_No", ylab = "Objective_Value")
    for (i in 2:(object@popSize)) {
      lines(seq(number_objectives), object@fitness[i, ], col = cl[i], type = "l")
    }

  }
}


.nsga1.progress <- function(object, ...) {
  callArgs <- as.list(...)

  nullRP <- is.null(callArgs$reference_dirs)

  first <- object@f[[1]]
  first_front_fit <- object@fitness[first, ]
  first_front_pop <- object@population[first, ]
  first_dum <- object@dumFitness[first, ]

  if("ecr" %in% rownames(utils::installed.packages())){
    if (!nullRP) {
      gd <- ecr::computeGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
      igd <- ecr::computeInvertedGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
    }
  }

  if("emoa" %in% rownames(utils::installed.packages())){
    if(!nullRP) {
      reference_point <- apply(callArgs$reference_dirs, 2, max)
      hv <- emoa::dominated_hypervolume(points = t(first_front_fit), ref = reference_point)
    }
  }

  if(nullRP) {
    result <- list(First_Front_Fit = first_front_fit,
                   First_Front_Pop = first_front_pop,
                   Dummy_Front_Fit = first_dum)

  } else{
    metric <- data.frame(Iternation = object@iter,
                         Generational_Distance = gd,
                         Inverse_Generational_Distance = igd,
                         Hypervolumen = hv)
    result <- list(First_Front_Fit = first_front_fit,
                   First_Front_Pop = first_front_pop,
                   Dummy_Front_Fit = first_dum,
                   metrics = metric)

  }
  return(invisible(result))
}

.nsga2.progress <- function(object, ...) {
  callArgs <- as.list(...)

  nullRP <- is.null(callArgs$reference_dirs)

  first <- object@f[[1]]
  first_front_fit <- object@fitness[first, ]
  first_front_pop <- object@population[first, ]
  first_cd <- object@crowdingDistance[first, ]

  if("ecr" %in% rownames(utils::installed.packages())){
    if (!nullRP) {
      gd <- ecr::computeGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
      igd <- ecr::computeInvertedGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
    }
  }
  if("emoa" %in% rownames(utils::installed.packages())){
    if(!nullRP) {
      reference_point <- apply(callArgs$reference_dirs, 2, max)
      hv <- emoa::dominated_hypervolume(points = t(first_front_fit), ref = reference_point)
    }
  }

  if(nullRP) {
    result <- list(First_Front_Fit = first_front_fit,
                   First_Front_Pop = first_front_pop,
                   Crowding_Dist = first_cd)

  } else{
    metric <- data.frame(Iternation = object@iter,
                         Generational_Distance = gd,
                         Inverse_Generational_Distance = igd,
                         Hypervolumen = hv)
    result <- list(First_Front_Fit = first_front_fit,
                   First_Front_Pop = first_front_pop,
                   Crowding_Dist = first_cd,
                   metrics = metric)

  }

  return(invisible(result))
}


.nsga3.progress <- function(object, ...) {
  callArgs <- as.list(...)
  reference_dirs <- callArgs$reference_dirs

  first <- object@f[[1]]
  first_front_fit <- object@fitness[first, ]
  first_front_pop <- object@population[first, ]
  ideal_point <- object@ideal_point
  worst_point <- object@worst_point
  extreme_points <- object@extreme_points

  if("ecr" %in% rownames(utils::installed.packages())){
    gd <- ecr::computeGenerationalDistance(t(object@fitness), t(reference_dirs))
    igd <- ecr::computeInvertedGenerationalDistance(t(object@fitness), t(reference_dirs))
  }
  if("emoa" %in% rownames(utils::installed.packages())){
    hv <- ecr::computeHV(t(object@fitness), ref.point = apply(reference_dirs, 2, max))
  }
  if (all((c("ecr", "emoa") %in% rownames(utils::installed.packages())))) {
    metric <- data.frame(Iternation = object@iter,
                         Generational_Distance = gd,
                         Inverse_Generational_Distance = igd,
                         Hypervolumen = hv)
    result <- list(first_front_fit = first_front_fit,
                   first_front_pop = first_front_pop,
                   ideal_point = ideal_point,
                   worst_point = worst_point,
                   extreme_points = extreme_points,
                   metrics = metric)
  } else{
    result <- list(first_front_fit = first_front_fit,
                   first_front_pop = first_front_pop,
                   ideal_point = ideal_point,
                   worst_point = worst_point,
                   extreme_points = extreme_points)
  }
  return(invisible(result))
}

.get.plotting <- function(x, y="missing",
                          type=c("scatter", "pcp", "heatmap", "polar"),
                          ...){
  type <- match.arg(type)

  switch(type,
         "scatter" = {
           scatter(x, ...)
         },
         "pcp"  = {
           pcp(x, ...)
         },
         "heatmap" = {
           heat_map(x, ...)
         },
         "polar" = {
           polar(x, ...)
         }
  )
}

scatter <- function(object, ...){
  if(!all(requireNamespace("reshape2", quietly = TRUE),
          #requireNamespace("ggplot2", quietly = TRUE),
          #requireNamespace("plotly", quietly = TRUE),
          #requireNamespace("grDevices", quietly = TRUE),
          requireNamespace("cdata", quietly = TRUE))){
    stop("packages 'reshape2' and 'cdata' required for scatter plotting!")
  }
  #algorithm <- class(object)[1]
  n_obj <- ncol(object@fitness)
  if (n_obj == 2){
    plotting_multi_objective(object, ...)
  } else if (n_obj == 3) {
    plotting_many_objective(object, ...)
  } else{
    plotting_pairwise(object,...)

  }
}

plotting_multi_objective <- function(object, ...) {
  callArgs <- list(...)
  algorithm <- class(object)[1]
  if (any("optimal" %in% names(callArgs))) {
    optimal <- callArgs$optimal
    name_optimal <- lapply(substitute(list(...))[-1], deparse)$optimal
    objective_values <- object@fitness
    colnames(objective_values) <- c("f_1", "f_2")
    colnames(optimal) <- c("f_1", "f_2")
    optimal <- as.data.frame(optimal)
    objective_values <- as.data.frame(objective_values)

    ggplot2::ggplot() + geom_line(data = optimal,
                                  aes(x = f_1,
                                      y = f_2,
                                      color = name_optimal)) +
      ggplot2::geom_point(data = objective_values,
                          aes(x = f_1,
                              y = f_2,
                              color = "Objective_Value")) +
      ggplot2::labs(title = paste(algorithm, "No Objective:", ncol(object@fitness)),
                    color = "Values") +
      ggplot2::scale_color_manual(labels = c("Objective_Value", name_optimal),
                                  values = c("green", "black"))

  } else {
    objective_values <- object@fitness
    colnames(objective_values) <- c("f_1", "f_2")
    objective_values <- as.data.frame(objective_values)

    ggplot2::ggplot() + ggplot2::geom_point(data = objective_values,
                                            aes(x = f_1,
                                                y = f_2,
                                                color = "Objective_Values")) +
      ggplot2::labs(title = paste(algorithm, "No Objective:", ncol(object@fitness)),
                    color = "Values") +
      ggplot2::scale_color_manual(labels = "Objective_Value",
                                  values = "black")
  }
}

plotting_many_objective <- function(object, ...) {
  callArgs <- list(...)
  algorithm <- class(object)[1]
  objective_values <- object@fitness
  if (any("optimal" %in% names(callArgs))) {
    optimal <- callArgs$optimal
    shapes <- c(rep(0, nrow(objective_values)), rep(1, nrow(optimal)))

    temp <- rbind(objective_values, optimal)
    temp <- cbind(temp,shapes)
    colnames(temp) <- c("f_1", "f_2", "f_3", "shapes")
    temp <- as.data.frame(temp)
    temp$shapes[which(temp$shapes == 0)] <- "circle"
    temp$shapes[which(temp$shapes == 1)] <- "square"
    temp$shapes <- as.factor(temp$shapes)

    fig <- plotly::plot_ly(temp, x = ~f_1,
                           y = ~f_2,
                           z = ~f_3,
                           color = ~shapes,
                           colors = c('#0C4B8E', '#BF382A'),
                           size = 1)
    fig <- fig %>% plotly::add_markers()
    fig <- fig %>% plotly::layout(scene = list(xaxis = list(title = 'f_1'),
                                               yaxis = list(title = 'f_2'),
                                               zaxis = list(title = 'f_3')))
    fig
  } else {
    colnames(objective_values) <- c("f_1", "f_2", "f_3")
    objective_values <- as.data.frame(objective_values)

    fig <- plotly::plot_ly(objective_values, x = ~f_1,
                           y = ~f_2,
                           z = ~f_3,
                           colors = '#0C4B8E', size = 1)
    fig <- fig %>% plotly::add_markers() #New plotly add
    fig <- fig %>% plotly::layout(scene = list(xaxis = list(title = 'f_1'),
                                               yaxis = list(title = 'f_2'),
                                               zaxis = list(title = 'f_3')))
    fig
  }
}

plotting_pairwise <- function(object, ...){
  fit <- as.data.frame(object@fitness)
  nObj <- ncol(object@fitness)
  algorithm <- class(object)[1]
  colnames(fit) <- sprintf("Obj_%s",seq(nObj))
  meas_vars <- colnames(fit)

  controlTable <- data.frame(expand.grid(meas_vars, meas_vars,
                                         stringsAsFactors = FALSE))

  colnames(controlTable) <- c("x", "y")

  controlTable <- cbind(data.frame(pair_key = paste(controlTable[[1]],
                                                    controlTable[[2]]),
                                   stringsAsFactors = FALSE),
                        controlTable)

  fit_aug <- cdata::rowrecs_to_blocks(fit, controlTable)

  splt <- strsplit(fit_aug$pair_key, split = " ", fixed = TRUE)
  fit_aug$columns <- vapply(splt, function(si) si[[1]], character(1))
  fit_aug$rows <- vapply(splt, function(si) si[[2]], character(1))

  fit_aug$columns <- factor(as.character(fit_aug$columns), meas_vars)
  fit_aug$rows <- factor(as.character(fit_aug$rows), meas_vars)

  ggplot2::ggplot(fit_aug, aes(x=x, y=y)) +
    ggplot2::geom_point(aes(color=rows, shape=columns)) +
    ggplot2::facet_grid(rows~columns,
                        labeller = label_value,
                        scale = "free") +
    ggplot2::ggtitle(paste(algorithm, "No Objective:", nObj)) +
    ggplot2::scale_shape_manual(values=seq(nObj)) +
    ggplot2::scale_fill_manual(values = grDevices::rainbow(nObj)) +
    ggplot2::theme(axis.text.x = element_text(angle = 90)) +
    ggplot2::ylab(NULL) +
    ggplot2::xlab(NULL)
}

pcp <- function(object) {

  if (!requireNamespace("reshape2", quietly = TRUE)){
    stop("packages 'reshape2' required for pcp plotting!")
  }

  # if(!all(requireNamespace("ggplot2", quietly = TRUE),
  #         requireNamespace("grDevices", quietly = TRUE),
  #         requireNamespace("reshape2", quietly = TRUE),
  #         requireNamespace("dplyr", quietly = TRUE)))
  #   stop("packages 'ggplot2', 'dplyr' and 'reshape2' required for pcp plotting!")
  nObj <- ncol(object@fitness)
  colnames(object@fitness) <- sprintf("f_%s",seq(nObj))
  fitness <- reshape2::melt(object@fitness)
  fitness$color <- grDevices::rainbow(object@popSize)
  fitness <- dplyr::rename(fitness,
                           'Objective_Value' = value,
                           'Objective_No' = Var2)
  ggplot2::ggplot(fitness, aes(x = Objective_No,
                               y = Objective_Value,
                               group = Var1, colour=factor(color))) +
    ggplot2::geom_line(show.legend = FALSE) +
    ggplot2::theme_classic()
}

heat_map <- function(object, ...){
  if (!requireNamespace("reshape2", quietly = TRUE)){
    stop("packages 'reshape2' required for heat map plotting!")
  }
  # if(!all(requireNamespace("ggplot2", quietly = TRUE),
  #         requireNamespace("reshape2", quietly = TRUE),
  #         requireNamespace("dplyr", quietly = TRUE)))
  #   stop("packages 'ggplot2', 'dplyr' and 'reshape2' required for heat map plotting!")
  callArgs <- list(...)
  individual <- callArgs$individual
  fitness <- object@fitness

  if(is.null(individual))
    stop("Please, define a vector with the individuals to plot")

  if (length(individual) > 10) {
    cat("Warning! Heatmap plot with more than 10 individuals will not be displayed correctly.\n
                  The plot will still be displayed.")
  }

  fitness <- fitness[individual,]

  if (is.null(dim(fitness))){
    fitness <- t(matrix(fitness))
  }

  nObj <- ncol(fitness)
  colnames(fitness) <- sprintf("f_%s",seq(nObj))
  fitness <- reshape2::melt(fitness)
  fitness <- dplyr::rename(fitness,
                           'Pop' = Var1,
                           'Objective_Value' = value,
                           'Objective_No' = Var2)
  ggplot2::ggplot(fitness, aes(x = Objective_No,
                               y = Pop,
                               fill = Objective_Value)) +
    ggplot2::geom_raster() +
    ggplot2::scale_y_continuous(labels = unique(as.character(fitness$Pop)),
                                breaks = unique(fitness$Pop))
}

polar <- function(object, ...){
  if (!requireNamespace("reshape2", quietly = TRUE)){
    stop("packages 'reshape2' required for polar coordinate plotting!")
  }
  # if(!all(requireNamespace("ggplot2", quietly = TRUE),
  #         requireNamespace("reshape2", quietly = TRUE),
  #         requireNamespace("dplyr", quietly = TRUE)))
  #   stop("packages 'ggplot2', 'dplyr' and 'reshape2' required for polar coordinate plotting!")

  callArgs <- list(...)
  individual <- callArgs$individual
  fitness <- object@fitness

  if(is.null(individual))
    stop("Please, define a vector with the individuals to plot")

  if (length(individual) > 10) {
    cat("Warning! Polar Coordinate plot with more than 10 individuals will not be displayed correctly.\n
                  The plot will still be displayed.")
  }

  fitness <- fitness[individual,]

  if (is.null(dim(fitness))){
    fitness <- t(matrix(fitness))
  }

  nObj <- ncol(fitness)
  colnames(fitness) <- sprintf("f_%s",seq(nObj))
  fitness <- reshape2::melt(fitness)
  fitness <- dplyr::rename(fitness,
                           'Pop' = Var1,
                           'Objective_Value' = value,
                           'Objective_No' = Var2)
  ggplot2::ggplot(fitness, aes(x = Objective_No,
                               y = Objective_Value,
                               fill = Objective_No)) +
    ggplot2::geom_bar(width = 1, stat="identity") +
    ggplot2::coord_polar() + ggplot2::theme_light() +
    ggplot2::facet_wrap(~Pop, nrow = 1)
}

utils::globalVariables(c("Pop","Objective_No", "Objective_Value", "Var1", "Var2", "color", "columns", "f_1", "f_2", "label_both", "rows", "value", "x", "y", "i.", "%DO%"))




# @export
# summary.rmoo <- function(object, ...) {
#   algorithm <- class(object)[1]
#   if(algorithm == "nsga1"){
#     out <- nsgaSummary(object)
#   }
#   else if(algorithm == "nsga2"){
#     out <- nsgaiiSummary(object)
#   }
#   else if(algorithm == "nsga3"){
#     out <- nsgaiiiSummary(object)
#   }
#   #class(out) <- "summary"
#   return(out)
# }

#colMax <- function(data) apply(data, 2, max)

#Summary for NSGA Algorithm
# nsgaSummary <- function(object, ...) {
#   # Calculate information for summary
#   callArgs <- list(...)
#
#   nullRP <- is.null(callArgs$reference_dirs)
#
#   first <- object@f[[1]]
#   first_front_fit <- object@fitness[first, ]
#   first_front_pop <- object@population[first, ]
#   first_dum <- object@dumFitness[first, ]
#
#   if("ecr" %in% rownames(utils::installed.packages())){
#     if (!nullRP) {
#       gd <- ecr::computeGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
#       igd <- ecr::computeInvertedGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
#     }
#   }
#
#   if("emoa" %in% rownames(utils::installed.packages())){
#     if(!nullRP) {
#       reference_point <- apply(callArgs$reference_dirs, 2, max)
#       hv <- emoa::dominated_hypervolume(points = t(first_front_fit), ref = reference_point)
#     }
#   }
#
#   if(nullRP) {
#     result <- list(`First Front Fit` = first_front_fit,
#                    `First Front Pop` = first_front_pop,
#                    `Dummy Front Fit` = first_dum)
#
#   } else{
#     result <- list(`First Front Fit` = first_front_fit,
#                    `First Front Pop` = first_front_pop,
#                    `Dummy Front Fit` = first_dum,
#                    Hypervolumen = hv,
#                    `Generational Distance` = gd,
#                    `InvertedGenerational Distance` = igd)
#
#   }
#   return(result)
# }
#
# #Summary for NSGA-II Algorithm
# nsgaiiSummary <- function(object, ...) {
#   # Calculate information for summary
#   callArgs <- list(...)
#
#   nullRP <- is.null(callArgs$reference_dirs)
#
#   first <- object@f[[1]]
#   first_front_fit <- object@fitness[first, ]
#   first_front_pop <- object@population[first, ]
#   first_cd <- object@crowdingDistance[first, ]
#
#   if("ecr" %in% rownames(utils::installed.packages())){
#     if (!nullRP) {
#       gd <- ecr::computeGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
#       igd <- ecr::computeInvertedGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
#     }
#   }
#   if("emoa" %in% rownames(utils::installed.packages())){
#     if(!nullRP) {
#       reference_point <- apply(callArgs$reference_dirs, 2, max)
#       hv <- emoa::dominated_hypervolume(points = t(first_front_fit), ref = reference_point)
#     }
#   }
#
#   if(nullRP) {
#     result <- list(`First Front Fit` = first_front_fit,
#                    `First Front Pop` = first_front_pop,
#                    `Crowding Dist` = first_cd)
#
#   } else{
#     result <- list(`First Front Fit` = first_front_fit,
#                    `First Front Pop` = first_front_pop,
#                    `Crowding Dist` = first_cd,
#                    Hypervolumen = hv,
#                    `Generational Distance` = gd,
#                    `InvertedGenerational Distance` = igd)
#
#   }
#
#   return(result)
# }
#
# #Summary for NSGA-III Algorithm
# nsgaiiiSummary <- function(object, ...) {
#   # Calculate information for summary
#   first <- object@f[[1]]
#   first_front_fit <- object@fitness[first, ]
#   first_front_pop <- object@population[first, ]
#   ideal_point <- object@ideal_point
#   worst_point <- object@worst_point
#   extreme_points <- object@extreme_points
#
#   if("ecr" %in% rownames(utils::installed.packages())){
#       gd <- ecr::computeGenerationalDistance(t(object@fitness), t(object@reference_points))
#       igd <- ecr::computeInvertedGenerationalDistance(t(object@fitness), t(object@reference_points))
#   }
#   if("emoa" %in% rownames(utils::installed.packages())){
#       hv <- ecr::computeHV(t(object@fitness), ref.point = apply(object@reference_points, 2, max))
#   }
#   if (all((c("ecr", "emoa") %in% rownames(utils::installed.packages())))) {
#     metric <- data.frame(Iternation = object@iter,
#                          Generational_Distance = gd,
#                          Inverse_Generational_Distance = igd,
#                          Hypervolumen = hv)
#     result <- list(first_front_fit = first_front_fit,
#                    first_front_pop = first_front_pop,
#                    ideal_point = ideal_point,
#                    worst_point = worst_point,
#                    extreme_points = extreme_points,
#                    metrics = metric)
#   } else{
#     result <- list(first_front_fit = first_front_fit,
#                    first_front_pop = first_front_pop,
#                    ideal_point = ideal_point,
#                    worst_point = worst_point,
#                    extreme_points = extreme_points)
#   }
#   return(result)
# }

# Association Operator

# Association process of each member of the population with the reference points.

# The association operator will work with the normalized space, reference points
# and population members. First, generating the reference lines corresponding to
# each reference point and then calculating the perpendicular distance of each
# population member from each reference line.
# This code section corresponds to Algorithm 3 of the referenced paper.

#' @export
associate_to_niches <- function(object, utopian_epsilon = 0) {
  fitness      <- object@fitness
  ideal_point  <- object@ideal_point
  niches       <- object@reference_points
  nadir_point  <- object@nadir_point

  utopian_point <- ideal_point - utopian_epsilon
  denom         <- nadir_point - utopian_point
  denom[denom == 0] <- 1e-12

  delta <- sweep(fitness, 2, utopian_point)
  N     <- sweep(delta,   2, denom, FUN = "/")

  # cos(θ) between each normalised individual and each reference direction
  dist_matrix <- 1 - compute_perpendicular_distance(N, niches)

  # Use the C++ function to compute dist_matrix
  # dist_matrix <- 1 - compute_perpendicular_distance_cpp(N, niches)

  # Perpendicular distance = ‖fitness‖ · sin(θ)
  fitness_norms <- sqrt(rowSums(fitness^2))

  norm_matrix <- matrix(fitness_norms,
                        nrow = nrow(fitness),
                        ncol = nrow(niches))

  dist_matrix <- norm_matrix * sqrt(pmax(1 - dist_matrix^2, 0))

  # Replace vapply + safe_which_min loop with vectorised max.col.
  # Handle all-NaN rows by substituting Inf so max.col still picks column 1.
  safe_dist <- dist_matrix
  safe_dist[is.nan(safe_dist)] <- Inf
  niche_of_individuals <- max.col(-safe_dist, ties.method = "first")

  dist_to_niche <- apply(dist_matrix, 1, min)
  # niche_of_individuals <- apply(dist_matrix, 1, which.min)

  safe_which_min <- function(x) {
    idx <- which.min(x)
    if (length(idx) == 0L) 1L else idx
  }

  niche_of_individuals <- vapply(seq_len(nrow(dist_matrix)),
                                 function(i) safe_which_min(dist_matrix[i, ]),
                                 integer(1L))

  dist_to_niche <- dist_matrix[cbind(seq_along(niche_of_individuals),
                                     niche_of_individuals)]

  list(distance = dist_to_niche, niches = niche_of_individuals)
}
# associate_to_niches <- function(object, utopian_epsilon = 0) {
#     fitness <- object@fitness
#     ideal_point <- object@ideal_point
#     niches <- object@reference_points
#     nadir_point <- object@nadir_point
#     utopian_point <- ideal_point - utopian_epsilon
#     denom <- nadir_point - utopian_point
#     denom[which(denom == 0)] <- 1 * 10^(-12)
#
#     delta <- sweep(fitness, 2, utopian_point)
#     N <- sweep(delta, 2, denom, FUN = "/")
#     dist_matrix <- 1 - compute_perpendicular_distance(N, niches)
#     dist_matrix <- do.call(cbind,
#                            replicate(nrow(niches),
#                                      sqrt(rowSums(fitness^2)),
#                                      simplify = FALSE)) * sqrt(1 - dist_matrix^2)
#     niche_of_individuals <- dist_to_niche <- c()
#     niche_of_individuals <- apply(dist_matrix, 1, which.min)
#
#     # write.csv(fitness,file="fitness.csv")
#     # write.csv(N,file="N.csv")
#     # write.csv(niches,file="niches.csv")
#
#     for (i in 1:nrow(fitness)) {
#       # write.csv(dist_matrix,file="dist_matrix.csv")
#       # write.csv(i,file="i.csv")
#       # write.csv(niche_of_individuals[i],file="niche_of_individualsi.csv")
#       # write.csv(niche_of_individuals,file="niche_of_individuals.csv")
#       dist_to_niche[i] <- dist_matrix[i, niche_of_individuals[i]]
#     }
#
#     out <- list(distance = dist_to_niche, niches = niche_of_individuals)
#     return(out)
# }


#' @export
compute_perpendicular_distance <- function(x, y) {
  x <- x / sqrt(rowSums(x^2))
  y <- y / sqrt(rowSums(y^2))
  1 - x %*% t(y)
}
# compute_perpendicular_distance <- function(x, y) {
#     p <- ncol(x)
#     xx <- sqrt(rowSums(x^2))
#     x <- x / matrix(rep(xx, p), ncol = p)
#     yy <- sqrt(rowSums(y^2))
#     y <- y / matrix(rep(yy, p), ncol = p)
#     D = 1 - x %*% t(y)
#     return( D)
# }

# cppFunction('NumericMatrix compute_perpendicular_distance_cpp(NumericMatrix x, NumericMatrix y) {
#   int p = x.ncol();
#   int n = x.nrow();
#   int m = y.nrow();
#   NumericMatrix D(n, m);
#
#   for (int i = 0; i < n; i++) {
#     for (int j = 0; j < m; j++) {
#       double sum = 0.0;
#       for (int k = 0; k < p; k++) {
#         sum += x(i, k) * y(j, k);
#       }
#       D(i, j) = 1 - sum;
#     }
#   }
#
#   return D;
# }')

#' @export
compute_niche_count <- function(n_niches, niche_of_individuals) {
  tabulate(niche_of_individuals, nbins = n_niches)    # drop redundant return()
}
# compute_niche_count <- function(n_niches, niche_of_individuals) {
#     niche_count <- rep(0, n_niches)
#     test <- split(seq_along(niche_of_individuals), niche_of_individuals)
#     a <- rbind(as.numeric(names(test)), unlist(lapply(test, length), use.names = FALSE))
#     niche_count[a[1, ]] <- a[2, ]
#     return(niche_count)
# }

context("Executing compute_perpendicular_distance")

test_that("compute_perpendicular_distance() return the value suggestions", {
  x <- matrix(rep(3,30), nrow = 10)
  y <- matrix(rep(1,3), nrow = 1)

  out <- compute_perpendicular_distance(x,y)

  expect_identical(colSums(out), 0)
})

#' Virtual Class 'numberOrNAOrMatrix - Simple Class for subassigment Values'
#'
#' The class 'numberOrNAOrMatrix' is a simple class union ([setClassUnion()])
#' of 'numeric', 'logical', 'logical' and 'matrix'.
#'
#' @section Objects from the Class:
#' Since it is a virtual Class, no objects may be created from it.
#'
#' @examples
#' showClass('numberOrNAOrMatrix')

# A new class is created with the union of numeric, logical and matrix
#' @export
setClassUnion("numberOrNAOrMatrix",  members = c("numeric", "logical", "matrix", "NULL"))


#' Virtual Parent Class Algorithm
#'
#' It will use when other algorithms are implemented. Equivalent to a Abstract
#' class in other languages.
#' @export
setClass("algorithm", contains = "VIRTUAL")

#' Virtual Class 'nsga'
#'
#' The 'nsga' class is the parent superclass of the \linkS4class{nsga1},
#' \linkS4class{nsga2},  and \linkS4class{nsga3} classes
#'
#' @section Objects from the Class:
#' Since it is a virtual Class, no objects may be created from it.
#'
#' @slot call an object of class 'call' representing the matched call.
#' @slot type a character string specifying the type of genetic algorithm used.
#' @slot lower a vector providing for each decision variable the lower bounds of
#' the search space in case of real-valued or permutation encoded optimisations.
#' @slot upper a vector providing for each decision variable the upper bounds of
#' the search space in case of real-valued or permutation encoded optimizations.
#' @slot nBits a value specifying the number of bits to be used in binary
#' encoded optimizations.
#' @slot names a vector of character strings providing the names of decision
#' variables (optional).
#' @slot nvars a
#' @slot popSize the population size.
#' @slot front Rank of individuals on the non-dominated front.
#' @slot f Front of individuals on the non-dominated front.
#' @slot iter the actual (or final) iteration of NSGA search.
#' @slot run the number of consecutive generations without any improvement in
#' the best fitness value before the NSGA is stopped.
#' @slot maxiter the maximum number of iterations to run before the NSGA search
#' is halted.
#' @slot suggestions a matrix of user provided solutions and included in the
#' initial population.
#' @slot population the current (or final) population.
#' @slot pcrossover the crossover probability.
#' @slot pmutation the mutation probability.
#' @slot fitness the values of fitness function for the current (or final)
#' population.
#' @slot summary a matrix of summary statistics for fitness values at each
#' iteration (along the rows).
#' @slot fitnessValue the best fitness value at the final iteration.
#' @slot solution the value(s) of the decision variables giving the best fitness
#' at the final iteration.
#' @slot execution_time a
#'
#' @examples
#' showClass('nsga')
#' @export
setClass(Class = "nsga",
         slots = list(call = "language",
                     type = "character",
                     lower = "numberOrNAOrMatrix",
                     upper = "numberOrNAOrMatrix",
                     nBits = "numberOrNAOrMatrix",
                     nvars = "numberOrNAOrMatrix",
                     names = "character",
                     popSize = "numeric",
                     front = "numberOrNAOrMatrix",
                     f = "list",
                     iter = "numeric",
                     run = "numeric",
                     maxiter = "numeric",
                     suggestions = "matrix",
                     population = "numberOrNAOrMatrix",
                     pcrossover = "numeric",
                     pmutation = "numberOrNAOrMatrix",
                     fitness = "numberOrNAOrMatrix",
                     summary = "list",
                     fitnessValue = "numberOrNAOrMatrix",
                     solution = "matrix",
                     execution_time = "numberOrNAOrMatrix"),
         contains = c("VIRTUAL", "algorithm"),
         #contains = "algorithm"
)


#' Class 'nsga1'
#'
#' The class 'nsga1' is instantiated within the execution of rmoo and will be
#' returned as a result of it. All data generated during execution will be
#' stored in it.
#'
#' @slot dumFitness a large dummy fitness value assigned to individuals from
#' the nondominated front.
#' @slot dShare the maximun phenotypic distance allowed between any two
#' individuals to become members of a niche.
#' @slot deltaDummy value to decrease the dummy fitness of individuals by
#' non-dominated fronts.
#'
#' @examples
#' showClass('nsga1')
#' @export
setClass(Class = "nsga1",
         slots = list(dumFitness = "numberOrNAOrMatrix",
                      dShare = "numeric",
                      deltaDummy = "numeric"),
         contains = "nsga"
)

#' Class 'nsga2'
#'
#' The class 'nsga2' is instantiated within the execution of rmoo and will be
#' returned as a result of it. All data generated during execution will be
#' stored in it.
#'
#' @slot crowdingDistance Crowding-comparison approach to estimate of the
#' perimeter of the cuboid formed by using the nearest neighbors as the vertices.
#'
#' @examples
#' showClass('nsga2')
#' @export
setClass(Class = "nsga2",
         slots = list(crowdingDistance = "numberOrNAOrMatrix"),
         contains = "nsga"
)


#' Class 'rnsga2'
#'
#' The class 'rnsga2' is instantiated within the execution of rmoo and will be
#' returned as a result of it. All data generated during execution will be
#' stored in it.
#'
#' @slot crowdingDistance Crowding-comparison approach to estimate of the
#' perimeter of the cuboid formed by using the nearest neighbors as the vertices.
#' @slot reference_points R-NSGA-II uses a set of reference points defined by the user to
#' ensure diversity in obtained solutions.
#' @slot extreme_points are selected using the ASF in the ([PerformScalarizing()]).
#' Necessary in the  nadir point generation.
#' @slot smin Index used to obtain the extreme points.
#'
#' @examples
#' showClass('rnsga2')
#' @export
setClass(Class = "rnsga2",
         slots = list(crowdingDistance = "numberOrNAOrMatrix",
                      reference_points = "numberOrNAOrMatrix",
                      extreme_points = "numberOrNAOrMatrix",
                      smin = "numberOrNAOrMatrix"),
         contains = "nsga2")


#' Class 'nsga3'
#'
#' The class 'nsga3' is instantiated within the execution of rmoo and will be
#' returned as a result of it. All data generated during execution will be
#' stored in it.
#'
#' @slot ideal_point Nadir point estimate used as lower bound in normalization.
#' @slot worst_point Worst point generated over generations.
#' @slot smin Index used to obtain the extreme points.
#' @slot extreme_points are selected using the ASF in the ([PerformScalarizing()]).
#' Necessary in the  nadir point generation.
#' @slot worst_of_population The worst individuals generated by objectives in
#' the current generation.
#' @slot worst_of_front The worst individuals in the first front generated by
#' objectives in the current generation.
#' @slot nadir_point Nadir point estimate used as upper bound in normalization.
#' @slot reference_points NSGA-III uses a predefined set of reference points to
#' ensure diversity in obtained solutions.
#' The chosen refenrece points can be predefined in structured manner or
#' supplied by the user. We use the Das and Dennis procedure.
#'
#' @examples
#' showClass('nsga3')
#' @export
setClass(Class = "nsga3",
         slots = list(ideal_point = "numberOrNAOrMatrix",
                      worst_point = "numberOrNAOrMatrix",
                      smin = "numberOrNAOrMatrix",
                      extreme_points = "numberOrNAOrMatrix",
                      worst_of_population = "numberOrNAOrMatrix",
                      worst_of_front = "numberOrNAOrMatrix",
                      nadir_point = "numberOrNAOrMatrix",
                      reference_points = "numberOrNAOrMatrix"),
         contains = "nsga"
)

# Agregar a la clases NSGA
# Atributos necesarios para el elitismo de las soluciones
# p_fit, q_fit
# p_pop, q_pop


# Agregar a la clases R-NSGA-II
# smin

#' Reference Point Based Non-Dominated Sorting in Genetic Algorithms II
#'
#' Minimization of a fitness function using reference point based non-dominated
#' sorting genetic algorithms - II (R-NSGA-IIs). Multiobjective evolutionary algorithms
#'
#' R-NSGA-II is a meta-heuristic proposed by K. Deb and J. Sundar in 2006.
#' It is a modification of NSGA-II based on reference points in which the
#' decision-maker supplies one or more preference points and a weight vector
#' that will guide the solutions towards regions desired by the user.
#'
#'
#' @param type the type of genetic algorithm to be run depending on the nature
#' of decision variables. Possible values are:
#' \describe{
#'     \item{'binary'}{for binary representations of decision variables.}
#'     \item{'real-valued'}{for optimization problems where the decision
#'     variables are floating-point representations of real numbers.}
#'     \item{'permutation'}{for problems that involves reordering of a list of
#'     objects.}
#' }
#'
#' @param fitness the fitness function, any allowable R function which takes as
#' input an individual string representing a potential solution, and returns a
#' numerical value describing its 'fitness'.
#' @param ... additional arguments to be passed to the fitness function. This
#' allows to write fitness functions that keep some variables fixed during the
#' search
#' @param lower a vector of length equal to the decision variables providing the
#' lower bounds of the search space in case of real-valued or permutation
#' encoded optimizations.
#' @param upper a vector of length equal to the decision variables providing the
#' upper bounds of the search space in case of real-valued or permutation
#' encoded optimizations.
#' @param nBits a value specifying the number of bits to be used in binary
#' encoded optimizations
#' @param population an R function for randomly generating an initial population.
#' See [rmoo_Population()] for available functions.
#' @param selection an R function performing selection, i.e. a function which
#' generates a new population of individuals from the current population
#' probabilistically according to individual fitness.
#' @param crossover an R function performing crossover, i.e. a function which
#' forms offsprings by combining part of the genetic information from their
#' parents. See [rmoo_Crossover()] for available functions.
#' @param mutation an R function performing mutation, i.e. a function which
#' randomly alters the values of some genes in a parent chromosome.
#' See [rmoo_Mutation()] for available functions.
#' @param reference_dirs Function to generate reference points using Das and
#' Dennis approach or matrix with supplied reference points.
#' @param epsilon controls the extent of obtained solutions by grouping all
#' solutions that have a normalized difference sum in objective values of epsilon or less.
#' @param normalization of the ideal points and nadir. They can be:
#' \describe{
#'     \item{'ever'}{.}
#'     \item{'front'}{.}
#'     \item{'no'}{.}
#' }
#'
#' @param extreme_points_as_ref_dirs flag to use extreme points as reference points.
#' @param weights vector specifies the importance of one objective function over
#' the other, by default all objectives have equal weights.
#' @param popSize the population size.
#' @param nObj number of objective in the fitness function.
#' @param pcrossover the probability of crossover between pairs of chromosomes.
#' Typically this is a large value and by default is set to 0.8.
#' @param pmutation the probability of mutation in a parent chromosome. Usually
#' mutation occurs with a small probability, and by default is set to 0.1.
#' @param maxiter the maximum number of iterations to run before the NSGA search
#' is halted.
#' @param run the number of consecutive generations without any improvement in
#' the best fitness value before the NSGA is stopped
#' @param maxFitness the upper bound on the fitness function after that the NSGA
#' search is interrupted.
#' @param names a vector of character strings providing the names of decision
#' variables.
#' @param suggestions a matrix of solutions strings to be included in the
#' initial population. If provided the number of columns must match the number
#' of decision variables.
#' @param parallel An optional argument which allows to specify if the NSGA-II
#' should be run sequentially or in parallel.
#' @param monitor a logical or an R function which takes as input the current
#' state of the nsga-class object and show the evolution of the search. By
#' default, for interactive sessions the function rmooMonitor prints the average
#' and best fitness values at each iteration. If set to plot these information
#' are plotted on a graphical device. Other functions can be written by the user
#' and supplied as argument. In non interactive sessions, by default
#' monitor = FALSE so any output is suppressed.
#' @param summary If there will be a summary generation after generation.
#' @param seed an integer value containing the random number generator state.
#' This argument can be used to replicate the results of a NSGA search. Note
#' that if parallel computing is required, the doRNG package must be installed.
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @references Kalyanmoy Deb and J. Sundar. 2006. Reference point based
#' multi-objective optimization using evolutionary algorithms. In Proceedings of
#' the 8th annual conference on Genetic and evolutionary computation (GECCO '06).
#' Association for Computing Machinery, New York, NY, USA, 635–642.
#' doi: 10.1145/1143997.1144112
#'
#' @seealso [nsga()], [nsga2()], [nsga3()]
#'
#' @return Returns an object of class rnsga2-class. See [rnsga2-class] for a
#' description of available slots information.
#'
#' @examples
#' #Example
#' #Two Objectives - Real Valued
#' zdt1 <- function (x) {
#'  if (is.null(dim(x))) {
#'    x <- matrix(x, nrow = 1)
#'  }
#'  n <- ncol(x)
#'  g <- 1 + rowSums(x[, 2:n, drop = FALSE]) * 9/(n - 1)
#'  return(cbind(x[, 1], g * (1 - sqrt(x[, 1]/g))))
#' }
#'
#' #Define the reference points
#' reference_points = rbind(c(0.2, 0.8), c(0.8, 0.2), c(0.4, 0.5))
#'
#' #Not run:
#' \dontrun{
#' result <- rnsga2(type = "real-valued",
#'                 fitness = zdt1,
#'                 lower = c(0,0),
#'                 upper = c(1,1),
#'                 reference_dirs = reference_points,
#'                 popSize = 100,
#'                 nObj = 2,
#'                 monitor = FALSE,
#'                 maxiter = 500,
#'                 seed = 45)
#' }
#'
#' #Example 2
#' #Three Objectives - Real Valued
#' dtlz1 <- function (x, nobj = 3){
#'     if (is.null(dim(x))) {
#'         x <- matrix(x, 1)
#'     }
#'     n <- ncol(x)
#'     y <- matrix(x[, 1:(nobj - 1)], nrow(x))
#'     z <- matrix(x[, nobj:n], nrow(x))
#'     g <- 100 * (n - nobj + 1 + rowSums((z - 0.5)^2 - cos(20 * pi * (z - 0.5))))
#'     tmp <- t(apply(y, 1, cumprod))
#'     tmp <- cbind(t(apply(tmp, 1, rev)), 1)
#'     tmp2 <- cbind(1, t(apply(1 - y, 1, rev)))
#'     f <- tmp * tmp2 * 0.5 * (1 + g)
#'     return(f)
#' }
#'
#' #Define the reference points
#' reference_points <- rbind(c(1.0, 0.5, 0.0), c(0.0, 0.5, 1.0), c(0.5, 0.5, 0.5))
#'
#' #Not run:
#' \dontrun{
#' result <- rnsga2(type = "real-valued",
#'                 fitness = dtlz1,
#'                 lower = c(0,0,0),
#'                 upper = c(1,1,1),
#'                 reference_dirs = reference_points,
#'                 popSize = 92,
#'                 nObj = 3,
#'                 monitor = FALSE,
#'                 maxiter = 500)
#' }
#'
#' @export
rnsga2 <- function(type = c("binary", "real-valued", "permutation"),
                  fitness, ...,
                  lower, upper, nBits,
                  population = rmooControl(type)$population,
                  selection = rmooControl(type)$selection,
                  crossover = rmooControl(type)$crossover,
                  mutation = rmooControl(type)$mutation,
                  reference_dirs = NULL,
                  epsilon = 0.001,
                  normalization = c("ever", "front", "no"),
                  extreme_points_as_ref_dirs = FALSE,
                  weights = NULL,
                  popSize = 50,
                  nObj = NULL,
                  pcrossover = 0.8,
                  pmutation = 0.1,
                  maxiter = 100,
                  run = maxiter,
                  maxFitness = Inf,
                  names = NULL,
                  suggestions = NULL,
                  parallel = FALSE,
                  monitor = if (interactive()) rmooMonitor else FALSE,
                  summary = FALSE,
                  seed = NULL)
{

  call <- match.call()

  type <- match.arg(type, choices = eval(formals(rnsga2)$type))

  normalization <- match.arg(normalization, choices = eval(formals(rnsga2)$normalization))

  callArgs <- list(...)

  callArgs$strategy <- NULL

  if (!is.function(population))
    population <- get(population)
  if (!is.function(selection))
    selection <- get(selection)
  if (!is.function(crossover))
    crossover <- get(crossover)
  if (!is.function(mutation))
    mutation <- get(mutation)

  if (is.null(nObj)) {
    stop("Please, define the objective number (nObj)")
  } else {
    if (!is.numeric(nObj) | (nObj%%1!=0)) {
      stop("Objective number (nObj) is a character or is not an integer.")
    }
  }

  if (is.null(reference_dirs)) {
    stop("Please, define the reference points (reference_dirs)")
  } else {
    if ((ncol(reference_dirs) != nObj) & !is.matrix(reference_dirs)) {
      stop("The provided reference points must be a matrix and have the same
    columns as objective number function")
    }
  }

  if (missing(fitness)) {
    stop("A fitness function must be provided")
  }
  if (!is.function(fitness)) {
    stop("A fitness function must be provided")
  }
  if (popSize < 10) {
    warning("The population size is less than 10.")
  }
  if (maxiter < 1) {
    stop("The maximum number of iterations must be at least 1.")
  }
  if (pcrossover < 0 | pcrossover > 1) {
    stop("Probability of crossover must be between 0 and 1.")
  }
  if (is.numeric(pmutation)) {
    if (pmutation < 0 | pmutation > 1) {
      stop("If numeric probability of mutation must be between 0 and 1.")
    } else if (!is.function(population)) {
      stop("pmutation must be a numeric value in (0,1) or a function.")
    }
  }

  if (missing(lower) & missing(upper) & missing(nBits)) {
    stop("A lower and upper range of values (for 'real-valued' or 'permutation') or nBits (for 'binary') must be provided!")
  }

  # if (is.null(nObj)) {
  #   nObj <- ncol(fitness(matrix(10000, ncol = 100, nrow = 100)))
  # }

  switch(type, binary = {
    nBits <- as.vector(nBits)[1]
    lower <- upper <- NA
    nvars <- nBits
    if (is.null(names)) names <- paste0("x", 1:nvars)
  }, `real-valued` = {
    lnames <- names(lower)
    unames <- names(upper)
    lower <- as.vector(lower)
    upper <- as.vector(upper)
    nBits <- NA
    if (length(lower) != length(upper))
      stop("lower and upper must be vector of the same length")
    nvars <- length(upper)
    if (is.null(names) & !is.null(lnames)) names <- lnames
    if (is.null(names) & !is.null(unames)) names <- unames
    if (is.null(names))
      names <- paste0("x", 1:nvars)
  }, permutation = {
    lower <- as.vector(lower)[1]
    upper <- as.vector(upper)[1]
    nBits <- NA
    nvars <- length(seq.int(lower, upper))
    if (is.null(names))
      names <- paste0("x", 1:nvars)
  })

  if (is.null(suggestions)) {
    suggestions <- matrix(nrow = 0, ncol = nvars)
  } else {
    if (is.vector(suggestions)) {
      if (nvars > 1)
        suggestions <- matrix(suggestions, nrow = 1)
      else
        suggestions <- matrix(suggestions, ncol = 1)
    } else {
      suggestions <- as.matrix(suggestions)
    }
    if (nvars != ncol(suggestions))
      stop("Provided suggestions (ncol) matrix do not match number of variables of the problem")
  }

  # check monitor arg
  if (is.logical(monitor)) {
    if (monitor)
      monitor <- rmooMonitor
  }
  if (is.null(monitor))
    monitor <- FALSE

  # Start parallel computing (if needed)
  if(is.logical(parallel)){
    if(parallel) {
      parallel <- startParallel(parallel)
      stopCluster <- TRUE
    } else {
      parallel <- stopCluster <- FALSE
    }
  }else {
    stopCluster <- if(inherits(parallel, "cluster")) FALSE else TRUE
    parallel <- startParallel(parallel)
  }
  on.exit(if(parallel & stopCluster)
    stopParallel(attr(parallel, "cluster")))
  # define operator to use depending on parallel being TRUE or FALSE
  `%DO%` <- if(parallel && requireNamespace("doRNG", quietly = TRUE)){
    doRNG::`%dorng%` } else if (parallel){ foreach::`%dopar%` } else { foreach::`%do%` }

  # set seed for reproducibility
  if (!is.null(seed))
    set.seed(seed)

  i. <- NULL  #dummy to trick R CMD check

  Fitness <- matrix(NA, nrow = popSize, ncol = nObj)

  fitnessSummary <- vector("list", maxiter)

  # Creacion del objetivo tipo nsga
  object <- new("rnsga2",
                call = call,
                type = type,
                lower = lower,
                upper = upper,
                nBits = nBits,
                names = if (is.null(names))
                  character()
                else names,
                popSize = popSize,
                front = matrix(),
                f = list(),
                iter = 0,
                run = 1,
                maxiter = maxiter,
                suggestions = suggestions,
                population = matrix(),
                pcrossover = pcrossover,
                pmutation = if (is.numeric(pmutation))
                  pmutation
                else NA,
                crowdingDistance = c(),
                fitness = Fitness,
                reference_points = reference_dirs,
                extreme_points = matrix(),
                smin =  rep(NA, nObj),
                summary = fitnessSummary)

    # Generate initial population
    if (maxiter == 0)
      return(object)

    p_fit <- q_fit <- matrix(NA_real_, nrow = popSize, ncol = nObj)
    switch(type,
      binary = {
        Pop <- P <- Q <- matrix(NA_real_, nrow = popSize, ncol = nBits)
      },
      `real-valued` = {
        Pop <- P <- Q <- matrix(NA_real_, nrow = popSize, ncol = nvars)
      },
      permutation = {
        Pop <- P <- Q <- matrix(NA_real_, nrow = popSize, ncol = nvars)
      }
    )

    ng <- min(nrow(suggestions), popSize)

    if (ng > 0) {
      Pop[1:ng, ] <- suggestions
    }
    if (popSize > ng) {
      Pop[(ng + 1):popSize, ] <- population(object)[1:(popSize - ng), ]
    }
    object@population <- Pop

    if(!parallel) {
      for (i in seq_len(popSize)) {
        if (is.na(Fitness[i])) {
          fit <- do.call(fitness, c(list(Pop[i, ]), callArgs))
          Fitness[i, ] <- fit
        }
      }
    } else {
      Fitness <- foreach(i. = seq_len(popSize), .combine = "rbind") %DO%
        { if(is.na(Fitness[i.]))
          do.call(fitness, c(list(Pop[i.,]), callArgs))
          else
            Fitness[i.,]
        }
    }

    object@population <- P <- Pop
    object@fitness <- p_fit <- Fitness

    #First Non-dominated Ranking
    out <- non_dominated_fronts(object)
    object@f <- out$fit
    object@front <- matrix(unlist(out$fronts), ncol = 1, byrow = TRUE)
    # object@crowdingDistance <- c() Crowding measure with the smallest distance to reference points

    for (iter in seq_len(maxiter)) {
      object@iter <- iter

      #Selection Operator
      if (is.function(selection)) {
        sel <- selection(object, nObj)
        Pop <- sel$population
        Fitness <- sel$fitness
      } else {
        sel <- sample(1:popSize, size = popSize, replace = TRUE)
        Pop <- object@population[sel, ]
        Fitness <- object@fitness[sel, ]
      }
      object@population <- Pop
      object@fitness <- Fitness

      # Cross Operator
      if (is.function(crossover) & pcrossover > 0) {
        nmating <- floor(popSize / 2)
        mating <- matrix(sample(1:(2 * nmating), size = (2 * nmating)), ncol = 2)
        for (i in seq_len(nmating)) {
          if (pcrossover > runif(1)) {
            parents <- mating[i, ]
            Crossover <- crossover(object, parents)
            Pop[parents, ] <- Crossover$children
            Fitness[parents, ] <- Crossover$fitness
          }
        }
      }
      object@population <- Pop
      object@fitness <- Fitness

      #Mutation Operator
      pm <- if (is.function(pmutation)) {
        pmutation(object)
      } else {pmutation}
      if (is.function(mutation) & pm > 0) {
        for (i in seq_len(popSize)) {
          if (pm > runif(1)) {
            Mutation <- mutation(object, i)
            Pop[i, ] <- Mutation
            Fitness[i, ] <- NA
          }
        }
      }
      object@population <- Q <- Pop
      object@fitness <- q_fit <- Fitness

      #Evaluate Fitness
      if(!parallel) {
        for (i in seq_len(popSize)) {
          if (is.na(Fitness[i])) {
            fit <- do.call(fitness, c(list(Pop[i, ]), callArgs))
            Fitness[i, ] <- fit
          }
        }
      } else {
        Fitness <- foreach(i. = seq_len(popSize), .combine = "rbind") %DO%
          { if(is.na(Fitness[i.]))
            do.call(fitness, c(list(Pop[i.,]), callArgs))
            else
              Fitness[i.,]
          }
      }

      object@population <- Q <- Pop
      object@fitness <- q_fit <- Fitness

      #R = P U Q
      object@population <- Pop <- rbind(P, Q)
      object@fitness <- rbind(p_fit, q_fit)

      out <- non_dominated_fronts(object)
      object@f <- out$fit
      object@front <- matrix(unlist(out$fronts), ncol = 1, byrow = TRUE)

      #R-NSGA-II Operator
      cd <- modifiedCrowdingDistance(object, epsilon, weights, normalization, extreme_points_as_ref_dirs)
      object@crowdingDistance <- cd$survivors
      object@reference_points <- cd$reference_points
      object@smin <- cd$indexmin
      rm(cd)

      # Sorted population and fitness by front and crowding distance
      populationsorted <- object@population[object@crowdingDistance, ]
      fitnesssorted <- object@fitness[object@crowdingDistance, ]

      # Select de first N element
      object@population <- P <- Pop <- populationsorted
      object@fitness <- p_fit <- fitnesssorted

      out <- non_dominated_fronts(object)
      object@f <- out$fit
      object@front <- matrix(unlist(out$fronts), ncol = 1, byrow = TRUE)

      # ------------------------------------------------------------------------
      if (summary == TRUE) {
        fitnessSummary[[iter]] <- progress(object, callArgs)
        object@summary <- fitnessSummary
      } else {
        object@summary <- list(NULL)
      }

      # Plot front non-dominated by iteration
      if (is.function(monitor)) {
        monitor(object = object, callArgs)
        # monitor(object = object, number_objective = nObj)
      }

      if (max(Fitness, na.rm = TRUE) >= maxFitness)
        break
      if (object@iter == maxiter)
        break
    }

    solution <- object

    return(solution)
}

## R-NSGA-II Bare Process
# @export
r_nsga_ii <- function(object, epsilon, weights, normalization, extreme_points_as_ref_dirs) {
  cd <- modifiedCrowdingDistance(object, epsilon, weights, normalization, extreme_points_as_ref_dirs)
  object@crowdingDistance <- cd$survivors

  object@population <- P <- Pop <- object@population[object@crowdingDistance, ]
  object@fitness <- p_fit <- object@fitness[object@crowdingDistance, ]

  out <- non_dominated_fronts(object)
  object@f <- out$fit
  object@front <- matrix(unlist(out$fronts), ncol = 1, byrow = TRUE)

  cd <- modifiedCrowdingDistance(object, epsilon, weights, normalization, extreme_points_as_ref_dirs)
  object@crowdingDistance <- cd$survivors

  out <- list(object = object,
              p_pop = Pop,
              p_fit = p_fit)

  return(out)
}

# @export
# @rdname progress-methods
# @aliases progress,rnsga2-method
#setMethod("progress", "rnsga2", .rnsga2.progress)

# @export
#' @rdname plot-methods
#' @aliases plot,rnsga2-method
setMethod("plot", signature(x="rnsga2", y="missing"), .get.plotting)

#' KROA100
#'
#' A dataset containing the coord and section of 100 cities
#'
#' @format A data frame with 100 rows and 2 variables:
#' \describe{
#'     \item{COORD}{City Coordinates}
#'     \item{SECTION}{City Section}
#' }
#' @references Reinelt, G. (1991). TSPLIB—A traveling salesman problem library.
#' ORSA journal on computing, 3(4), 376-384
"kroA100"

#' KROB100
#'
#' A dataset containing the coord and section of 100 cities
#'
#' @format A data frame with 100 rows and 2 variables:
#' \describe{
#'     \item{COORD}{City Coordinates}
#'     \item{SECTION}{City Section}
#' }
#' @references Reinelt, G. (1991). TSPLIB—A traveling salesman problem library.
#' ORSA journal on computing, 3(4), 376-384
"kroB100"


#' KROC100
#'
#' A dataset containing the coord and section of 100 cities
#'
#' @format A data frame with 100 rows and 2 variables:
#' \describe{
#'     \item{COORD}{City Coordinates}
#'     \item{SECTION}{City Section}
#' }
#' @references Reinelt, G. (1991). TSPLIB—A traveling salesman problem library.
#' ORSA journal on computing, 3(4), 376-384
"kroC100"

# Real Value NSGA operators: Generate a real random population ----
#' @rdname rmoo_Population
#' @title Generate Random Initial Population
#' @description Functions for creating a random initial population to be used
#' in non-dominated genetic algorithms.
#'   \code{rmooreal_Population} generates a uniform real population in
#'   \code{[lower, upper]};
#'   \code{rmoobin_Population} generates a binary integer population in
#'   \code{\{0, 1\}};
#'   \code{rmooperm_Population} generates a random permutation population;
#'   \code{rmooint_Population} generates a discrete integer population in
#'   \code{[lower, upper]}.
#' @param object An object of class \code{\link{nsga-class}},
#'   \code{\link{nsga2-class}} or \code{\link{nsga3-class}}.
#' @return Numeric matrix popSize × nVars of candidate solutions.
#' @seealso \code{\link{nsga}}, \code{\link{nsga2}}, \code{\link{nsga3}}
#' @examples
#' obj@lower <- c(0,0); obj@upper <- c(1,1); obj@popSize <- 10
#' rmooreal_Population(obj)
#' @examples
#' \dontrun{
#' result <- nsga2(
#'   fitness  = function(x) c(x^2, (x - 2)^2),
#'   lower    = 0,
#'   upper    = 1,
#'   popSize  = 10,
#'   population = rmooreal_Population
#' )
#' }
#' @export
rmooreal_Population <- function(object) {
    lower <- object@lower
    upper <- object@upper
    nvars <- length(lower)
    population <- matrix(NA_real_, nrow = object@popSize, ncol = nvars)
    for (j in 1:nvars) {
        population[, j] <- runif(object@popSize, lower[j], upper[j])
    }
    return(population)
}

# Binary NSGA operators: Generate a binary random population ----
#' @rdname rmoo_Population
#' @export
rmoobin_Population <- function(object) {
    population <- matrix(NA_real_,
                         nrow = object@popSize,
                         ncol = object@nBits)
    for (j in 1:object@nBits) {
        population[, j] <- round(runif(object@popSize))
    }
    storage.mode(population) <- "integer"
    return(population)
}

# Permutation NSGA operators: Generate a permutation random population ----
#' @rdname rmoo_Population
#' @export
rmooperm_Population <- function(object) {
    int <- seq.int(object@lower, object@upper)
    n <- length(int)
    population <- matrix(NA, nrow = object@popSize, ncol = n)
    for (i in 1:object@popSize)
      population[i, ] <- sample(int, replace = FALSE)
    return(population)
}

# Integer NSGA operators: Generate a discrete random population ----
#' @rdname rmoo_Population
#' @export
rmooint_Population <- function(object) {
  lower <- object@lower
  upper <- object@upper
  popSize <- object@popSize
  nvars <- object@nvars

  population <- matrix(sample(x = lower:upper,
                              size = popSize * nvars,
                              replace = TRUE),
                       ncol = nvars,
                       nrow = popSize)

  storage.mode(population) <- "integer"
  return(population)
}
# rmooint_Population <- function(object) {
#   lower <- object@lower
#   upper <- object@upper
#   popSize <- object@popSize
#   nvars <- object@nvars
#
#   population <- matrix(sample(x = lower:upper,
#                               size = popSize * nvars,
#                               replace = TRUE),
#                        nrow = popSize,
#                        ncol = nvars)
#   storage.mode(population) <- "integer"
#   return(population)
# }
#
# rmooint_Population <- function(object) {
#   lower <- object@lower
#   upper <- object@upper
#   popSize <- object@popSize
#   nvars <- object@nvars
#
#   population <- matrix(NA_integer_, nrow = popSize, ncol = nvars)
#   for (i in 1:popSize) {
#     population[i, ] <- sample(lower:upper, nvars, replace = TRUE)
#   }
#   storage.mode(population) <- "integer"
#   return(population)
# }

# @param method   `"rank_crowding"` or `"cv_random"`.
## Selection Operators ---- //Change to method
#' @title Tournament Selection
#' @description Binarily o por pref
#' @param object   MOEA object with slots population, fitness, front, popSize.
#' @param k        Tournament size.
#' @param ...      Argument which all the values necessary for the configuration
#' will be passed as parameters. The user is encouraged to see the documentations.
#' @return List with `population` and `fitness` subsets.
#' @examples
#' \dontrun{
#' # Creamos un "dummy" objeto mínimo
#' object <- list(
#'   population       = matrix(runif(20), nrow=5),
#'   fitness          = matrix(runif(10), nrow=5),
#'   front            = matrix(sample(1:2, 5, TRUE), ncol=1),
#'   crowdingDistance = runif(5),
#'   popSize          = 5
#' )
#' class(object) <- "nsga2"
#' # Llamamos al selector
#' sel <- rmoo_tourSelection(object, k = 2)
#' str(sel)
#' }
#' @export
rmoo_tourSelection <- function(object, k = 2, ...) {
  class_object <- class(object)[1]
  if (class_object == "nsga2") {
    sel <- binary_tournament(object, k)
  } else if (class_object == "rnsga2" || class_object == "nsga3") {
    sel <- comp_by_cv_then_random(object, k)
  }
  out <- list(population = object@population[sel, ],
              fitness = object@fitness[sel, ])
  return(out)
}


# rmoo_tourSelection <- function(object, k = 2, ...) {
#   class_object <- class(object)[1]
#
#   if (class_object == "nsga2") {
#     popSize <- object@popSize
#     front <- object@front
#     cd <- crowding_distance(object, ncol(object@fitness))
#     sel <- rep(NA, popSize)
#     for (i in seq_len(popSize)) {
#       s <- sample(seq_len(popSize), size = k)
#       s <- s[which.min(front[s, ])]
#       if (!anyNA(cd[s, ])) {
#         sel[i] <- s[which.max(cd[s, ])]
#       } else {
#         sel[i] <- s[which.min(front[s, ])]
#       }
#     }
  # } else if (class_object == "rnsga2" || class_object == "nsga3") {
  #   popSize <- object@popSize
  #   front <- object@front
  #   fit <- object@fitness
  #   sel <- rep(NA, popSize)
  #   for (i in seq_len(popSize)) {
  #     s <- sample(seq_len(popSize), size = k)
  #     s <- s[which.min(front[s, ])]
  #     if (length(s) > 1 & !anyNA(fit[s, ])) {
  #       sel[i] <- s[which.max(front[s, ])]
  #     } else {
  #       sel[i] <- s[which.min(front[s, ])]
  #     }
  #   }
  # }
#   out <- list(population = object@population[sel, ],
#               fitness = object@fitness[sel, ])
#   return(out)
# }

# rmoo_tourSelection <- function(object, k = 3, ...) {
#     switch(class(object)[1], nsga1 = {
#         popSize <- object@popSize
#         front <- object@front
#         fit <- object@dumFitness
#         sel <- rep(NA, popSize)
#         for (i in 1:popSize) {
#             s <- sample(1:popSize, size = k)
#             s <- s[which.min(front[s, ])]
#             if (length(s) > 1 & !anyNA(fit[s, ])) {
#                 sel[i] <- s[which.max(front[s, ])]
#             } else {
#                 sel[i] <- s[which.min(front[s, ])]
#             }
#         }
#         out <- list(population = object@population[sel, ],
#                     fitness = object@fitness[sel, ])
#         return(out)
#     }, nsga2 = {
#         popSize <- object@popSize
#         front <- object@front
#         cd <- object@crowdingDistance
#         sel <- rep(NA, popSize)
#         for (i in 1:popSize) {
#             s <- sample(1:popSize, size = k)
#             s <- s[which.min(front[s, ])]
#             if (!anyNA(cd[s, ])) {
#                 sel[i] <- s[which.max(cd[s, ])]
#             } else {
#                 sel[i] <- s[which.min(front[s, ])]
#             }
#         }
#         out <- list(population = object@population[sel, ],
#                     fitness = object@fitness[sel, ])
#         return(out)
#     }, rnsga2 = {
#       popSize <- object@popSize
#       front <- object@front
#       fit <- object@fitness
#       sel <- rep(NA, popSize)
#       for (i in 1:popSize) {
#         s <- sample(1:popSize, size = k)
#         s <- s[which.min(front[s, ])]
#         if (length(s) > 1 & !anyNA(fit[s, ])) {
#           sel[i] <- s[which.max(front[s, ])]
#         } else {
#           sel[i] <- s[which.min(front[s, ])]
#         }
#       }
#       out <- list(population = object@population[sel, ],
#                   fitness = object@fitness[sel, ])
#       return(out)
#     }, nsga3 = {
#         popSize <- object@popSize
#         front <- object@front
#         fit <- object@fitness
#         sel <- rep(NA, popSize)
#         for (i in 1:popSize) {
#             s <- sample(1:popSize, size = k)
#             s <- s[which.min(front[s, ])]
#             if (length(s) > 1 & !anyNA(fit[s, ])) {
#                 sel[i] <- s[which.max(front[s, ])]
#             } else {
#                 sel[i] <- s[which.min(front[s, ])]
#             }
#         }
#         out <- list(population = object@population[sel, ],
#                     fitness = object@fitness[sel, ])
#         return(out)
#     })
# }

#' @rdname rmoo_tourSelection
#' @export
rmooreal_tourSelection <- rmoo_tourSelection
#' @rdname rmoo_tourSelection
#' @export
rmoobin_tourSelection <- rmoo_tourSelection
#' @rdname rmoo_tourSelection
#' @export
rmooperm_tourSelection <- rmoo_tourSelection
# @export
rmooint_tourSelection <- rmoo_tourSelection

#' @title Linear Rank Selection
#' @description Selects individuals from the population using linear rank-based
#'   probabilities. Individuals are ranked by their Pareto front, and selection
#'   probability is assigned linearly so better-ranked individuals are more
#'   likely to be chosen.
#' @param object  MOEA object with slots population, fitness, front, popSize.
#' @param r       Slope parameter controlling probability spread (computed from
#'                popSize if missing).
#' @param q       Intercept parameter (computed from popSize if missing).
#' @return A list with \code{population} and \code{fitness} of the selected
#'   individuals.
#' @export
rmoo_lrSelection <- function(object, r, q) {
  if (missing(r))
    r <- 2 / (object@popSize * (object@popSize - 1))
  if (missing(q))
    q <- 2 / object@popSize
  rank <- (object@popSize + 1) - as.vector(object@front)
  prob <- 1 + q - (rank - 1) * r
  prob <- pmin(pmax(0, prob / sum(prob)), 1, na.rm = TRUE)
  sel <- sample(1:object@popSize,
                size = object@popSize,
                prob = prob, replace = TRUE)
  out <- list(population = object@population[sel, ],
              fitness = object@fitness[sel, ])
  return(out)
}

#' @rdname rmoo_lrSelection
#' @export
rmoobin_lrSelection <- rmoo_lrSelection

#' @rdname rmoo_lrSelection
#' @export
rmooperm_lrSelection <- rmoo_lrSelection

#' @rdname rmoo_lrSelection
#' @export
rmooreal_lrSelection <- rmoo_lrSelection

# @export
rmooint_lrSelection <- rmoo_lrSelection

## Crossover Operators ----
#' @rdname rmoo_Crossover
#' @param object  An object of class \code{"nsga"}, \code{"nsga2"}, or
#'   \code{"nsga3"}, usually from a call to \code{\link{nsga}},
#'   \code{\link{nsga2}}, or \code{\link{nsga3}}.
#' @param parents A two-element integer vector indexing the parents from the
#'   current population.
#' @param eta The distribution index. A higher eta produces offspring closer to
#' the parents, while a lower eta allows for larger differences.
#' @param indpb The probability of a particular gene being crossed.
#' @export
rmooreal_sbxCrossover <- function(object, parents, eta = 20, indpb = 0.5) {
  parents <- object@population[parents, ]
  n <- ncol(parents)
  nObj <- ncol(object@fitness)
  children <- matrix(NA_real_, nrow = 2, ncol = n)

  parent1 <- parents[1, ]
  parent2 <- parents[2, ]

  yl <- object@lower
  yu <- object@upper

  for (i in 1:n) {
    if (runif(1) <= indpb) {
      if (abs(parent1[i] - parent2[i]) > 1e-4) {
        x1 <- min(parent1[i], parent2[i])
        x2 <- max(parent1[i], parent2[i])
        rand <- runif(1)

        beta <- 1 + 2 * (x1 - yl[i]) / (x2 - x1)
        alpha <- 2 - beta^(-(eta + 1))

        if (rand <= (1 / alpha)) {
          beta_q <- (rand * alpha)^(1 / (eta + 1))
        } else {
          beta_q <- (1 / (2 - rand * alpha))^(1 / (eta + 1))
        }

        c1 <- 0.5 * (x1 + x2 - beta_q * (x2 - x1))

        beta <- 1 + 2 * (yu[i] - x2) / (x2 - x1)
        alpha <- 2 - beta^-(eta + 1)

        if (rand <= (1 / alpha)) {
          beta_q <- (rand * alpha)^(1 / (eta + 1))
        } else {
          beta_q <- (1 / (2 - rand * alpha))^(1 / (eta + 1))
        }

        c2 <- 0.5 * (x1 + x2 + beta_q * (x2 - x1))

        c1 <- pmin(pmax(c1, yl[i]), yu[i])
        c2 <- pmin(pmax(c2, yl[i]), yu[i])

        if (runif(1) <= 0.5) {
          parent1[i] <- c2
          parent2[i] <- c1
        } else {
          parent1[i] <- c1
          parent2[i] <- c2
        }
      }
    }
  }
  children[1, ] <- parent1
  children[2, ] <- parent2

  out <- list(children = children,
              fitness = matrix(NA_real_, nrow = 2, ncol = nObj))

  return(out)
}

# rmooreal_sbxCrossover <- function(object, parents, nc = 20) {
#     parents <- object@population[parents, ]
#     n <- ncol(parents)
#     nObj <- ncol(object@fitness)
#     children <- matrix(NA_real_, nrow = 2, ncol = n)
#     for (j in 1:n) {
#         parent1 <- parents[1, j]
#         parent2 <- parents[2, j]
#         yl <- object@lower[j]
#         yu <- object@upper[j]
#         rnd <- runif(1)
#         if (rnd <= 0.5) {
#             if (abs(parent1 - parent2) > 1e-06) {
#                 if (parent2 > parent1) {
#                   y2 <- parent2
#                   y1 <- parent1
#                 } else {
#                   y2 <- parent1
#                   y1 <- parent2
#                 }
#                 if ((y1 - yl) > (yu - y2)) {
#                   beta <- 1 + (2 * (yu - y2) / (y2 - y1))
#                 } else {
#                   beta <- 1 + (2 * (y1 - yl) / (y2 - y1))
#                 }
#                 alpha <- 2 - (beta^(-(1 + nc)))
#                 rnd <- runif(1)
#                 if (rnd <= 1 / alpha) {
#                   alpha <- alpha * rnd
#                   betaq <- alpha^(1 / (1 + nc))
#                 } else {
#                   alpha <- alpha * rnd
#                   alpha <- 1 / (2 - alpha)
#                   betaq <- alpha^(1 / (1 + nc))
#                 }
#                 child1 <- 0.5 * ((y1 + y2) - betaq * (y2 - y1))
#                 child2 <- 0.5 * ((y1 + y2) + betaq * (y2 - y1))
#             } else {
#                 betaq <- 1
#                 y1 <- parent1
#                 y2 <- parent2
#                 child1 <- 0.5 * ((y1 + y2) - betaq * (y2 - y1))
#                 child2 <- 0.5 * ((y1 + y2) + betaq * (y2 - y1))
#             }
#             if (child1 > yu) {
#                 child1 <- yu
#             } else if (child1 < yl) {
#                 child1 <- yl
#             }
#             if (child2 > yu) {
#                 child2 <- yu
#             } else if (child2 < yl) {
#                 child2 <- yl
#             }
#         } else {
#             child1 <- parent1
#             child2 <- parent2
#         }
#         children[1, j] <- child1
#         children[2, j] <- child2
#     }
#     out <- list(children = children,
#                 fitness = matrix(NA_real_, ncol = nObj))
#     return(out)
# }


#' @name rmoo_Crossover
#' @rdname rmoo_Crossover
#' @title Crossover Operators in Non-Dominated Genetic Algorithms
#' @description Functions implementing crossover operators for non-dominated
#'   genetic algorithms. \code{rmoo_spCrossover} (and its typed variants
#'   \code{rmoobin_}, \code{rmooreal_}, \code{rmooint_}) performs single-point
#'   crossover; \code{rmooreal_sbxCrossover} performs simulated binary
#'   crossover; \code{rmooperm_oxCrossover} performs order crossover for
#'   permutation representations.
#' @param object  An object of class \code{"nsga"}, \code{"nsga2"}, or
#'   \code{"nsga3"}, usually from a call to \code{\link{nsga}},
#'   \code{\link{nsga2}}, or \code{\link{nsga3}}.
#' @param parents A two-element integer vector indexing the parents from the
#'   current population.
#' @return A list with two elements:
#'   \describe{
#'     \item{children}{A matrix of dimension 2 × nVars containing the
#'       generated offspring.}
#'     \item{fitness}{A 2 × nObj matrix of \code{NA} values, indicating that
#'       offspring fitness has not yet been evaluated.}
#'   }
#' @references Scrucca, L. (2017) On some extensions to 'GA' package: hybrid
#'   optimisation, parallelisation and islands evolution. \emph{The R Journal},
#'   9/1, 187--206, \doi{10.32614/RJ-2017-008}.
#' @author Francisco Benitez
#' @seealso \code{\link[=nsga]{nsga()}}, \code{\link[=nsga2]{nsga2()}},
#'   \code{\link[=nsga3]{nsga3()}}
#' @export
rmoo_spCrossover <- function(object, parents) {
    fitness <- object@fitness[parents, ]
    parents <- object@population[parents, ]
    n <- ncol(parents)
    children <- matrix(NA_real_, nrow = 2, ncol = n)
    crossOverPoint <- sample(0:n, size = 1)
    if (crossOverPoint == 0) {
        fitnessChildren <- matrix(NA_real_, nrow = 2, ncol = ncol(fitness))
        children[1:2, ] <- parents[2:1, ]
        fitnessChildren[1:2, ] <- fitness[2:1, ]
    } else if (crossOverPoint == n) {
        children <- parents
        fitnessChildren <- fitness
    } else {
        fitnessChildren <- matrix(NA_real_, nrow = 2, ncol = ncol(object@fitness))
        children[1, ] <- c(parents[1, 1:crossOverPoint], parents[2, (crossOverPoint + 1):n])
        children[2, ] <- c(parents[2, 1:crossOverPoint], parents[1, (crossOverPoint + 1):n])
    }
    out <- list(children = children,
                fitness = fitnessChildren)
    return(out)
}

#' @rdname rmoo_Crossover
#' @export
rmoobin_spCrossover <- rmoo_spCrossover

#' @rdname rmoo_Crossover
#' @export
rmooreal_spCrossover <- rmoo_spCrossover

#' @rdname rmoo_Crossover
#' @export
rmooint_spCrossover <- rmoo_spCrossover

#' @rdname rmoo_Crossover
#' @export
rmooperm_oxCrossover <- function(object, parents) {
    parents <- object@population[parents, ]
    n <- ncol(parents)
    #
    cxPoints <- sample(seq(2, n - 1), size = 2)
    cxPoints <- seq(min(cxPoints), max(cxPoints))
    children <- matrix(NA_real_, nrow = 2, ncol = n)
    children[, cxPoints] <- parents[, cxPoints]
    #
    for (j in 1:2) {
        pos <- c((max(cxPoints) + 1):n, 1:(max(cxPoints)))
        val <- setdiff(parents[-j, pos], children[j, cxPoints])
        ival <- intersect(pos, which(is.na(children[j, ])))
        children[j, ival] <- val
    }
    #
    out <- list(children = children,
                fitness  = matrix(NA_real_, nrow = 2, ncol = ncol(object@fitness)))
    return(out)
}

#' @title Uniform Crossover
#' @description Produces two children by randomly swapping genes between two
#'   parents with equal probability at each locus. Each gene is inherited from
#'   either parent independently, giving maximum gene-level mixing.
#' @param object  MOEA object with slots population and fitness.
#' @param parents Integer vector of length 2 with the parent row indices.
#' @return A list with \code{children} (2 × n integer matrix) and
#'   \code{fitness} (2 × nObj NA matrix).
#' @export
rmoo_uxCrossover <- function(object, parents) {
  parents <- object@population[parents, ]

  n_matings <- nrow(parents)
  n <- ncol(parents)

  M <- matrix(runif(n_matings * n) < 0.5, nrow = n_matings)
  fitnessChildren <- matrix(NA_integer_, nrow = 2, ncol = ncol(object@fitness))

  children <- crossover_mask(parents, M)

  storage.mode(children) <- "integer"

  out <- list(children = children,
              fitness = fitnessChildren)
  return(out)
}
#' @rdname rmoo_uxCrossover
#' @export
rmooint_uxCrossover <- rmoo_uxCrossover
#' @rdname rmoo_uxCrossover
#' @export
rmoobin_uxCrossover <- rmoo_uxCrossover

#' @title Half Uniform Crossover (HUX)
#' @description Produces two children by swapping exactly half of the genes
#'   that differ between the two parents. Only loci where the parents disagree
#'   are eligible, making HUX more conservative than full uniform crossover.
#' @param object    MOEA object with slots population and fitness.
#' @param parents   Integer vector of length 2 with the parent row indices.
#' @param prob_hux  Proportion of differing loci to swap (default 0.5).
#' @return A list with \code{children} (2 × n integer matrix) and
#'   \code{fitness} (2 × nObj NA matrix).
#' @export
rmoo_huxCrossover <- function(object, parents, prob_hux=0.5) {
  parents <- object@population[parents, ]

  n_matings <- nrow(parents)
  n <- ncol(parents)

  M <- matrix(FALSE, nrow = n_matings, ncol = n)
  fitnessChildren <- matrix(NA_integer_, nrow = 2, ncol = ncol(object@fitness))

  not_equal <- (parents[1, ] != parents[2, ])

  for (i in 1:n_matings) {
    ind <- which(not_equal[i])

    n <- ceiling(length(ind) / 2)
    if (n > 0) {
      indx <- ind[sample(length(ind), size = n)]
      M[i, indx] <- TRUE
    }
  }

  children <- crossover_mask(parents, M)
  storage.mode(children) <- "integer"

  out <- list(children = children,
              fitness = fitnessChildren)
  return(out)
}
#' @rdname rmoo_huxCrossover
#' @export
rmooint_huxCrossover <- rmoo_huxCrossover
#' @rdname rmoo_huxCrossover
#' @export
rmoobin_huxCrossover <- rmoo_huxCrossover

## Mutation Operator ----
#' @export
rmooreal_polMutation <- function(object, parent, eta=20, indpb=0.5) {
  mutate <- as.vector(object@population[parent, ])
  n <- length(parent)
  lower <- object@lower
  upper <- object@upper

  for (i in 1:n) {
    if (runif(1) <= indpb) {
      x <- mutate[i]
      delta_1 <- (x - lower[i]) / (upper[i] - lower[i])
      delta_2 <- (upper[i] - x) / (upper[i] - lower[i])
      mut_pow <- 1 / (eta + 1)

      rand <- runif(1)
      if (rand < 0.5) {
        xy <- 1 - delta_1
        val <- 2 * rand + (1 - 2 * rand) * (xy^(eta + 1))
        delta_q <- (val^mut_pow) - 1
      } else {
        xy <- 1 - delta_2
        val <- 2 * (1 - rand) + 2 * (rand - 0.5) * (xy^(eta + 1))
        delta_q <- 1 - (val^mut_pow)
      }

      x <- x + delta_q * (upper[i] - lower[i])
      x <- min(max(x, lower[i]), upper[i])
      mutate[i] <- x
    }
  }

  return(mutate)
}
# rmooreal_polMutation <- function(object, parent, nm = 0.2, indpb = 0.2) {
#   mutate <- parent <- as.vector(object@population[parent, ])
#   n <- length(parent)
#   upper <- object@upper
#   lower <- object@lower
#   delta <- upper - lower
#   delta1 <- (mutate - lower) / (upper - lower)
#   delta2 <- (upper - mutate) / (upper - lower)
#   mut_pow <- 1/(nm + 1)
#   for (i in seq_len(n)) {
#     if(runif(1) <= indpb) {
#       x <- parent[i]
#       u <- runif(1)
#       if (u <= 0.5) {
#         xy <- 1 - delta1[i]
#         val <- 2 * u + (1 - 2 * u) * (xy^(nm + 1))
#         deltaq <- (val^mut_pow) - 1
#       } else {
#         xy <- 1 - delta2[i]
#         val <- 2 * (1 - u) + 2 * (u - 0.5) * (xy^(nm + 1))
#         deltaq <- 1 - (val^mut_pow)
#       }
#       mutate[i] <- deltaq * delta[i]
#       mutate[i] <- min(max(c(x[1], lower[i])), upper[i])
#     }
#   }
#   return(mutate)
# }

#' @export
rmooreal_raMutation <- function(object, parent) {
    mutate <- parent <- as.vector(object@population[parent, ])
    n <- length(parent)
    j <- sample(1:n, size = 1)
    mutate[j] <- runif(1, object@lower[j], object@upper[j])
    return(mutate)
}

#' @export
rmoobin_raMutation <- function(object, parent) {
    mutate <- parent <- as.vector(object@population[parent, ])
    n <- length(parent)
    j <- sample(1:n, size = 1)
    mutate[j] <- abs(mutate[j] - 1)
    return(mutate)
}

#' @export
rmooperm_simMutation <- function(object, parent) {
    parent <- as.vector(object@population[parent, ])
    n <- length(parent)
    m <- sort(sample(1:n, size = 2))
    m <- seq(m[1], m[2], by = 1)
    if (min(m) == 1 & max(m) == n)
        i <- rev(m) else if (min(m) == 1)
        i <- c(rev(m), seq(max(m) + 1, n, by = 1))
    else if (max(m) == n)
        i <- c(seq(1, min(m) - 1, by = 1), rev(m))
    else i <- c(seq(1, min(m) - 1, by = 1), rev(m), seq(max(m) + 1, n, by = 1))
    mutate <- parent[i]
    return(mutate)
}

#' @title Uniform Mutation
#' @description Mutates an individual by randomly replacing each gene with a
#'   new value drawn uniformly from the integer range \code{[lower, upper]},
#'   independently at each locus with probability \code{indpb}.
#' @param object  MOEA object with slots population, lower, upper.
#' @param parent  Index of the individual to mutate.
#' @param indpb   Per-gene mutation probability (default 0.1).
#' @return An integer vector of the mutated individual.
#' @export
rmoo_uxMutation <- function(object, parent, indpb=0.1) {
  mutate <- parent <- as.vector(object@population[parent, ])

  n <- length(parent)
  lower <- rep(object@lower,n)
  upper <- rep(object@upper,n)

  for (i in seq_along(parent)) {
    if (runif(1) < indpb) {
      mutate[i] <- sample(lower[i]:upper[i], 1)
    }
  }

  storage.mode(mutate) <- "integer"
  return(mutate)
}
#' @rdname rmoo_uxMutation
#' @export
rmooint_uxMutation <- rmoo_uxMutation
#' @rdname rmoo_uxMutation
#' @export
rmoobin_uxMutation <- rmoo_uxMutation

crossover_mask <- function(X, M) {
  parent <- X
  parent[1,][M[2,]] <- X[2,][M[2,]]
  parent[2,][M[1,]] <- X[1,][M[1,]]
  return(parent)
}

rmoobin_fbMutation <- function(object, parent, indpb=0.2) {
  mutate <- parent <- as.vector(object@population[parent, ])

  mutate <- as.logical(mutate)
  for (i in seq_along(parent)) {
    if (runif(1) < indpb) {
      mutate[i] <- !mutate[i]
    }
  }
  mutate <- as.numeric(mutate)
  storage.mode(mutate) <- "integer"
  return(mutate)
}

# Two Point Cx
rmoo_tpCrossover <- function(object, parents) {
  parents <- object@population[parents, ]
  n <- ncol(parents)
  children <- matrix(NA_integer_, nrow = 2, ncol = n)
  fitnessChildren <- matrix(NA_real_, nrow = 2, ncol = ncol(object@fitness))

  ind1 <- parents[1, ]
  ind2 <- parents[2, ]

  size <- min(sum(ind1), sum(ind2))
  cxpoint1 <- sample(1:size, 1)
  cxpoint2 <- sample(1:(size - 1), 1)

  if (cxpoint2 >= cxpoint1) {
    cxpoint2 <- cxpoint2 + 1
  } else {
    temp <- cxpoint1
    cxpoint1 <- cxpoint2
    cxpoint2 <- temp
  }

  temp <- ind1[cxpoint1:cxpoint2]
  ind1[cxpoint1:cxpoint2] <- ind2[cxpoint1:cxpoint2]
  ind2[cxpoint1:cxpoint2] <- temp

  children[1,] <- ind1
  children[2,] <- ind2

  storage.mode(children) <- "integer"
  out <- list(children = children,
              fitness = fitnessChildren)
  return(out)
}

pointCrossover <- function(object, parents, n_points=2) {
  parents <- object@population[parents, ]
  n_matings <- nrow(parents)
  n <- ncol(parents)

  fitnessChildren <- matrix(NA_real_, nrow = 2, ncol = ncol(object@fitness))


  r <- t(replicate(n_matings, sample(n - 1)))
  r <- r[, 1:n_points]
  r <- apply(r, 1, sort)
  r <- cbind(r, rep(n, n_matings))

  M <- matrix(FALSE, nrow = n_matings, ncol = n)

  for (i in seq_len(n_matings)) {
    j <- 1
    while (j < (ncol(r) - 1)) {
      a <- r[i, j]
      b <- r[i, j + 1]
      M[i, a:b] <- TRUE
      j <- j + 1
    }
  }
  # for (i in seq_len(n_matings)) {
  #   j <- seq(1, (ncol(r) - 1), by=2)
  #   a <- r[i, j]
  #   b <- r[i, j + 1]
  #   M[i, a:b] <- TRUE
  # }
  children <- crossover_mask(parents, M)

  storage.mode(children) <- "integer"
  out <- list(children = children,
              fitness = fitnessChildren)
  return(out)
}


binary_tournament <- function(object, k=2, tournament_type = "comp_by_rank_and_crowding"){
  sel <- rep(NA, object@popSize)
  s <- random_permutations(k, object@popSize)
  s <- matrix(s, nrow = object@popSize, ncol = k)

  for(i in seq_len(object@popSize)) {
    a <- s[i, 1]
    b <- s[i, 2]

    a_f <- object@fitness[a, ]
    rank_a <- object@front[a, ]
    cd_a <- object@crowdingDistance[a]

    b_f <- object@fitness[b, ]
    rank_b <- object@front[b, ]
    cd_b <- object@crowdingDistance[b]

    if (tournament_type == 'comp_by_dom_and_crowding') {
      rel <- get_relation(a=a_f, b=b_f)
      if (rel == 1) {
        sel[i] <- a
      } else if (rel == -1) {
        sel[i] <- b
      }
    } else if (tournament_type == 'comp_by_rank_and_crowding') {
      sel[i] <- compare(a, rank_a, b, rank_b, method = 'smaller_is_better')
    } else {
      stop("Unknown tournament type.")
    }

    if (is.na(sel[i])) {
      sel[i] <- compare(a, cd_a, b, cd_b, method = 'larger_is_better', return_random_if_equal = TRUE)
    }

  }
  # out <- list(population = object@population[sel, ],
  #             fitness = object@fitness[sel, ])
  return(sel)
}


comp_by_cv_then_random <- function(object, k=2) {
  sel <- rep(NA, object@popSize)
  s <- random_permutations(k, object@popSize)
  s <- matrix(s, nrow = object@popSize, ncol = k)

  for(i in seq_len(object@popSize)) {
    a <- s[i, 1]
    b <- s[i, 2]

    a_f <- object@fitness[a, ]
    rank_a <- object@front[a, ]
    # cv_a <- object@cv[a]

    b_f <- object@fitness[b, ]
    rank_b <- object@front[b, ]
    # cv_b <- object@cv[b]

    # if (cv_a > 0.0 || cv_b > 0.0) {
    #   sel[i] <- compare(a, cv_a, b, cv_b, method = 'smaller_is_better', return_random_if_equal = TRUE)
    # } else {
    sel[i] <- compare(a, rank_a, b, rank_b, method = 'smaller_is_better')
    # sel[i] <- sample(c(a, b), 1)
    # }
  }
  return(sel)
}



# roundingRepair <- function(pop) {
#   pop[] <- as.integer(round(pop))
#   return(pop)
# }

context("Crossover fitness matrix dimensions")

# Helper to create a minimal nsga2 object for testing crossover functions
create_test_object <- function(type = "real-valued", nObj = 2) {
  if (type == "real-valued") {
    pop <- matrix(runif(30), nrow = 10, ncol = 3)
  } else {
    pop <- matrix(sample(0:1, 30, replace = TRUE), nrow = 10, ncol = 3)
    storage.mode(pop) <- "integer"
  }

  new("nsga2",
      call = call("test"),
      type = type,
      lower = c(0, 0, 0),
      upper = c(1, 1, 1),
      nBits = NA_integer_,
      names = c("x1", "x2", "x3"),
      popSize = 10L,
      front = matrix(1L, nrow = 10, ncol = 1),
      f = list(),
      iter = 1L,
      run = 1L,
      maxiter = 10L,
      suggestions = matrix(nrow = 0, ncol = 3),
      population = pop,
      pcrossover = 0.8,
      pmutation = 0.1,
      crowdingDistance = matrix(runif(10), nrow = 10),
      fitness = matrix(runif(10 * nObj), nrow = 10, ncol = nObj),
      summary = list())
}

test_that("rmooreal_sbxCrossover returns 2-row fitness matrix", {
  object <- create_test_object("real-valued", nObj = 2)
  parents <- c(1L, 2L)

  result <- rmooreal_sbxCrossover(object, parents)

  expect_equal(nrow(result$children), 2)
  expect_equal(nrow(result$fitness), 2)
  expect_equal(ncol(result$fitness), 2)
})

test_that("rmoo_spCrossover returns 2-row fitness matrix", {
  object <- create_test_object("real-valued", nObj = 3)
  parents <- c(1L, 2L)

  # Run multiple times to hit different branches (crossOverPoint = 0, n, or middle)
  set.seed(123)
  for (i in 1:10) {
    result <- rmoo_spCrossover(object, parents)

    expect_equal(nrow(result$children), 2)
    expect_true(is.matrix(result$fitness),
                info = "fitness should be a matrix, not a vector")
    expect_equal(nrow(result$fitness), 2)
    expect_equal(ncol(result$fitness), 3)
  }
})

test_that("rmoo_uxCrossover returns 2-row fitness matrix", {
  object <- create_test_object("binary", nObj = 2)
  parents <- c(1L, 2L)

  result <- rmoo_uxCrossover(object, parents)

  expect_equal(nrow(result$children), 2)
  expect_equal(nrow(result$fitness), 2)
  expect_equal(ncol(result$fitness), 2)
})

test_that("rmoo_huxCrossover returns 2-row fitness matrix", {
  object <- create_test_object("binary", nObj = 2)
  parents <- c(1L, 2L)

  result <- rmoo_huxCrossover(object, parents)

  expect_equal(nrow(result$children), 2)
  expect_equal(nrow(result$fitness), 2)
  expect_equal(ncol(result$fitness), 2)
})

test_that("crossover fitness can be assigned to 2 parent rows without recycling", {
  # This test verifies the fix works in the actual algorithm context
  object <- create_test_object("real-valued", nObj = 2)
  parents <- c(1L, 2L)

  result <- rmooreal_sbxCrossover(object, parents)

  # Simulate nsga2.R line 401: Fitness[parents, ] <- Crossover$fitness
  Fitness <- matrix(1:20, nrow = 10, ncol = 2)
  original_row3 <- Fitness[3, ]

  # This should work without R recycling a 1-row matrix
  Fitness[parents, ] <- result$fitness

  # Verify row 3 wasn't affected (would be if recycling occurred incorrectly)
  expect_equal(Fitness[3, ], original_row3)

  # Verify both parent rows were updated
  expect_true(all(is.na(Fitness[1, ])))
  expect_true(all(is.na(Fitness[2, ])))
})


test_that("rmoo_tpCrossover returns 2-row fitness matrix", {
  # tpCrossover works on binary/integer populations; it swaps a segment between
  # two parents, so both rows of the fitness output must be NA placeholders.
  object  <- create_test_object("binary", nObj = 2)
  parents <- c(1L, 2L)

  result <- rmoo_tpCrossover(object, parents)

  expect_true(is.matrix(result$fitness),
              info = "fitness must be a matrix, not a vector")
  expect_equal(nrow(result$fitness), 2,
               info = "tpCrossover produces 2 children, so fitness needs 2 rows")
  expect_equal(ncol(result$fitness), 2)
  expect_equal(nrow(result$children), 2)
})

test_that("pointCrossover returns 2-row fitness matrix", {
  # pointCrossover is the generalised n-point version; it always produces
  # exactly 2 children regardless of n_points.
  object  <- create_test_object("binary", nObj = 2)
  parents <- c(1L, 2L)

  result <- pointCrossover(object, parents, n_points = 2)

  expect_true(is.matrix(result$fitness),
              info = "fitness must be a matrix, not a vector")
  expect_equal(nrow(result$fitness), 2,
               info = "pointCrossover produces 2 children, so fitness needs 2 rows")
  expect_equal(ncol(result$fitness), 2)
  expect_equal(nrow(result$children), 2)
})

test_that("rmooperm_oxCrossover returns 2-row fitness matrix", {
  # oxCrossover operates on permutation representations; its fitness output
  # was a bare rep(NA, 2) vector — a bug missed by the PR.
  # ncol >= 5 is required so cxPoints can always find 2 interior positions.
  set.seed(42)
  pop <- t(replicate(10, sample(1:6)))        # 10 × 6 permutation matrix

  object <- new("nsga2",
                call     = call("test"),
                type     = "permutation",
                lower    = 1L,
                upper    = 6L,
                nBits    = NA_integer_,
                names    = as.character(1:6),
                popSize  = 10L,
                front    = matrix(1L, nrow = 10, ncol = 1),
                f        = list(),
                iter     = 1L,
                run      = 1L,
                maxiter  = 10L,
                suggestions      = matrix(nrow = 0, ncol = 6),
                population       = pop,
                pcrossover       = 0.8,
                pmutation        = 0.1,
                crowdingDistance = matrix(runif(10), nrow = 10),
                fitness          = matrix(runif(20), nrow = 10, ncol = 2),
                summary          = list())
  parents <- c(1L, 2L)

  result <- rmooperm_oxCrossover(object, parents)

  expect_true(is.matrix(result$fitness),
              info = "fitness must be a matrix, not a bare rep(NA, 2) vector")
  expect_equal(nrow(result$fitness), 2,
               info = "oxCrossover produces 2 children, so fitness needs 2 rows")
  expect_equal(ncol(result$fitness), 2)
  expect_equal(nrow(result$children), 2)
})

#' Accessor methods to the crowding distance for NSGA-II results
#'
#' @param obj an object resulting from the execution of NSGA-II algorithm
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @return Returns a vector with the crowding distances of class nsga2. See [nsga2-class]
#' for a description of available slots information.
#'
#' @examples
#' # Where 'out' is an object resulting from the execution of the NSGA-II algorithm.
#' #
#' # getCrowdingDistance(out)
#' #
#'
#' @export
#' @docType methods
#' @rdname getCrowdingDistance-methods
setGeneric("getCrowdingDistance", function(obj) standardGeneric("getCrowdingDistance"))

#' Accessor methods to the dummy fitness for NSGA-I results
#'
#' @param obj an object resulting from the execution of NSGA-I algorithm
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @return Returns a matrix with the dummy fitness of class nsga1. See [nsga1-class]
#' for a description of available slots information.
#'
#' @examples
#' # Where 'out' is an object resulting from the execution of the NSGA-I algorithm.
#' #
#' # getDummyFitness(out)
#' #
#'
#' @export
#' @docType methods
#' @rdname getDummyFitness-methods
setGeneric("getDummyFitness", function(obj) standardGeneric("getDummyFitness"))

#' Accessor methods to the population for rmoo results
#'
#' @param obj an object resulting from the execution of NSGA-I, NSGA-II or NSGA-III
#' algorithm
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @return Prints the resulting population and when the result of the method-call
#' is assigned to a variable, the population is stored as a data frame.
#' See [nsga1-class] [nsga2-class], [nsga3-class] for a description of available
#' slots information.
#'
#' @examples
#' # Where 'out' is an object resulting from the execution of rmoo.
#' #
#' # population_result <- getPopulation(out)
#' #
#' # population_result
#'
#' @export
#' @docType methods
#' @rdname getPopulation-methods
setGeneric("getPopulation", function(obj) standardGeneric("getPopulation"))

#' Accessor methods to the fitness for rmoo results
#'
#' @param obj an object resulting from the execution of NSGA-I, NSGA-II or NSGA-III
#' algorithm
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @return Prints the resulting fitness and when the result of the method-call
#' is assigned to a variable, the fitness is stored as a data frame.
#' See [nsga1-class] [nsga2-class], [nsga3-class] for a description of available
#' slots information.
#'
#' @examples
#' # Where 'out' is an object resulting from the execution of the rmoo.
#' #
#' # fitness_result <- getFitness(out)
#' #
#' # fitness_result
#'
#' @export
#' @docType methods
#' @rdname getFitness-methods
setGeneric("getFitness", function(obj) standardGeneric("getFitness"))


#' Methods for Function 'plot' in Package 'rmoo'
#'
#' Method used to visualize the fitness of the individuals during the execution
#' of the algorithms.
#'
#' The following plots are available:
#'
#' \itemize{
#'  \item "Scatter Plot"
#'  \item "Parallel Coordinate Plot"
#'  \item "Heat Map"
#'  \item "Polar Coordinate"
#' }
#'
#' @param x,y Objects of either class \linkS4class{nsga1},
#'   \linkS4class{nsga2},  or \linkS4class{nsga3}.
#' @param type Type of graph to draw, the graphs can be of the type "scatter",
#' 	"pcp", "heatmap", or "polar"
#' @param ... other arguments passed on to methods
#' \describe{
#'	\item{"optimal"}{An argument passed to the "scatter" plot. A matrix of
#'	dimension equal to the fitness with which they are compared. This value can
#'	only be compared in 2 and 3 dimensional "scatter" plots.}
#' 	\item{"individual"}{An argument passed to the "heatmap" and "polar" plots.
#' 	A vector that represents the fitness of the individuals to be displayed.}
#' }
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @return A graph of the evaluated type.
#'
#' @examples
#' # Where 'out' is an object of class nsga1, nsga2, or nsga3.
#' # The plot method will by default plot a scatter plot.
#' #
#' # plot(out)
#' #
#' # The Parallel Coordinate Plot will be plotted if "pcp" is passed as a parameter to "type".
#' #
#' # plot(out, type="pcp")
#' #
#' # A heat map plot will be plotted if "heatmap" is passed as a parameter to "type"
#' # and a vector with the individuals to plot to "individual"
#' #
#' # plot(out, type = "heatmap", individual = c(1:5))
#' #
#' # A polar coordinate plot will be plotted if "polar" is passed as a parameter to "type"
#' # and a vector with the individuals to plot to "individual"
#' #
#' # plot(out, type = "polar", individual = c(1:5))
#'
#' @export
#' @docType methods
#' @rdname plot-methods
setGeneric("plot", function(x, y, ...) standardGeneric("plot"))

#if (!isGeneric("plot"))
# @name plot-method
NULL
# @name plot-method

#' Methods for Function 'print' in Package 'rmoo'.
#'
#' Method used to print the slots and relevant values of the object.
#'
#' @param x Objects of either class \linkS4class{nsga1},
#'   \linkS4class{nsga2},  or \linkS4class{nsga3}.
#' @param ... other arguments passed on to methods
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @return Print the slots and relevant values of the object.
#'
#' @examples
#' # Where 'out' is an object of class nsga1, nsga2, or nsga3
#' #
#' # print(out)
#'
#' @export
#' @docType methods
#' @rdname print-methods
setGeneric("print", function(x, ...) standardGeneric("print"))

#if (!isGeneric("print"))
# @name print-method
NULL
# @name print-method

#' Methods for Function 'summary' in Package 'rmoo'
#'
#' Method used to summarize the results of the evaluations, passing additional
#' arguments in the summary method the performance metrics is evaluated.
#'
#' @param object Objects of either class \linkS4class{nsga1},
#'   \linkS4class{nsga2},  or \linkS4class{nsga3}.
#' @param ... other arguments passed on to methods. Passing \code{"reference_dirs"}
#' as arguments will evaluate the performance metrics Hypervolumen,
#' Generational Distance, and Inverse Generational Distance.
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @return A summary of the values resulting from the execution of an algorithm.
#'
#' @examples
#' # Where 'out' is an object of class nsga1, nsga2, or nsga3
#' #
#' # summary(out)
#' #
#' # For the evaluation of the metrics, pass the reference point
#' #
#' # ref_points <- generate_reference_points(3,12)
#' # summary(out, reference_dirs = ref_points)
#'
#' @export
#' @docType methods
#' @rdname summary-methods
setGeneric("summary", function(object, ...) standardGeneric("summary"))

# if (!isGeneric("summary"))
# @name summary-method
NULL
# @name summary-method

#' Methods for Function 'progress' in Package 'rmoo'
#'
#' Method used to save the progress of the evaluation results, similar to the
#' summary method. Passing additional arguments to the progress method evaluates
#' performance metrics per iteration. This method cannot be called outside of
#' rmoo execution.
#'
#' @param object Objects of either class \linkS4class{nsga1},
#'   \linkS4class{nsga2},  or \linkS4class{nsga3}.
#' @param ... other arguments passed on to methods. Passing \code{"reference_dirs"}
#' as arguments will evaluate the performance metrics Hypervolumen,
#' Generational Distance, and Inverse Generational Distance.
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @return A list of length equal to the number of iterations, where the progress made during execution is saved.
#'
#' @examples
#' # Where 'out' is an object of class nsga1, nsga2, or nsga3, and callArgs are
#' # the additional arguments passed when calling the rmoo function, for the
#' # evaluation of performance metrics, reference points are expected to be passed
#' # as an argument to reference_dirs.
#' #
#' # progress(object, callArgs)
#' #
#'
#' @export
#' @docType methods
#' @rdname progress-methods
setGeneric("progress", function(object, ...) standardGeneric("progress"))

#' Accessor methods to the metrics evaluated during execution
#'
#' @param obj an object resulting from the execution of NSGA-I, NSGA-II or NSGA-III
#' algorithm. During the execution of the performance metrics must be evaluated.
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @return A dataframe with performance metrics evaluated iteration by iteration.
#'
#' @examples
#' # Where 'out' is an object resulting from the execution of the rmoo.
#' #
#' # metrics_result <- getMetrics(out)
#' #
#' # metrics_result
#'
#' @export
#' @docType methods
#' @rdname getMetrics-methods
setGeneric("getMetrics", function(obj) standardGeneric("getMetrics"))
# if (!isGeneric("progress"))


#' @rdname getPopulation-methods
#' @aliases getPopulation,nsga,nsga-method
setMethod("getPopulation", "nsga",
          function(obj) {
            print(obj@population)
            n_value <- ncol(obj@population)
            population <- data.frame(obj@population)
            colnames(population) <- sprintf("Val_%s",seq(n_value))
            return(invisible(population))
          }
)

# @export
#' @rdname getPopulation-methods
#' @aliases getFitness,nsga,nsga-method
setMethod("getFitness", "nsga",
          function(obj) {
            print(obj@fitness)
            n_value <- ncol(obj@fitness)
            fitness <- data.frame(obj@fitness)
            colnames(fitness) <- sprintf("Fit_%s",seq(n_value))
            return(invisible(fitness))
          }
)


# @export
#' @rdname print-methods
#' @aliases print,nsga,missing-method
setMethod("print", signature(x = "nsga"),
          function(x, ...) {
            # algorithm <- class(object)[1]
            # Print
            cat("Slots Configuration:\n")
            print((slotNames(x)))
            cat("\n#========================================#\n")
            cat("\nTotal iterations: ", x@iter)
            cat("\nPopulation size: ", x@popSize)
            if (x@type == "binary") {
              cat("\nNumber of Bits: ", x@nBits)
            } else{
              cat("\nLower Bounds: ", x@lower)
              cat("\nLower Bounds: ", x@upper)
            }
            cat("\nNumber of Nondominated Front:  ", length(x@f[[1]]))
            cat("\n#========================================#\n")
          }
)

# @export
#' @rdname plot-methods
#' @aliases plot,nsga,missing
setMethod("plot", signature(x="nsga", y="missing"), .get.plotting)

# @export
#' @rdname summary-methods
#' @aliases summary,nsga,nsga-method
setMethod("summary", signature(object = "nsga"),
          function(object, ...) {
            str(object)
          }
)

# @export
#' @rdname progress-methods
#' @aliases progress,nsga,nsga-method
setMethod("progress", signature(object = "nsga"),
          function(object, ...) {
            callArgs <- as.list(...)

            nullRP <- is.null(callArgs$reference_dirs)

            first <- object@f[[1]]
            first_front_fit <- object@fitness[first, ]
            first_front_pop <- object@population[first, ]

            result <- list(Iternation = object@iter,
                           first_front_fit = first_front_fit,
                           first_front_pop = first_front_pop)

            return(invisible(result))
          }
)


# @export
#' @rdname getMetrics-methods
#' @aliases getMetrics,nsga,nsga-method
setMethod("getMetrics", "nsga",
          function(obj) {
            iter <- obj@iter
            out <- data.frame()
            if(!is.null(obj@summary[[1]]$metrics)){
              for (i in seq_len(iter)){
                out <- rbind(out, obj@summary[[i]]$metrics)
              }
            } else {
              stop("No metrics have been evaluated.")
            }
            return(out)
          }
)


# -----------------------------------------------------------------------------
# setMethod("getCrowdingDistance", "nsga2", function(obj) print(obj@crowdingDistance))
# setMethod("getDummyFitness", "nsga1",
#           function(obj) {
#             cat("NSGA-I Dummy Fitness: \n")
#             cat("\n#========================================#\n")
#             print(obj@dumFitness)
#             n_dum <- ncol(obj@dumFitness)
#             dum_Fitness <- data.frame(obj@dumFitness)
#             colnames(dum_Fitness) <- sprintf("FitDummy_%s",seq(n_dum))
#             return(invisible(dum_Fitness))
#           }
# )

# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------


# setMethod("print", "nsga1",
#           function(x=object, y="missing", ...) {
#             algorithm <- class(x)[1]
#             # Print
#             cat("\nSlots Configuration:\n")
#             print(as.list(slotNames(x)))
#             cat("\n#========================================#\n")
#             cat("\nTotal iterations: ", x@iter)
#             cat("\nPopulation size: ", x@popSize)
#             cat("\nLower Bounds: ", x@lower)
#             cat("\nLower Bounds:  ", x@upper)
#             cat("\nDelta Distance (dShare):  ", x@dShare)
#             cat("\nDistance of sharing function:  ", x@deltaDummy)
#             cat("\nNumber of Nondominated Front:  ", length(x@f[[1]]))
#             cat("\n#========================================#\n")
#
#           }
# )

# setMethod("print", "nsga3",
#           function(x=object, y="missing", ...) {
#             # algorithm <- class(object)[1]
#             # Print
#             cat("Slots Configuration:\n")
#             print(as.list(slotNames(x)))
#             cat("\n#========================================#\n")
#             cat("\nTotal iterations: ", x@iter)
#             cat("\nPopulation size: ", x@popSize)
#             cat("\nLower Bounds: ", x@lower)
#             cat("\nUpper Bounds:  ", x@upper)
#             cat("\nEstimated Ideal Point:  ", x@ideal_point)
#             cat("\nEstimated Worst Point:  ", x@worst_point)
#             cat("\nEstimated Nadir Point:  ", x@nadir_point)
#             cat("\nNumber of Nondominated Front:  ", length(x@f[[1]]))
#             cat("\n#========================================#\n")
#           }
# )

# -----------------------------------------------------------------------------

# setMethod("summary", "nsga3",
#           function(object, ...){
#               callArgs <- list(...)
#               nullRP <- is.null(callArgs$reference_dirs)
#
#               # Calculate information for summary
#
#               first <- object@f[[1]]
#               first_front_fit <- object@fitness[first, ]
#               first_front_pop <- object@population[first, ]
#               nadir_point <- object@nadir_point
#
#               #first_dum <- object@dumFitness[first, ] for nsga1 summary method
#
#               if("ecr" %in% rownames(utils::installed.packages())){
#                 if (nullRP) {
#                   cat("Warning: reference points not provided:\n
#                       value necessary to evaluate GD and IGD.")
#
#                 } else{
#                   gd <- ecr::computeGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
#                   igd <- ecr::computeInvertedGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
#                 }
#               }
#
#               if("emoa" %in% rownames(utils::installed.packages())){
#                 if(nullRP) {
#                   cat("Warning: reference points not provided:\n
#                       using the maximum in each dimension to evaluate Hypervolumen")
#                   reference_point <- nadir_point
#                 } else {reference_point <- apply(callArgs$reference_dirs, 2, max)}
#                 hv <- emoa::dominated_hypervolume(points = t(object@fitness[first, ]), ref = reference_point)
#               }
#
#               cat("\nSummary of NSGA-III run")
#               cat("\n#====================================")
#               cat("\nNumber of Objectives evaluated: ", ncol(object@fitness))
#               cat("\nTotal iterations: ", object@iter)
#               cat("\nPopulation size: ", object@popSize)
#               #cat("\nFeasible points found: ", nfeas,paste0("(", signif(100 * nfeas / npts, 3), "%"),"of total)")
#               cat("\nNondominated points found: ", length(first),
#                   paste0("(", signif(100 * length(first) / object@popSize, 3), "%"),
#                   "of total)")
#               cat("\nEstimated ideal point: ", round(object@ideal_point, 3))
#               cat("\nEstimated nadir point: ", round(object@nadir_point, 3))
#               cat("\nMutation Probability: ",
#                   paste0(signif(100 * object@pmutation, 3), "%"))
#               cat("\nCrossover Probability: ",
#                   paste0(signif(100 * object@pcrossover, 3), "%"))
#               if("ecr" %in% rownames(utils::installed.packages())){
#                 if(!nullRP) cat("\nEstimated IGD: ", igd)
#                 if(!nullRP) cat("\nEstimated GD: ", gd)
#               } else cat("\n\nPlease install package 'ecr' to calculate IGD and GD.")
#               if("emoa" %in% rownames(utils::installed.packages())) {
#                 cat("\nEstimated HV: ", hv)
#                 cat("\nRef point used for HV: ", reference_point)
#               } else cat("\n\nPlease install package 'emoa' to calculate hypervolume.")
#               cat("\n#====================================")
#           }
# )

# setMethod("summary", "nsga2",
#           function(object, ...){
#             callArgs <- list(...)
#             nullRP <- is.null(callArgs$reference_dirs)
#
#             # Calculate information for summary
#
#             first <- object@f[[1]]
#             first_front_fit <-
#             first_front_pop <- object@population[first, ]
#             nadir_point <- apply(object@fitness[first, ], 2, max)
#
#             #first_dum <- object@dumFitness[first, ] for nsga1 summary method
#
#             if("ecr" %in% rownames(utils::installed.packages())){
#               if (nullRP) {
#                 cat("Warning: reference points not provided:\n
#                       value necessary to evaluate GD and IGD.")
#
#               } else{
#                 gd <- ecr::computeGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
#                 igd <- ecr::computeInvertedGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
#               }
#             }
#
#             if("emoa" %in% rownames(utils::installed.packages())){
#               if(nullRP) {
#                 cat("Warning: reference points not provided:\n
#                       using the maximum in each dimension to evaluate Hypervolumen")
#                 reference_point <- nadir_point
#               } else {reference_point <- apply(callArgs$reference_dirs, 2, max)}
#               hv <- emoa::dominated_hypervolume(points = t(object@fitness[first, ]), ref = reference_point)
#             }
#
#             cat("\nSummary of NSGA-II run")
#             cat("\n#====================================")
#             cat("\nNumber of Objectives evaluated: ", ncol(object@fitness))
#             cat("\nTotal iterations: ", object@iter)
#             cat("\nPopulation size: ", object@popSize)
#             #cat("\nFeasible points found: ", nfeas,paste0("(", signif(100 * nfeas / npts, 3), "%"),"of total)")
#             cat("\nNondominated points found: ", length(first),
#                 paste0("(", signif(100 * length(first) / object@popSize, 3), "%"),
#                 "of total)")
#             cat("\nCrowding distance bounds: ", c(max(object@crowdingDistance), min(object@crowdingDistance)))
#             #cat("\nEstimated nadir point: ", round(object@nadir_point, 3))
#             cat("\nMutation Probability: ",
#                 paste0(signif(100 * object@pmutation, 3), "%"))
#             cat("\nCrossover Probability: ",
#                 paste0(signif(100 * object@pcrossover, 3), "%"))
#             if("ecr" %in% rownames(utils::installed.packages())){
#               if(!nullRP) cat("\nEstimated IGD: ", igd)
#               if(!nullRP) cat("\nEstimated GD: ", gd)
#             } else cat("\n\nPlease install package 'ecr' to calculate IGD and GD.")
#             if("emoa" %in% rownames(utils::installed.packages())) {
#               cat("\nEstimated HV: ", hv)
#               cat("\nRef point used for HV: ", reference_point)
#             } else cat("\n\nPlease install package 'emoa' to calculate hypervolume.")
#             cat("\n#====================================")
#           }
# )

# setMethod("summary", "nsga1",
#           function(object, ...){
#             callArgs <- list(...)
#             nullRP <- is.null(callArgs$reference_dirs)
#
#             # Calculate information for summary
#
#             first <- object@f[[1]]
#             first_front_fit <-
#             first_front_pop <- object@population[first, ]
#             nadir_point <- apply(object@fitness[first, ], 2, max)
#
#             #first_dum <- object@dumFitness[first, ] for nsga1 summary method
#
#             if("ecr" %in% rownames(utils::installed.packages())){
#               if (nullRP) {
#                 cat("Warning: reference points not provided:\n
#                       value necessary to evaluate GD and IGD.")
#
#               } else{
#                 gd <- ecr::computeGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
#                 igd <- ecr::computeInvertedGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
#               }
#             }
#
#             if("emoa" %in% rownames(utils::installed.packages())){
#               if(nullRP) {
#                 cat("Warning: reference points not provided:\n
#                       using the maximum in each dimension to evaluate Hypervolumen")
#                 reference_point <- nadir_point
#               } else {reference_point <- apply(callArgs$reference_dirs, 2, max)}
#               hv <- emoa::dominated_hypervolume(points = t(object@fitness[first, ]), ref = reference_point)
#             }
#
#             cat("\nSummary of NSGA-I run")
#             cat("\n#====================================")
#             cat("\nNumber of Objectives evaluated: ", ncol(object@fitness))
#             cat("\nTotal iterations: ", object@iter)
#             cat("\nPopulation size: ", object@popSize)
#             #cat("\nFeasible points found: ", nfeas,paste0("(", signif(100 * nfeas / npts, 3), "%"),"of total)")
#             cat("\nNondominated points found: ", length(first),
#                 paste0("(", signif(100 * length(first) / object@popSize, 3), "%"),
#                 "of total)")
#             cat("\nShare Distance: ", object@dShare)
#             cat("\nSharing Values calculated: ", object@deltaDummy)
#             #cat("\nEstimated nadir point: ", round(object@nadir_point, 3))
#             cat("\nMutation Probability: ",
#                 paste0(signif(100 * object@pmutation, 3), "%"))
#             cat("\nCrossover Probability: ",
#                 paste0(signif(100 * object@pcrossover, 3), "%"))
#             if("ecr" %in% rownames(utils::installed.packages())){
#               if(!nullRP) cat("\nEstimated IGD: ", igd)
#               if(!nullRP) cat("\nEstimated GD: ", gd)
#             } else cat("\n\nPlease install package 'ecr' to calculate IGD and GD.")
#             if("emoa" %in% rownames(utils::installed.packages())) {
#               cat("\nEstimated HV: ", hv)
#               cat("\nRef point used for HV: ", reference_point)
#             } else cat("\n\nPlease install package 'emoa' to calculate hypervolume.")
#             cat("\n#====================================")
#           }
# )

# -----------------------------------------------------------------------------

# heat_map <- function(object, ...){
#   if(!all(requireNamespace("ggplot2", quietly = TRUE),
#           requireNamespace("reshape2", quietly = TRUE),
#           requireNamespace("dplyr", quietly = TRUE)))
#     stop("packages 'ggplot2', 'dplyr' and 'reshape2' required for heat map plotting!")
#   callArgs <- list(...)
#   individual <- callArgs$individual
#   fitness <- object@fitness
#
#   if(is.null(individual))
#     stop("Please, define a vector with the individuals to plot")
#
#   if (length(individual) > 10) {
#     cat("Warning: Heatmap plot with more than 10 individuals will not be displayed correctly.\n
#                   The plot will still be displayed.")
#   }
#
#   fitness <- fitness[individual,]
#
#   if (is.null(dim(fitness))){
#     fitness <- t(matrix(fitness))
#   }
#
#   nObj <- ncol(fitness)
#   colnames(fitness) <- sprintf("f_%s",seq(nObj))
#   fitness <- reshape2::melt(fitness)
#   fitness <- dplyr::rename(fitness,
#                            'Pop' = Var1,
#                            'Objective_Value' = value,
#                            'Objective_No' = Var2)
#   ggplot2::ggplot(fitness, aes(x = Objective_No,
#                                y = Pop,
#                                fill = Objective_Value)) +
#     ggplot2::geom_raster() +
#     ggplot2::scale_y_continuous(labels = unique(as.character(fitness$Pop)),
#                                 breaks = unique(fitness$Pop))
# }

# polar <- function(object, ...){
#   if(!all(requireNamespace("ggplot2", quietly = TRUE),
#           requireNamespace("reshape2", quietly = TRUE),
#           requireNamespace("dplyr", quietly = TRUE)))
#     stop("packages 'ggplot2', 'dplyr' and 'reshape2' required for polar coordinate plotting!")
#
#   callArgs <- list(...)
#   individual <- callArgs$individual
#   fitness <- object@fitness
#
#   if(is.null(individual))
#     stop("Please, define a vector with the individuals to plot")
#
#   if (length(individual) > 10) {
#     cat("Warning: Polar Coordinate plot with more than 10 individuals will not be displayed correctly.\n
#                   The plot will still be displayed.")
#   }
#
#   fitness <- fitness[individual,]
#
#   if (is.null(dim(fitness))){
#     fitness <- t(matrix(fitness))
#   }
#
#   nObj <- ncol(fitness)
#   colnames(fitness) <- sprintf("f_%s",seq(nObj))
#   fitness <- reshape2::melt(fitness)
#   fitness <- dplyr::rename(fitness,
#                            'Pop' = Var1,
#                            'Objective_Value' = value,
#                            'Objective_No' = Var2)
#   ggplot2::ggplot(fitness, aes(x = Objective_No,
#                                y = Objective_Value,
#                                fill = Objective_No)) +
#     ggplot2::geom_bar(width = 1, stat="identity") +
#     ggplot2::coord_polar() + ggplot2::theme_light() +
#     ggplot2::facet_wrap(~Pop, nrow = 1)
# }



# .get.plotting <- function(x, y="missing",
#                           type="scatter", ...){
#   switch(type,
#          "scatter" = {
#            scatter(x, ...)
#          },
#          "pcp"  = {
#            pcp(x, ...)
#          },
#          "heatmap" = {
#            heat_map(x, ...)
#          },
#          "polar" = {
#            polar(x, ...)
#          }
#   )
# }

# setMethod("plot", signature(x="nsga1", y="missing"), .get.plotting)
# setMethod("plot", signature(x="nsga2", y="missing"), .get.plotting)
# setMethod("plot", signature(x="nsga3", y="missing"), .get.plotting)

# -----------------------------------------------------------------------------

# .nsga1.progress <- function(object, ...) {
#   callArgs <- as.list(...)
#
#   nullRP <- is.null(callArgs$reference_dirs)
#
#   first <- object@f[[1]]
#   first_front_fit <- object@fitness[first, ]
#   first_front_pop <- object@population[first, ]
#   first_dum <- object@dumFitness[first, ]
#
#   if("ecr" %in% rownames(utils::installed.packages())){
#     if (!nullRP) {
#       gd <- ecr::computeGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
#       igd <- ecr::computeInvertedGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
#     }
#   }
#
#   if("emoa" %in% rownames(utils::installed.packages())){
#     if(!nullRP) {
#       reference_point <- apply(callArgs$reference_dirs, 2, max)
#       hv <- emoa::dominated_hypervolume(points = t(first_front_fit), ref = reference_point)
#     }
#   }
#
#   if(nullRP) {
#     result <- list(`First Front Fit` = first_front_fit,
#                    `First Front Pop` = first_front_pop,
#                    `Dummy Front Fit` = first_dum)
#
#   } else{
#     result <- list(`First Front Fit` = first_front_fit,
#                    `First Front Pop` = first_front_pop,
#                    `Dummy Front Fit` = first_dum,
#                    Hypervolumen = hv,
#                    `Generational Distance` = gd,
#                    `InvertedGenerational Distance` = igd)
#
#   }
#   return(invisible(result))
# }
#
# .nsga2.progress <- function(object, ...) {
#   callArgs <- as.list(...)
#
#   nullRP <- is.null(callArgs$reference_dirs)
#
#   first <- object@f[[1]]
#   first_front_fit <- object@fitness[first, ]
#   first_front_pop <- object@population[first, ]
#   first_cd <- object@crowdingDistance[first, ]
#
#   if("ecr" %in% rownames(utils::installed.packages())){
#     if (!nullRP) {
#       gd <- ecr::computeGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
#       igd <- ecr::computeInvertedGenerationalDistance(t(object@fitness), t(callArgs$reference_dirs))
#     }
#   }
#   if("emoa" %in% rownames(utils::installed.packages())){
#     if(!nullRP) {
#       reference_point <- apply(callArgs$reference_dirs, 2, max)
#       hv <- emoa::dominated_hypervolume(points = t(first_front_fit), ref = reference_point)
#     }
#   }
#
#   if(nullRP) {
#     result <- list(`First Front Fit` = first_front_fit,
#                    `First Front Pop` = first_front_pop,
#                    `Crowding Dist` = first_cd)
#
#   } else{
#     result <- list(`First Front Fit` = first_front_fit,
#                    `First Front Pop` = first_front_pop,
#                    `Crowding Dist` = first_cd,
#                    Hypervolumen = hv,
#                    `Generational Distance` = gd,
#                    `InvertedGenerational Distance` = igd)
#
#   }
#
#   return(invisible(result))
# }
#
#
# .nsga3.progress <- function(object, ...) {
#   first <- object@f[[1]]
#   first_front_fit <- object@fitness[first, ]
#   first_front_pop <- object@population[first, ]
#   ideal_point <- object@ideal_point
#   worst_point <- object@worst_point
#   extreme_points <- object@extreme_points
#
#   if("ecr" %in% rownames(utils::installed.packages())){
#     gd <- ecr::computeGenerationalDistance(t(object@fitness), t(object@reference_points))
#     igd <- ecr::computeInvertedGenerationalDistance(t(object@fitness), t(object@reference_points))
#   }
#   if("emoa" %in% rownames(utils::installed.packages())){
#     hv <- ecr::computeHV(t(object@fitness), ref.point = apply(object@reference_points, 2, max))
#   }
#   if (all((c("ecr", "emoa") %in% rownames(utils::installed.packages())))) {
#     metric <- data.frame(Iternation = object@iter,
#                          Generational_Distance = gd,
#                          Inverse_Generational_Distance = igd,
#                          Hypervolumen = hv)
#     result <- list(first_front_fit = first_front_fit,
#                    first_front_pop = first_front_pop,
#                    ideal_point = ideal_point,
#                    worst_point = worst_point,
#                    extreme_points = extreme_points,
#                    metrics = metric)
#   } else{
#     result <- list(first_front_fit = first_front_fit,
#                    first_front_pop = first_front_pop,
#                    ideal_point = ideal_point,
#                    worst_point = worst_point,
#                    extreme_points = extreme_points)
#   }
#   return(invisible(result))
# }


#setMethod("progress", "nsga1", .nsga1.progress)
#setMethod("progress", "nsga2", .nsga2.progress)
#setMethod("progress", "nsga3", .nsga3.progress)

#' Determination of Multi-layer Reference Points
#'
#' A implementation of Multi-layer Reference Points Generation.
#'
#' The Multi-layer reference point implementation is based on Blank and Deb's
#' pymoo library, the approach generates different layers of references point
#' at different scales, provided by the user.
#'
#' @param ... The different layers provided by the user
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @references J. Blank and K. Deb, "Pymoo: Multi-Objective Optimization in
#' Python," in IEEE Access, vol. 8, pp. 89497-89509, 2020,
#' doi: 10.1109/ACCESS.2020.2990567.
#'
#' Das, Indraneel & Dennis, J. (2000). Normal-Boundary Intersection: A New
#' Method for Generating the Pareto Surface in Nonlinear Multicriteria
#' Optimization Problems. SIAM Journal on Optimization. 8.
#' 10.1137/S1052623496307510.
#'
#' @seealso [generate_reference_points()] and [get_fixed_rowsum_integer_matrix()]
#'
#' @return A matrix with the multi-layer reference points
#' @export
reference_point_multi_layer <- function(...){
  callArgs <- list(...)
  if (var(sapply(callArgs, ncol)) != 0){
    stop("All the input array dimensions must match exactly for the column axis.")
  }
  multi_number <- do.call(rbind, callArgs)
  multi_number <- round(multi_number, digits = 8)
  multi_number <- multi_number[!duplicated(multi_number),]

  return(multi_number)
}

#' Determine the division points on the hyperplane
#'
#' Implementation of the recursive function in Generation of Reference points of
#' Das and Dennis..
#'
#' The implemented Reference Point Generation is based on the Das and Dennis's
#' systematic approach that places points on a normalized hyper-plane which is
#' equally inclined to all objective axes and has an intercept of one on each
#' axis.
#'
#' @param m,h Number of reference points 'h' in M-objective problems
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @references K. Deb and H. Jain, 'An Evolutionary Many-Objective Optimization
#' Algorithm Using Reference-Point-Based Nondominated Sorting Approach, Part I:
#' Solving Problems With Box Constraints,' in IEEE Transactions on Evolutionary
#' Computation, vol. 18, no. 4, pp. 577-601, Aug. 2014,
#' doi: 10.1109/TEVC.2013.2281535.
#'
#' Das, Indraneel & Dennis, J.. (2000). Normal-Boundary Intersection: A New
#' Method for Generating the Pareto Surface in Nonlinear Multicriteria
#' Optimization Problems. SIAM Journal on Optimization. 8.
#' 10.1137/S1052623496307510.
#'
#' @seealso [non_dominated_fronts()] and [generate_reference_points()]
#'
#' @return A matrix with the reference points uniformly distributed.
#' @export

#' @export
get_fixed_rowsum_integer_matrix <- function(m, h) {
    if (m < 1) {
        stop("M cannot be less than 1.")
    }
    if (floor(m) != m) {
        stop("M must be an integer.")
    }
    if (m == 1) {
        a <- h
        return(a)
    }
    a <- c()
    for (i in 0:h) {
        b <- get_fixed_rowsum_integer_matrix(m - 1, h - i)
        c <- cbind(i, b)
        colnames(c) <- NULL
        a <- rbind(a, c)
        rownames(a) <- NULL
    }
    return(a)
}

#' Calculation of Crowding Distance
#'
#' A Crowded-comparison approach.
#'
#' The crowded-comparison operator guides the selection process at the various
#' stages of the algorithm toward a uniformly spread-out Pareto-optimal front
#'
#' @param object,nObj An object of class 'nsga2', usually resulting from a call
#' to function nsga2. Fitness Function Objective Numbers
#'
#' @author Francisco Benitez
#' \email{benitezfj94@gmail.com}
#'
#' @references K. Deb, A. Pratap, S. Agarwal and T. Meyarivan, 'A fast and
#' elitist multiobjective genetic algorithm: NSGA-II,' in IEEE Transactions on
#' Evolutionary Computation, vol. 6, no. 2, pp. 182-197, April 2002,
#' doi: 10.1109/4235.996017.
#'
#' @seealso [non_dominated_fronts()]
#'
#' @return A vector with the crowding-distance between individuals of a population.
#' @export
crowding_distance <- function(object, nObj) {
  nFront <- length(object@f)
  popSize <- nrow(object@population)
  crowding <- matrix(NA, nrow = popSize)
  for (i in seq_len(nFront)) {
    f <- object@f[[i]]
    n <- length(f)
    costs <- object@fitness[f, ]
    d <- ecr::computeCrowdingDistance(t(costs))
    for (i in seq_len(n)) {
      crowding[f[i]] <- d[i]
    }
  }
  return(crowding)
}

# crowding_distance <- function(object, nObj) {
#   nFront <- length(object@f)
#   popSize <- nrow(object@population)
#   deltaf <- apply(object@fitness, 2, max) - apply(object@fitness, 2, min)
#   crowding <- matrix(NA, nrow = popSize)
#   for (i in seq_len(nFront)) {
#     f <- object@f[[i]]
#     n <- length(f)
#     costs <- object@fitness[f, ]
#     d <- matrix(0, nrow = n, ncol = nObj)
#     for (j in seq_len(nObj)) {
#       if (n > 1) {
#         ord <- order(costs[, j])
#         srt <- costs[ord, ]
#         d[ord[1], j] <- Inf
#         if (n > 2) {
#           for (k in 2:(n - 1)) {
#             d[ord[k], j] <- abs(srt[(k + 1), j] - srt[(k - 1), j]) / abs(deltaf[j])
#           }
#         }
#         d[ord[n], j] <- Inf
#       } else {
#         costs <- matrix(costs, 1)
#         ord <- order(costs[, 1])
#         srt <- costs[ord, ]
#         d[ord[1], j] <- Inf
#       }
#     }
#     for (i in seq_len(n)) {
#       crowding[f[i]] <- sum(d[i, ])
#     }
#   }
#   return(crowding)
# }




# calc_crowding_distance <- function(F) {
#   popSize <- nrow(F)
#   n_obj <- ncol(F)
#
#   F_sorted <- matrix(nrow = n_points, ncol = n_obj)
#   dist_to_last_sorted <- matrix(nrow = n_points, ncol = n_obj)
#   dist_to_next_sorted <- matrix(nrow = n_points, ncol = n_obj)
#
#   for (j in 1:n_obj) {
#     F_sorted[,j] <- F[order(F[,j]), j]
#   }
#
#   # calculate the distance from each point to the last and next
#   dist <- rbind(F_sorted, rep(Inf, n_obj)) - rbind(rep(-Inf, n_obj), F_sorted)
#
#   # calculate the norm for each objective - set to NaN if all values are equal
#   norm <- apply(F_sorted, 2, max) - apply(F_sorted, 2, min)
#   norm[norm == 0] <- NA
#
#   # prepare the distance to last and next vectors
#   dist_to_last <- dist_to_next <- dist
#   dist_to_last <- dist_to_last[-nrow(dist_to_last),] / norm
#   dist_to_next <- dist_to_next[-1,] / norm
#
#   # if we divide by zero because all values in one columns are equal replace by none
#   dist_to_last[is.na(dist_to_last)] <- 0.0
#   dist_to_next[is.na(dist_to_next)] <- 0.0
#
#   # sum up the distance to next and last and norm by objectives - also reorder from sorted list
#   J <- apply(I, 2, order)
#
#   for (j in 1:n_obj) {
#     dist_to_last_sorted[,j] <- dist_to_last[J[,j], j]
#     dist_to_next_sorted[,j] <- dist_to_next[J[,j], j]
#   }
#
#   cd <- rowSums(dist_to_last_sorted + dist_to_next_sorted) / n_obj
#
#   return(cd)
# }



# library(Rcpp)
# library(RcppArmadillo)
# # Define the C++ function for crowding distance calculation
# cppFunction('NumericVector crowding_distance_cpp(List object, int nObj) {
#   int nFront = object.size();
#   int popSize = object["population"].nrow();
#   NumericMatrix fitness = object["fitness"];
#   NumericVector deltaf = apply(fitness, 2, max) - apply(fitness, 2, min);
#   NumericVector crowding(popSize, 0.0);
#
#   for (int front_idx = 0; front_idx < nFront; ++front_idx) {
#     IntegerVector front = object["f"][front_idx];
#     int n = front.size();
#     if (n > 1) {
#       NumericMatrix costs = fitness(front - 1, _);
#       IntegerVector ord = order(costs(_, 0));
#       NumericMatrix srt = costs(ord, _);
#       NumericMatrix d(n, nObj);
#
#       d(ord[0], _) = R_PosInf;
#       d(ord[n - 1], _) = R_PosInf;
#
#       for (int k = 1; k < n - 1; ++k) {
#         d(ord[k], _) = abs(srt(k + 1, _) - srt(k - 1, _)) / deltaf;
#       }
#       crowding[front - 1] = rowSums(d);
#     }
#   }
#   return crowding;
# }')
#
# # Define the optimized R function using Rcpp
# crowding_distance <- function(object, nObj) {
#   crowding <- crowding_distance_cpp(object, nObj)
#   return(matrix(crowding, ncol = 1))
# }

# crowding_distance <- function(object, nObj) {
#   nFront <- length(object@f)
#   popSize <- nrow(object@population)
#   deltaf <- apply(object@fitness, 2, max) - apply(object@fitness, 2, min)
#   crowding <- numeric(popSize)
#
#   for (front_idx in seq_len(nFront)) {
#     front <- object@f[[front_idx]]
#     n <- length(front)
#     if (n > 1) {
#       costs <- object@fitness[front, ]
#       ord <- order(costs[, 1])
#       srt <- costs[ord, ]
#       d <- matrix(0, nrow = n, ncol = nObj)
#
#       d[ord[1], ] <- Inf
#       d[ord[n], ] <- Inf
#
#       for (k in 2:(n - 1)) {
#         d[ord[k], ] <- abs(srt[(k + 1), ] - srt[(k - 1), ]) / abs(deltaf)
#       }
#       crowding[front] <- rowSums(d)
#     }
#   }
#   crowding <- matrix(crowding, ncol=1)
#   return(crowding)
# }

# Define the Rcpp function
# cppFunction('
# NumericVector crowding_distance_rcpp(List object_list, int nObj) {
#   int nFront = object_list.size();
#   NumericVector crowding(popSize, 0.0);
#   NumericMatrix fitness = object_list["fitness"];
#   IntegerVector population = object_list["population"];
#   NumericVector deltaf = colMax(fitness) - colMin(fitness);
#
#   for (int i = 0; i < nFront; i++) {
#     IntegerVector f = population[i];
#     int n = f.size();
#     NumericMatrix costs = fitness(f - 1, _);
#     NumericMatrix d(n, nObj);
#
#     if (n > 1) {
#       NumericVector ord = order(costs(_, 0));
#       NumericMatrix srt = costs(ord, _);
#       d(ord[0], _) = R_PosInf;
#       d(ord[n - 1], _) = R_PosInf;
#
#       if (n > 2) {
#         for (int k = 1; k < n - 1; k++) {
#           d(ord[k], _) = abs(srt(k + 1, _) - srt(k - 1, _)) / deltaf;
#         }
#       }
#     } else {
#       NumericMatrix costs_mat = matrix(costs, 1);
#       NumericVector ord = order(costs_mat(_, 0));
#       NumericMatrix srt = costs_mat(ord, _);
#       d(ord[0], _) = R_PosInf;
#     }
#
#     for (int j = 0; j < n; j++) {
#       crowding[f[j] - 1] += sum(d(j, _));
#     }
#   }
#
#   return crowding;
# }')

#' Calculation of Modified Crowding Distance
#'
#' A Crowded-comparison approach.
#'
#' The crowded-comparison operator maintain diversity in the Pareto front
#' during multi-objective optimization. This version uses a reference point-based
#' normalization and preference distance strategy.
#'
#' @param object An object of class 'rnsga2', typically from a call to r-nsga2.
#'               Must contain fitness, population, fronts, popSize, and reference_points.
#' @param epsilon Minimum allowed distance between solutions to avoid duplicates.
#' @param weights A numeric vector of weights for preference distance (default is equal weights).
#' @param normalization Type of normalization to apply: `"ever"`, `"front"`, or `"no"`.
#' @param extreme_points_as_ref_dirs Logical; whether to use extreme points as reference directions.
#'
#' @author Francisco Benitez
#'
#' @references Kalyanmoy Deb and J. Sundar (2006). GECCO '06. doi:10.1145/1143997.1144112
#'
#' @seealso [rnsga2()]
#'
#' @return A list with:
#' \describe{
#'   \item{survivors}{Indices of selected individuals}
#'   \item{indexmin}{Index of individuals with minimum scalarizing value (optional)}
#'   \item{reference_points}{Updated reference points matrix}
#' }
#' @export
modifiedCrowdingDistance <- function(object,
                                     epsilon,
                                     weights = NULL,
                                     normalization = "front",
                                     extreme_points_as_ref_dirs = FALSE) {
  fitness          <- object@fitness
  population       <- object@population
  nObj             <- ncol(fitness)
  fronts           <- object@f
  nFront           <- length(fronts)
  popSize          <- object@popSize
  reference_points <- object@reference_points

  # if (is.null(weights)) {
  #   weights <- rep((1/nObj),nObj)
  # }
  weights <- weights %||% rep(1 / nObj, nObj)

  ideal_point <- rep(Inf,  nObj)
  nadir_point <- rep(-Inf, nObj)

  #Normalization
  if (normalization == "ever") {
    ideal_point <- apply(rbind(ideal_point, fitness), 2, min)
    nadir_point <- apply(rbind(ideal_point, fitness), 2, max)
  } else if (normalization == "front" && length(fronts[[1]]) > 1) {
    front_fitness <- fitness[fronts[[1]], , drop = FALSE]
    ideal_point   <- apply(front_fitness, 2, min)
    nadir_point   <- apply(front_fitness, 2, max)
  } else if (normalization == "no") {
    ideal_point <- rep(1, nObj)
    nadir_point <- rep(0, nObj)
  }

  if (extreme_points_as_ref_dirs) {
    ps <- PerformScalarizing(population    = population[unlist(fronts), ],
                             fitness       = fitness[unlist(fronts), ],
                             smin          = object@smin,
                             extreme_points = reference_points,
                             ideal_point   = ideal_point)
    reference_points <- rbind(reference_points, ps$extremepoint)
    smin             <- ps$indexmin
  } else {
    smin <- NULL
  }

  n_remaining <- popSize
  survivors <- integer(0)

  distance_to_ref_points <- calc_norm_pref_distance(
    fitness     = fitness,
    ref_points  = reference_points,
    weight      = weights,
    ideal_point = ideal_point,
    nadir_point = nadir_point
  )

  for (i in seq_len(nFront)) {
    #cat(i, " Iter: ", object@iter, "\n")
    n_remaining <- popSize - length(survivors)
    if (n_remaining == 0L) break

    fi <- fronts[[i]]
    nf <- length(fi)


    if (nf > 1L) {
      # rank_by_distance <- apply(apply(distance_to_ref_points[fronts[[i]],], 2, order), 2, order)
      rank_by_distance <- apply(
        apply(as.matrix(distance_to_ref_points[fi, ]), 2, order),
        2, order
      ) #We use as.matrix in the case when the distance ob to the reference points has one dimension
      ref_point_of_best_rank <- max.col(-rank_by_distance, ties.method = "first")

      # ref_point_of_best_rank <- vapply(seq_len(nrow(rank_by_distance)),
      #                                  function(i) {
      #                                    idx <- which.min(rank_by_distance[i, ])
      #                                    if (length(idx) == 0L) 1L else idx
      #                                  },
      #                                  integer(1L))
    }else{
      rank_by_distance       <- t(order(order(distance_to_ref_points[fi, ])))
      ref_point_of_best_rank <- which.min(rank_by_distance)
    }
    ranking <- rank_by_distance[cbind(seq_len(nf), ref_point_of_best_rank)]

    if (nf < n_remaining) {
      I <- seq_len(nf)
    } else{
      dist_to_others <- calc_norm_pref_distance(
        fitness     = fitness[fi, ],
        ref_points  = fitness[fi, ],
        weight      = weights,
        ideal_point = ideal_point,
        nadir_point = nadir_point
      )
      diag(dist_to_others) <- Inf

      crowding     <- rep(NA_real_, nf)
      not_selected <- order(ranking)

      while (length(not_selected) > 0L) {
        idx        <- not_selected[1L]
        crowding[idx] <- ranking[idx]
        to_remove  <- idx

        dist  <- dist_to_others[idx, not_selected]
        group <- not_selected[which(dist < epsilon)[1L]]

        if (!is.na(group)) {
          crowding[group] <- ranking[group] + round(nf / 2)
          to_remove       <- c(to_remove, group)
        }
        not_selected <- setdiff(not_selected, to_remove)

      }
      I <- order(crowding)[seq_len(n_remaining)]

    }
    survivors <- c(survivors, fi[I])

  }
  list(survivors        = survivors,
       indexmin         = smin,
       reference_points = reference_points)
}

#' Calculate Normalized Preference Distance

#' Computes the weighted normalized Euclidean distance between a set of fitness
#' vectors and a set of reference points.

#' @param fitness A matrix of fitness values.
#' @param ref_points A matrix of reference points.
#' @param weight A numeric vector of weights for each objective.
#' @param ideal_point A numeric vector of ideal point values.
#' @param nadir_point A numeric vector of nadir point values.
#'
#' @return A matrix of distances where element (i, j) is the distance from
#'        fitness to ref_points.
#' @export
calc_norm_pref_distance <- function(fitness, ref_points, weight, ideal_point, nadir_point) {
  fitness    <- as.matrix(fitness)
  ref_points <- as.matrix(ref_points)
  p          <- ncol(fitness)
  # Calculate the denominator
  denom <- nadir_point - ideal_point # New
  denom[denom == 0] <- 1e-12

  # Calculate the difference between fitness and ref_points
  D2 <- Reduce(`+`, lapply(seq_len(p), function(j) {
    weight[j] * outer(fitness[, j], ref_points[, j], "-")^2 / denom[j]^2
  }))


  # denom[which(denom == 0)] <- 1e-12

  # Calculate the normalized preference distance
  sqrt(D2 * p)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geneticoperator.R
\name{rmooreal_sbxCrossover}
\alias{rmooreal_sbxCrossover}
\alias{rmoo_Crossover}
\alias{rmoo_spCrossover}
\alias{rmoobin_spCrossover}
\alias{rmooreal_spCrossover}
\alias{rmooint_spCrossover}
\alias{rmooperm_oxCrossover}
\title{Crossover Operators in Non-Dominated Genetic Algorithms}
\usage{
rmooreal_sbxCrossover(object, parents, eta = 20, indpb = 0.5)

rmoo_spCrossover(object, parents)

rmoobin_spCrossover(object, parents)

rmooreal_spCrossover(object, parents)

rmooint_spCrossover(object, parents)

rmooperm_oxCrossover(object, parents)
}
\arguments{
\item{object}{An object of class \code{"nsga"}, \code{"nsga2"}, or
\code{"nsga3"}, usually from a call to \code{\link{nsga}},
\code{\link{nsga2}}, or \code{\link{nsga3}}.}

\item{parents}{A two-element integer vector indexing the parents from the
current population.}

\item{eta}{The distribution index. A higher eta produces offspring closer to
the parents, while a lower eta allows for larger differences.}

\item{indpb}{The probability of a particular gene being crossed.}
}
\value{
A list with two elements:
\describe{
\item{children}{A matrix of dimension 2 × nVars containing the
generated offspring.}
\item{fitness}{A 2 × nObj matrix of \code{NA} values, indicating that
offspring fitness has not yet been evaluated.}
}
}
\description{
Functions implementing crossover operators for non-dominated
genetic algorithms. \code{rmoo_spCrossover} (and its typed variants
\code{rmoobin_}, \code{rmooreal_}, \code{rmooint_}) performs single-point
crossover; \code{rmooreal_sbxCrossover} performs simulated binary
crossover; \code{rmooperm_oxCrossover} performs order crossover for
permutation representations.
}
\references{
Scrucca, L. (2017) On some extensions to 'GA' package: hybrid
optimisation, parallelisation and islands evolution. \emph{The R Journal},
9/1, 187--206, \doi{10.32614/RJ-2017-008}.
}
\seealso{
\code{\link[=nsga]{nsga()}}, \code{\link[=nsga2]{nsga2()}},
\code{\link[=nsga3]{nsga3()}}
}
\author{
Francisco Benitez
}

\name{update_points}
\alias{update_points}
%
\alias{UpdateIdealPoint}
\alias{UpdateWorstPoint}
\alias{PerformScalarizing}
\alias{get_nadir_point}

\title{Adaptive normalization of population members}

\description{Functions to scalarize the members of the population to locate them in a normalized hyperplane, finding the ideal point, nadir point, worst point and the extreme points.}

\usage{
  UpdateIdealPoint(object, nObj)
  UpdateWorstPoint(object, nObj)
  PerformScalarizing(population, fitness, smin, extreme_points, ideal_point)
  get_nadir_point(object)
}

\arguments{
  \item{object}{An object of class \code{"nsga3"}.}
  
  \item{nObj}{numbers of objective values of the function to evaluate.}
  
  \item{population}{individuals of the population until last front.}

  \item{fitness}{objective values of the population until last front.}

  \item{smin}{Achievement Escalation Function Index.}

  \item{extreme_points}{Extreme points of the previous generation to upgrade.}

  \item{ideal_point}{Ideal point of the current generation to translate objectives.}
}

\value{Return scalarized objective values in a normalized hyperplane.}

\references{
  J. Blank and K. Deb, "Pymoo: Multi-Objective Optimization in Python," in IEEE Access, vol. 8, pp. 89497-89509, 2020, doi: 10.1109/ACCESS.2020.2990567.

  K. Deb and H. Jain, "An Evolutionary Many-Objective Optimization Algorithm Using Reference-Point-Based Nondominated Sorting Approach, Part I: Solving Problems With Box Constraints," in IEEE Transactions on Evolutionary Computation, vol. 18, no. 4, pp. 577-601, Aug. 2014, doi: 10.1109/TEVC.2013.2281535.
}

\author{Francisco Benitez}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nsga3.R
\name{nsga3}
\alias{nsga3}
\title{Non-Dominated Sorting in Genetic Algorithms III}
\usage{
nsga3(
  type = c("binary", "real-valued", "permutation"),
  fitness,
  ...,
  lower,
  upper,
  nBits,
  population = rmooControl(type)$population,
  selection = rmooControl(type)$selection,
  crossover = rmooControl(type)$crossover,
  mutation = rmooControl(type)$mutation,
  popSize = 50,
  nObj = NULL,
  n_partitions = NULL,
  pcrossover = 0.8,
  pmutation = 0.1,
  reference_dirs = generate_reference_points,
  maxiter = 100,
  run = maxiter,
  maxFitness = Inf,
  names = NULL,
  suggestions = NULL,
  parallel = FALSE,
  monitor = if (interactive()) rmooMonitor else FALSE,
  summary = FALSE,
  seed = NULL
)
}
\arguments{
\item{type}{the type of genetic algorithm to be run depending on the nature
of decision variables. Possible values are:
\describe{
\item{\code{"binary"}}{for binary representations of decision variables.}
\item{\code{"real-valued"}}{for optimization problems where the decision
variables are floating-point representations of real numbers.}
\item{\code{"permutation"}}{for problems that involves reordering of a list
of objects.}
}}

\item{fitness}{the fitness function, any allowable R function which takes as
input an individual string representing a potential solution, and returns a
numerical value describing its “fitness”.}

\item{...}{additional arguments to be passed to the fitness function. This
allows to write fitness functions that keep some variables fixed during the
search}

\item{lower}{a vector of length equal to the decision variables providing the
lower bounds of the search space in case of real-valued or permutation
encoded optimizations.}

\item{upper}{a vector of length equal to the decision variables providing the
upper bounds of the search space in case of real-valued or permutation
encoded optimizations.}

\item{nBits}{a value specifying the number of bits to be used in binary
encoded optimizations.}

\item{population}{an R function for randomly generating an initial population.
See \code{\link[=rmoo_Population]{rmoo_Population()}} for available functions.}

\item{selection}{an R function performing selection, i.e. a function which
generates a new population of individuals from the current population
probabilistically according to individual fitness.}

\item{crossover}{an R function performing crossover, i.e. a function which
forms offsprings by combining part of the
genetic information from their parents. See \code{\link[=rmoo_Crossover]{rmoo_Crossover()}}
for available functions.}

\item{mutation}{an R function performing mutation, i.e. a function which
randomly alters the values of some genes in a parent chromosome.
See \code{\link[=rmoo_Mutation]{rmoo_Mutation()}} for available functions.}

\item{popSize}{the population size.}

\item{nObj}{number of objective in the fitness function.}

\item{n_partitions}{Partition number of generated reference points}

\item{pcrossover}{the probability of crossover between pairs of chromosomes.
Typically this is a large value and by default is set to 0.8.}

\item{pmutation}{the probability of mutation in a parent chromosome. Usually
mutation occurs with a small probability, and by default is set to 0.1.}

\item{reference_dirs}{Function to generate reference points using Das and
Dennis approach or matrix with supplied reference points.}

\item{maxiter}{the maximum number of iterations to run before the NSGA search
is halted.}

\item{run}{the number of consecutive generations without any improvement in
the best fitness value before the NSGA is stopped}

\item{maxFitness}{the upper bound on the fitness function after that the NSGA
search is interrupted.}

\item{names}{a vector of character strings providing the names of decision
variables.}

\item{suggestions}{a matrix of solutions strings to be included in the initial
population. If provided the number of columns must match the number of
decision variables.}

\item{parallel}{An optional argument which allows to specify if the NSGA-II
should be run sequentially or in parallel.}

\item{monitor}{a logical or an R function which takes as input the current
state of the nsga-class object and show the evolution of the search.
By default, for interactive sessions the function rmooMonitor prints the
average and best fitness values at each iteration. If set to plot these
information are plotted on a graphical device. Other functions can be written
by the user and supplied as argument. In non interactive sessions, by default
monitor = FALSE so any output is suppressed.}

\item{summary}{If there will be a summary generation after generation.}

\item{seed}{an integer value containing the random number generator state.
This argument can be used to replicate the results of a NSGA search. Note
that if parallel computing is required, the doRNG package must be installed.}
}
\value{
Returns an object of class nsga3-class. See \linkS4class{nsga3} for a
description of available slots information.
}
\description{
Minimization of a fitness function using non-dominated sorting genetic
algorithms - III (NSGA-IIIs). Multiobjective evolutionary algorithms
}
\details{
The Non-dominated genetic algorithms III is a meta-heuristic proposed by
K. Deb and H. Jain in 2013.
The purpose of the algorithms is to find an efficient way to optimize
multi-objectives functions (more than three).
}
\examples{
#Example 1
#Two Objectives - Real Valued
zdt1 <- function (x) {
 if (is.null(dim(x))) {
   x <- matrix(x, nrow = 1)
 }
 n <- ncol(x)
 g <- 1 + rowSums(x[, 2:n, drop = FALSE]) * 9/(n - 1)
 return(cbind(x[, 1], g * (1 - sqrt(x[, 1]/g))))
}

#Not run
\dontrun{
result <- nsga3(type = "real-valued",
                fitness = zdt1,
                lower = c(0,0),
                upper = c(1,1),
                popSize = 100,
                nObj = 2,
                n_partitions = 100,
                monitor = FALSE,
                maxiter = 500)
}

#Example 2
#Three Objectives - Real Valued
dtlz1 <- function (x, nobj = 3, ...){
    if (is.null(dim(x))) {
        x <- matrix(x, 1)
    }
    n <- ncol(x)
    y <- matrix(x[, 1:(nobj - 1)], nrow(x))
    z <- matrix(x[, nobj:n], nrow(x))
    g <- 100 * (n - nobj + 1 + rowSums((z - 0.5)^2 - cos(20 * pi * (z - 0.5))))
    tmp <- t(apply(y, 1, cumprod))
    tmp <- cbind(t(apply(tmp, 1, rev)), 1)
    tmp2 <- cbind(1, t(apply(1 - y, 1, rev)))
    f <- tmp * tmp2 * 0.5 * (1 + g)
    return(f)
}

#Not Run
\dontrun{
result <- nsga3(type = "real-valued",
                fitness = dtlz1,
                lower = c(0,0,0),
                upper = c(1,1,1),
                popSize = 92,
                nObj = 3,
                n_partitions = 12,
                monitor = FALSE,
                maxiter = 500)
}

}
\references{
K. Deb and H. Jain, "An Evolutionary Many-Objective Optimization
Algorithm Using Reference-Point-Based Nondominated Sorting Approach, Part I:
Solving Problems With Box Constraints," in IEEE Transactions on Evolutionary
Computation, vol. 18, no. 4, pp. 577-601, Aug. 2014,
doi: 10.1109/TEVC.2013.2281535.

Scrucca, L. (2017) On some extensions to 'GA' package: hybrid optimisation,
parallelisation and islands evolution. The R Journal, 9/1, 187-206.
doi: 10.32614/RJ-2017-008
}
\seealso{
\code{\link[=nsga]{nsga()}}, \code{\link[=nsga2]{nsga2()}}
}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parallel.R
\name{stopParallel}
\alias{stopParallel}
\title{Stop Parallel Backend}
\usage{
stopParallel(cluster, ...)
}
\arguments{
\item{cluster}{A cluster object, typically retrieved from \code{attr(parallel, "cluster")}}

\item{...}{Additional arguments (currently unused).}
}
\value{
\code{invisible(NULL)}, used for side effects.
}
\description{
Stops the parallel backend and reverts to sequential execution.
}
\examples{
\dontrun{
  cl <- startParallel()
  stopParallel(attr(cl, "cluster"))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmoo.R
\docType{package}
\name{rmoo-package}
\alias{rmoo-package}
\title{rmoo: Multi-Objective Optimization in R}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

The 'rmoo' package is a framework for multi- and many-objective optimization, which allows researchers and users versatility in parameter configuration, as well as tools for analysis, replication and visualization of results. The 'rmoo' package was built as a fork of the 'GA' package by Luca Scrucca(2017) \doi{10.32614/RJ-2017-008} and implementing the Non-Dominated Sorting Genetic Algorithms proposed by K. Deb's.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/Evolutionary-Optimization-Laboratory/rmoo/}
  \item Report bugs at \url{https://github.com/Evolutionary-Optimization-Laboratory/rmoo/issues/}
}

}
\author{
\strong{Maintainer}: Francisco Benitez \email{benitezfj94@gmail.com}

Authors:
\itemize{
  \item Diego P. Pinto-Roa \email{dpinto@pol.una.py} (\href{https://orcid.org/0000-0003-2479-9876}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmoo_main.R
\name{rmoo_main}
\alias{rmoo_main}
\alias{rmoo}
\alias{rmoo,rmoo-main,rmoo-function}
\title{R Multi-Objective Optimization Main Function}
\usage{
rmoo(
  type = c("binary", "real-valued", "permutation", "discrete"),
  algorithm = c("NSGA-II", "NSGA-III", "R-NSGA-II"),
  fitness,
  ...,
  lower,
  upper,
  nBits,
  nvars,
  population = rmooControl(type)$population,
  selection = rmooControl(type)$selection,
  crossover = rmooControl(type)$crossover,
  mutation = rmooControl(type)$mutation,
  pcrossover = 0.8,
  pmutation = 0.1,
  popSize = 50,
  maxiter = 100,
  nObj = NULL,
  names = NULL,
  suggestions = NULL,
  monitor = if (interactive()) rmooMonitor else FALSE,
  parallel = FALSE,
  summary = FALSE,
  seed = NULL,
  reference_dirs = NULL,
  epsilon = 0.001,
  normalization = NULL,
  extreme_points_as_ref_dirs = FALSE,
  weights = NULL
)
}
\arguments{
\item{type}{the type of genetic algorithm to be run depending on the nature
of decision variables. Possible values are:
\describe{
\item{'binary'}{for binary representations of decision variables.}
\item{'real-valued'}{for optimization problems where the decision
variables are floating-point representations of real numbers.}
\item{'permutation'}{for problems that involves reordering of a list of
objects.}
\item{'discrete'}{for discrete representations of decision variables.}
}}

\item{algorithm}{the type of genetic algorithm to be run depending on the nature
of decision variables. Possible values are:
\describe{
\item{'NSGA-II'}{for .}
\item{'NSGA-III'}{for .}
\item{'R-NSGA-II'}{for .}
}}

\item{fitness}{the fitness function, any allowable R function which takes as
input an individual string representing a potential solution, and returns a
numerical value describing its 'fitness'.}

\item{...}{argument in which all the values necessary for the configuration
will be passed as parameters. The user is encouraged to see the documentations.}

\item{lower}{a vector of length equal to the decision variables providing the
lower bounds of the search space in case of real-valued or permutation
encoded optimizations.}

\item{upper}{a vector of length equal to the decision variables providing the
upper bounds of the search space in case of real-valued or permutation
encoded optimizations.}

\item{nBits}{a value specifying the number of bits to be used in binary
encoded optimizations.}

\item{nvars}{a value .}

\item{population}{an R function for randomly generating an initial population.
See \code{\link[=rmoo_Population]{rmoo_Population()}} for available functions.}

\item{selection}{an R function performing selection, i.e. a function which
generates a new population of individuals from the current population
probabilistically according to individual fitness.}

\item{crossover}{an R function performing crossover, i.e. a function which
forms offsprings by combining part of the genetic information from their
parents. See \code{\link[=rmoo_Crossover]{rmoo_Crossover()}} for available functions.}

\item{mutation}{an R function performing mutation, i.e. a function which
randomly alters the values of some genes in a parent chromosome.
See \code{\link[=rmoo_Mutation]{rmoo_Mutation()}} for available functions.}

\item{pcrossover}{the probability of crossover between pairs of chromosomes.
Typically this is a large value and by default is set to 0.8.}

\item{pmutation}{the probability of mutation in a parent chromosome. Usually
mutation occurs with a small probability, and by default is set to 0.1.}

\item{popSize}{the population size.}

\item{maxiter}{the maximum number of iterations to run before the NSGA search
is halted.}

\item{nObj}{number of objective in the fitness function.}

\item{names}{a vector of character strings providing the names of decision
variables.}

\item{suggestions}{a matrix of solutions strings to be included in the
initial population. If provided the number of columns must match the number
of decision variables.}

\item{monitor}{a logical or an R function which takes as input the current
state of the nsga-class object and show the evolution of the search. By
default, for interactive sessions the function rmooMonitor prints the average
and best fitness values at each iteration. If set to plot these information
are plotted on a graphical device. Other functions can be written by the user
and supplied as argument. In non interactive sessions, by default
monitor = FALSE so any output is suppressed.}

\item{parallel}{An optional argument which allows to specify if the NSGA-II
should be run sequentially or in parallel.}

\item{summary}{If there will be a summary generation after generation.}

\item{seed}{an integer value containing the random number generator state.
This argument can be used to replicate the results of a NSGA search. Note
that if parallel computing is required, the doRNG package must be installed.}

\item{reference_dirs}{Function to generate reference points using Das and
Dennis approach or matrix with supplied reference points.}

\item{epsilon}{controls the extent of obtained solutions by grouping all
solutions that have a normalized difference sum in objective values of epsilon or less.}

\item{normalization}{of the ideal points and nadir. They can be:
\describe{
\item{'ever'}{.}
\item{'front'}{.}
\item{'no'}{.}
}}

\item{extreme_points_as_ref_dirs}{flag to use extreme points as reference points.}

\item{weights}{vector specifies the importance of one objective function over
the other, by default all objectives have equal weights.
of \code{\link[=nsga2]{nsga2()}}, \code{\link[=rnsga2]{rnsga2()}}, \code{\link[=nsga3]{nsga3()}} in which the necessary parameters for each
algorithm are cited, in addition, the chosen strategy to execute must be
passed as an argument. This can be seen more clearly in the examples.}
}
\value{
Returns an object of class nsga2-class, rnsga2-class or nsga3-class.
See \linkS4class{nsga2}, \linkS4class{rnsga2}, \linkS4class{nsga3} for a description of
available slots information.
}
\description{
Main function of rmoo, based on the parameters it will call the different
algorithms implemented in the package. Optimization algorithms will minimize
a fitness function. For more details of the algorithms
see \code{\link[=nsga2]{nsga2()}}, \code{\link[=nsga3]{nsga3()}}, \code{\link[=rnsga2]{rnsga2()}}.
}
\details{
Multi- and Many-Optimization of a fitness function using Non-dominated
Sorting Genetic Algorithms. The algorithms currently implemented by rmoo
are: NSGA-II, NSGA-III and R-NSGA-II

The Non-dominated genetic algorithms II (NSGA-II) is a meta-heuristic proposed by
K. Deb, A. Pratap, S. Agarwal and T. Meyarivan in 2002. The purpose of the
algorithms is to find an efficient way to optimize multi-objectives functions
(two or more).

The Non-dominated genetic algorithms III (NSGA-III) is a meta-heuristic proposed by
K. Deb and H. Jain in 2013.
The purpose of the algorithms is to find an efficient way to optimize
multi-objectives functions (more than three).

The Reference point-based Non-dominated genetic algorithms II (R-NSGA-II) is
a meta-heuristic proposed by K. Deb and J. Sundar in 2006. It is a modification
of NSGA-II based on reference points in which the decision-maker supplies one
or more preference points and a weight vector that will guide the solutions
towards regions desired by the user.
}
\examples{
#Example 1
#Two Objectives - Real Valued
zdt1 <- function (x) {
 if (is.null(dim(x))) {
   x <- matrix(x, nrow = 1)
 }
 n <- ncol(x)
 g <- 1 + rowSums(x[, 2:n, drop = FALSE]) * 9/(n - 1)
 return(cbind(x[, 1], g * (1 - sqrt(x[, 1]/g))))
}

#Not run:
\dontrun{
result <- rmoo(type = "real-valued",
               fitness = zdt1,
               algorithm = "NSGA-II",
               lower = c(0,0),
               upper = c(1,1),
               popSize = 100,
               nObj = 2,
               monitor = FALSE,
               maxiter = 500)

}

#Example 2
#Three Objectives - Real Valued
dtlz1 <- function (x, nobj = 3){
    if (is.null(dim(x))) {
        x <- matrix(x, 1)
    }
    n <- ncol(x)
    y <- matrix(x[, 1:(nobj - 1)], nrow(x))
    z <- matrix(x[, nobj:n], nrow(x))
    g <- 100 * (n - nobj + 1 + rowSums((z - 0.5)^2 - cos(20 * pi * (z - 0.5))))
    tmp <- t(apply(y, 1, cumprod))
    tmp <- cbind(t(apply(tmp, 1, rev)), 1)
    tmp2 <- cbind(1, t(apply(1 - y, 1, rev)))
    f <- tmp * tmp2 * 0.5 * (1 + g)
    return(f)
}

#Define uniformly distributed reference points.
ref_points <- generate_reference_points(3,12)

#Not Run
\dontrun{
result <- rmoo(type = "real-valued",
                fitness = dtlz1,
                algorithm = "NSGA-III",
                lower = c(0,0,0),
                upper = c(1,1,1),
                popSize = 92,
                nObj = 3,
                reference_dirs = ref_points,
                monitor = FALSE,
                maxiter = 500)
}

#Example 3
#Two Objectives - Real Valued with Preference-guided
zdt2 <- function (x)
{
  if (is.null(dim(x))) {
    x <- matrix(x, nrow = 1)
  }
  n <- ncol(x)
  g <- 1 + rowSums(x[, 2:n, drop = FALSE]) * 9/(n - 1)
  return(cbind(x[, 1], g * (1 - (x[, 1]/g)^2)))
}

#Define uniformly distributed reference points.
ref_points <- rbind(c(1.0, 0.0), c(0.0, 1.0), c(0.5, 0.5))

#Not run
\dontrun{
result <- rmoo(type = "real-valued",
               fitness = zdt2,
               algorithm = "R-NSGA-II",
               lower = c(0,0),
               upper = c(1,1),
               reference_dirs = ref_points,
               popSize = 92,
               nObj = 2,
               monitor = FALSE,
               maxiter = 500)

}

}
\references{
Scrucca, L. (2017) On some extensions to 'GA' package: hybrid
optimisation, parallelisation and islands evolution. The R Journal, 9/1, 187-206.
doi: 10.32614/RJ-2017-008

Kalyanmoy Deb and J. Sundar. 2006. Reference point based multi-objective
optimization using evolutionary algorithms. In Proceedings of the 8th annual
conference on Genetic and evolutionary computation (GECCO '06). Association
for Computing Machinery, New York, NY, USA, 635–642. doi: 10.1145/1143997.1144112

K. Deb, A. Pratap, S. Agarwal and T. Meyarivan, 'A fast and
elitist multiobjective genetic algorithm: NSGA-II,' in IEEE Transactions on
Evolutionary Computation, vol. 6, no. 2, pp. 182-197, April 2002,
doi: 10.1109/4235.996017.

K. Deb and H. Jain, "An Evolutionary Many-Objective Optimization
Algorithm Using Reference-Point-Based Nondominated Sorting Approach, Part I:
Solving Problems With Box Constraints," in IEEE Transactions on Evolutionary
Computation, vol. 18, no. 4, pp. 577-601, Aug. 2014,
doi: 10.1109/TEVC.2013.2281535.
}
\seealso{
\code{\link[=nsga2]{nsga2()}}, \code{\link[=rnsga2]{rnsga2()}}, \code{\link[=nsga3]{nsga3()}}
}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\docType{methods}
\name{getFitness}
\alias{getFitness}
\title{Accessor methods to the fitness for rmoo results}
\usage{
getFitness(obj)
}
\arguments{
\item{obj}{an object resulting from the execution of NSGA-I, NSGA-II or NSGA-III
algorithm}
}
\value{
Prints the resulting fitness and when the result of the method-call
is assigned to a variable, the fitness is stored as a data frame.
See \linkS4class{nsga1} \linkS4class{nsga2}, \linkS4class{nsga3} for a description of available
slots information.
}
\description{
Accessor methods to the fitness for rmoo results
}
\examples{
# Where 'out' is an object resulting from the execution of the rmoo.
#
# fitness_result <- getFitness(out)
#
# fitness_result

}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/nsga.R, R/nsga3.R
\docType{methods}
\name{print}
\alias{print}
\alias{print,nsga-method}
\alias{print,nsga,missing-method}
\alias{print,nsga1-method}
\alias{print,nsga3-method}
\title{Methods for Function 'print' in Package 'rmoo'.}
\usage{
print(x, ...)

\S4method{print}{nsga}(x, ...)

\S4method{print}{nsga1}(x, ...)

\S4method{print}{nsga3}(x, ...)
}
\arguments{
\item{x}{Objects of either class \linkS4class{nsga1},
\linkS4class{nsga2},  or \linkS4class{nsga3}.}

\item{...}{other arguments passed on to methods}
}
\value{
Print the slots and relevant values of the object.
}
\description{
Method used to print the slots and relevant values of the object.
}
\examples{
# Where 'out' is an object of class nsga1, nsga2, or nsga3
#
# print(out)

}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geneticoperator.R
\name{rmoo_tourSelection}
\alias{rmoo_tourSelection}
\alias{rmooreal_tourSelection}
\alias{rmoobin_tourSelection}
\alias{rmooperm_tourSelection}
\title{Tournament Selection}
\usage{
rmoo_tourSelection(object, k = 2, ...)

rmooreal_tourSelection(object, k = 2, ...)

rmoobin_tourSelection(object, k = 2, ...)

rmooperm_tourSelection(object, k = 2, ...)
}
\arguments{
\item{object}{MOEA object with slots population, fitness, front, popSize.}

\item{k}{Tournament size.}

\item{...}{Argument which all the values necessary for the configuration
will be passed as parameters. The user is encouraged to see the documentations.}
}
\value{
List with \code{population} and \code{fitness} subsets.
}
\description{
Binarily o por pref
}
\examples{
\dontrun{
# Creamos un "dummy" objeto mínimo
object <- list(
  population       = matrix(runif(20), nrow=5),
  fitness          = matrix(runif(10), nrow=5),
  front            = matrix(sample(1:2, 5, TRUE), ncol=1),
  crowdingDistance = runif(5),
  popSize          = 5
)
class(object) <- "nsga2"
# Llamamos al selector
sel <- rmoo_tourSelection(object, k = 2)
str(sel)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parallel.R
\name{startParallel}
\alias{startParallel}
\title{Start Parallel Backend for rmoo Package}
\usage{
startParallel(parallel = TRUE, ...)
}
\arguments{
\item{parallel}{Logical, numeric, character, or a cluster object. If \code{TRUE}, uses all detected cores.
If numeric, it specifies the number of cores. If character, should be \code{"snow"} or \code{"multicore"}.
If a cluster object, it will use that cluster.}

\item{...}{Additional arguments (currently unused).}
}
\value{
An object of class \code{logical} with attributes:
\itemize{
\item \code{type}: cluster type ("snow" or "multicore")
\item \code{cores}: number of cores used
\item \code{cluster}: the cluster object created or passed
}
}
\description{
This function sets up parallel computing using the \code{parallel} and \code{doParallel} packages.
It supports both "snow" (PSOCK) and "multicore" backends depending on the OS.
}
\examples{
\dontrun{
  cl <- startParallel(TRUE)
  stopParallel(attr(cl, "cluster"))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{numberOrNAOrMatrix-class}
\alias{numberOrNAOrMatrix-class}
\title{Virtual Class 'numberOrNAOrMatrix - Simple Class for subassigment Values'}
\description{
The class 'numberOrNAOrMatrix' is a simple class union (\code{\link[=setClassUnion]{setClassUnion()}})
of 'numeric', 'logical', 'logical' and 'matrix'.
}
\section{Objects from the Class}{

Since it is a virtual Class, no objects may be created from it.
}

\examples{
showClass('numberOrNAOrMatrix')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/nsga.R, R/nsga2.R, R/nsga3.R
\docType{methods}
\name{summary}
\alias{summary}
\alias{summary,nsga-method}
\alias{summary,nsga,nsga-method}
\alias{summary,nsga1-method}
\alias{summary,nsga2-method}
\alias{summary,nsga3-method}
\title{Methods for Function 'summary' in Package 'rmoo'}
\usage{
summary(object, ...)

\S4method{summary}{nsga}(object, ...)

\S4method{summary}{nsga1}(object, ...)

\S4method{summary}{nsga2}(object, ...)

\S4method{summary}{nsga3}(object, ...)
}
\arguments{
\item{object}{Objects of either class \linkS4class{nsga1},
\linkS4class{nsga2},  or \linkS4class{nsga3}.}

\item{...}{other arguments passed on to methods. Passing \code{"reference_dirs"}
as arguments will evaluate the performance metrics Hypervolumen,
Generational Distance, and Inverse Generational Distance.}
}
\value{
A summary of the values resulting from the execution of an algorithm.
}
\description{
Method used to summarize the results of the evaluations, passing additional
arguments in the summary method the performance metrics is evaluated.
}
\examples{
# Where 'out' is an object of class nsga1, nsga2, or nsga3
#
# summary(out)
#
# For the evaluation of the metrics, pass the reference point
#
# ref_points <- generate_reference_points(3,12)
# summary(out, reference_dirs = ref_points)

}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reference_point_multi_layer.R
\name{reference_point_multi_layer}
\alias{reference_point_multi_layer}
\title{Determination of Multi-layer Reference Points}
\usage{
reference_point_multi_layer(...)
}
\arguments{
\item{...}{The different layers provided by the user}
}
\value{
A matrix with the multi-layer reference points
}
\description{
A implementation of Multi-layer Reference Points Generation.
}
\details{
The Multi-layer reference point implementation is based on Blank and Deb's
pymoo library, the approach generates different layers of references point
at different scales, provided by the user.
}
\references{
J. Blank and K. Deb, "Pymoo: Multi-Objective Optimization in
Python," in IEEE Access, vol. 8, pp. 89497-89509, 2020,
doi: 10.1109/ACCESS.2020.2990567.

Das, Indraneel & Dennis, J. (2000). Normal-Boundary Intersection: A New
Method for Generating the Pareto Surface in Nonlinear Multicriteria
Optimization Problems. SIAM Journal on Optimization. 8.
10.1137/S1052623496307510.
}
\seealso{
\code{\link[=generate_reference_points]{generate_reference_points()}} and \code{\link[=get_fixed_rowsum_integer_matrix]{get_fixed_rowsum_integer_matrix()}}
}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{nsga-class}
\alias{nsga-class}
\title{Virtual Class 'nsga'}
\description{
The 'nsga' class is the parent superclass of the \linkS4class{nsga1},
\linkS4class{nsga2},  and \linkS4class{nsga3} classes
}
\section{Slots}{

\describe{
\item{\code{call}}{an object of class 'call' representing the matched call.}

\item{\code{type}}{a character string specifying the type of genetic algorithm used.}

\item{\code{lower}}{a vector providing for each decision variable the lower bounds of
the search space in case of real-valued or permutation encoded optimisations.}

\item{\code{upper}}{a vector providing for each decision variable the upper bounds of
the search space in case of real-valued or permutation encoded optimizations.}

\item{\code{nBits}}{a value specifying the number of bits to be used in binary
encoded optimizations.}

\item{\code{names}}{a vector of character strings providing the names of decision
variables (optional).}

\item{\code{nvars}}{a}

\item{\code{popSize}}{the population size.}

\item{\code{front}}{Rank of individuals on the non-dominated front.}

\item{\code{f}}{Front of individuals on the non-dominated front.}

\item{\code{iter}}{the actual (or final) iteration of NSGA search.}

\item{\code{run}}{the number of consecutive generations without any improvement in
the best fitness value before the NSGA is stopped.}

\item{\code{maxiter}}{the maximum number of iterations to run before the NSGA search
is halted.}

\item{\code{suggestions}}{a matrix of user provided solutions and included in the
initial population.}

\item{\code{population}}{the current (or final) population.}

\item{\code{pcrossover}}{the crossover probability.}

\item{\code{pmutation}}{the mutation probability.}

\item{\code{fitness}}{the values of fitness function for the current (or final)
population.}

\item{\code{summary}}{a matrix of summary statistics for fitness values at each
iteration (along the rows).}

\item{\code{fitnessValue}}{the best fitness value at the final iteration.}

\item{\code{solution}}{the value(s) of the decision variables giving the best fitness
at the final iteration.}

\item{\code{execution_time}}{a}
}}

\section{Objects from the Class}{

Since it is a virtual Class, no objects may be created from it.
}

\examples{
showClass('nsga')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_reference_points.R
\name{scale_reference_directions}
\alias{scale_reference_directions}
\title{Scale Reference Points}
\usage{
scale_reference_directions(ref_dirs, scaling)
}
\arguments{
\item{ref_dirs, scaling}{where 'ref_dirs' are the reference points generated and
'scaling' are the scale on which the points are distributed.}
}
\value{
A matrix with rescaled reference points uniformly distributed.
}
\description{
A implementation of Das and Dennis's Reference Points Generation.
}
\details{
The implemented Reference Point Generation is based on the Das and Dennis's
systematic approach that places points on a normalized hyper-plane which is
equally inclined to all objective axes and has an intercept of one on each axis.
}
\references{
J. Blank and K. Deb, "Pymoo: Multi-Objective Optimization in Python," in
IEEE Access, vol. 8, pp. 89497-89509, 2020, doi: 10.1109/ACCESS.2020.2990567.
}
\seealso{
\code{\link[=generate_reference_points]{generate_reference_points()}} and \code{\link[=get_fixed_rowsum_integer_matrix]{get_fixed_rowsum_integer_matrix()}}
}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/nsga.R, R/nsga2.R, R/nsga3.R
\docType{methods}
\name{progress}
\alias{progress}
\alias{progress,nsga-method}
\alias{progress,nsga,nsga-method}
\alias{progress,nsga1-method}
\alias{progress,nsga2-method}
\alias{progress,nsga3-method}
\title{Methods for Function 'progress' in Package 'rmoo'}
\usage{
progress(object, ...)

\S4method{progress}{nsga}(object, ...)

\S4method{progress}{nsga1}(object, ...)

\S4method{progress}{nsga2}(object, ...)

\S4method{progress}{nsga3}(object, ...)
}
\arguments{
\item{object}{Objects of either class \linkS4class{nsga1},
\linkS4class{nsga2},  or \linkS4class{nsga3}.}

\item{...}{other arguments passed on to methods. Passing \code{"reference_dirs"}
as arguments will evaluate the performance metrics Hypervolumen,
Generational Distance, and Inverse Generational Distance.}
}
\value{
A list of length equal to the number of iterations, where the progress made during execution is saved.
}
\description{
Method used to save the progress of the evaluation results, similar to the
summary method. Passing additional arguments to the progress method evaluates
performance metrics per iteration. This method cannot be called outside of
rmoo execution.
}
\examples{
# Where 'out' is an object of class nsga1, nsga2, or nsga3, and callArgs are
# the additional arguments passed when calling the rmoo function, for the
# evaluation of performance metrics, reference points are expected to be passed
# as an argument to reference_dirs.
#
# progress(object, callArgs)
#

}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_reference_points.R
\name{generate_reference_points}
\alias{generate_reference_points}
\title{Determination of Reference Points on a Hyper-Plane}
\usage{
generate_reference_points(m, h, scaling = NULL)
}
\arguments{
\item{m, h, scaling}{Number of reference points 'h' in M-objective problems, and
scaling that is the scale on which the points are distributed.}
}
\value{
A matrix with the reference points uniformly distributed.
}
\description{
A implementation of Das and Dennis's Reference Points Generation.
}
\details{
The implemented Reference Point Generation is based on the Das and Dennis's
systematic approach that places points on a normalized hyper-plane which is
equally inclined to all objective axes and has an intercept of one on each axis.
}
\references{
K. Deb and H. Jain, 'An Evolutionary Many-Objective Optimization
Algorithm Using Reference-Point-Based Nondominated Sorting Approach, Part I:
Solving Problems With Box Constraints,' in IEEE Transactions on Evolutionary
Computation, vol. 18, no. 4, pp. 577-601, Aug. 2014,
doi: 10.1109/TEVC.2013.2281535.

Das, Indraneel & Dennis, J. (2000). Normal-Boundary Intersection: A New
Method for Generating the Pareto Surface in Nonlinear Multicriteria
Optimization Problems. SIAM Journal on Optimization. 8.
10.1137/S1052623496307510.
}
\seealso{
\code{\link[=non_dominated_fronts]{non_dominated_fronts()}} and \code{\link[=get_fixed_rowsum_integer_matrix]{get_fixed_rowsum_integer_matrix()}}
}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modified_crowding_distance.R
\name{calc_norm_pref_distance}
\alias{calc_norm_pref_distance}
\title{Calculate Normalized Preference Distance
Computes the weighted normalized Euclidean distance between a set of fitness
vectors and a set of reference points.}
\usage{
calc_norm_pref_distance(fitness, ref_points, weight, ideal_point, nadir_point)
}
\arguments{
\item{fitness}{A matrix of fitness values.}

\item{ref_points}{A matrix of reference points.}

\item{weight}{A numeric vector of weights for each objective.}

\item{ideal_point}{A numeric vector of ideal point values.}

\item{nadir_point}{A numeric vector of nadir point values.}
}
\value{
A matrix of distances where element (i, j) is the distance from
fitness to ref_points.
}
\description{
Calculate Normalized Preference Distance
Computes the weighted normalized Euclidean distance between a set of fitness
vectors and a set of reference points.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geneticoperator.R
\name{rmoo_huxCrossover}
\alias{rmoo_huxCrossover}
\alias{rmooint_huxCrossover}
\alias{rmoobin_huxCrossover}
\title{Half Uniform Crossover (HUX)}
\usage{
rmoo_huxCrossover(object, parents, prob_hux = 0.5)

rmooint_huxCrossover(object, parents, prob_hux = 0.5)

rmoobin_huxCrossover(object, parents, prob_hux = 0.5)
}
\arguments{
\item{object}{MOEA object with slots population and fitness.}

\item{parents}{Integer vector of length 2 with the parent row indices.}

\item{prob_hux}{Proportion of differing loci to swap (default 0.5).}
}
\value{
A list with \code{children} (2 × n integer matrix) and
\code{fitness} (2 × nObj NA matrix).
}
\description{
Produces two children by swapping exactly half of the genes
that differ between the two parents. Only loci where the parents disagree
are eligible, making HUX more conservative than full uniform crossover.
}

\name{performance_metrics}
\alias{performance_metrics}
%
\alias{generational_distance}

\title{Objective Values performance metrics}

\description{Functions to evaluate the quality of the results obtained by the algorithms, evaluating their diversity and convergence, providing or not some parameters to compare.}

\usage{
  generational_distance(front, true_pareto_front, p, inverted, plus)

}

\arguments{
  \item{front}{a N×M matrix where N is the number of points and M is the number of objectives.}

  \item{true_pareto_front}{a N×M matrix where N is the number of points and M is the number of objectives.}

  \item{p}{is the power in which the normalized distance is calculated.}

  \item{inverted}{if TRUE then computes IGD.}

  \item{plus}{if TRUE then computes the GD+.}
}

\value{A vector with the measurement metric.}

\references{
  Lamont, G., & Veldhuizen, D.V. (1999). Multiobjective evolutionary algorithms: classifications, analyses, and new innovations.

}

\author{Francisco Benitez}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\docType{methods}
\name{getMetrics}
\alias{getMetrics}
\alias{getMetrics,nsga-method}
\alias{getMetrics,nsga,nsga-method}
\title{Accessor methods to the metrics evaluated during execution}
\usage{
getMetrics(obj)

\S4method{getMetrics}{nsga}(obj)
}
\arguments{
\item{obj}{an object resulting from the execution of NSGA-I, NSGA-II or NSGA-III
algorithm. During the execution of the performance metrics must be evaluated.}
}
\value{
A dataframe with performance metrics evaluated iteration by iteration.
}
\description{
Accessor methods to the metrics evaluated during execution
}
\examples{
# Where 'out' is an object resulting from the execution of the rmoo.
#
# metrics_result <- getMetrics(out)
#
# metrics_result

}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

\name{rmoo_Mutation}
\alias{rmoo_Mutation}
%
\alias{rmoobin_raMutation}
%
\alias{rmooreal_raMutation}
\alias{rmooreal_polMutation}
%
\alias{rmooperm_simMutation}

\title{Mutation operators in non-dominated genetic algorithms}

\description{Functions implementing mutation non-dominated genetic operator.}

\usage{
  rmoobin_raMutation(object, parent)

  rmooreal_raMutation(object, parent)
  rmooreal_polMutation(object, parent, eta = 20, indpb = 0.5)

  rmooperm_simMutation(object, parent)
}

\arguments{
  \item{object}{An object of class \code{"nsga"}, \code{"nsga2"} or \code{"nsga3"} usually resulting from a call to function \code{\link{nsga}}, \code{\link{nsga2}}, \code{\link{nsga3}}.}

  \item{parent}{A vector of values for the parent from the current population where mutation should occur.}

  \item{eta}{Distribution parameter for mutation operator.}

  \item{indpb}{Independent mutation probability.}
}

\value{Return a vector of values containing the mutated string.}

\references{
  Scrucca, L. (2017) On some extensions to 'GA' package: hybrid optimisation, parallelisation and islands evolution. The R Journal, 9/1, 187-206, doi: 10.32614/RJ-2017-008.
}

\author{Francisco Benitez}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/nsga.R, R/nsga2.R, R/nsga3.R,
%   R/rnsga2.R
\docType{methods}
\name{plot}
\alias{plot}
\alias{plot,nsga,missing-method}
\alias{plot,nsga,missing}
\alias{plot,nsga1,missing-method}
\alias{plot,nsga1-method}
\alias{plot,nsga2,missing-method}
\alias{plot,nsga2-method}
\alias{plot,nsga3,missing-method}
\alias{plot,nsga3-method}
\alias{plot,rnsga2,missing-method}
\alias{plot,rnsga2-method}
\title{Methods for Function 'plot' in Package 'rmoo'}
\usage{
plot(x, y, ...)

\S4method{plot}{nsga,missing}(x, y = "missing", type = c("scatter", "pcp", "heatmap", "polar"), ...)

\S4method{plot}{nsga1,missing}(x, y = "missing", type = c("scatter", "pcp", "heatmap", "polar"), ...)

\S4method{plot}{nsga2,missing}(x, y = "missing", type = c("scatter", "pcp", "heatmap", "polar"), ...)

\S4method{plot}{nsga3,missing}(x, y = "missing", type = c("scatter", "pcp", "heatmap", "polar"), ...)

\S4method{plot}{rnsga2,missing}(x, y = "missing", type = c("scatter", "pcp", "heatmap", "polar"), ...)
}
\arguments{
\item{x, y}{Objects of either class \linkS4class{nsga1},
\linkS4class{nsga2},  or \linkS4class{nsga3}.}

\item{...}{other arguments passed on to methods
\describe{
\item{"optimal"}{An argument passed to the "scatter" plot. A matrix of
dimension equal to the fitness with which they are compared. This value can
only be compared in 2 and 3 dimensional "scatter" plots.}
\item{"individual"}{An argument passed to the "heatmap" and "polar" plots.
A vector that represents the fitness of the individuals to be displayed.}
}}

\item{type}{Type of graph to draw, the graphs can be of the type "scatter",
"pcp", "heatmap", or "polar"}
}
\value{
A graph of the evaluated type.
}
\description{
Method used to visualize the fitness of the individuals during the execution
of the algorithms.
}
\details{
The following plots are available:

\itemize{
\item "Scatter Plot"
\item "Parallel Coordinate Plot"
\item "Heat Map"
\item "Polar Coordinate"
}
}
\examples{
# Where 'out' is an object of class nsga1, nsga2, or nsga3.
# The plot method will by default plot a scatter plot.
#
# plot(out)
#
# The Parallel Coordinate Plot will be plotted if "pcp" is passed as a parameter to "type".
#
# plot(out, type="pcp")
#
# A heat map plot will be plotted if "heatmap" is passed as a parameter to "type"
# and a vector with the individuals to plot to "individual"
#
# plot(out, type = "heatmap", individual = c(1:5))
#
# A polar coordinate plot will be plotted if "polar" is passed as a parameter to "type"
# and a vector with the individuals to plot to "individual"
#
# plot(out, type = "polar", individual = c(1:5))

}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{rnsga2-class}
\alias{rnsga2-class}
\title{Class 'rnsga2'}
\description{
The class 'rnsga2' is instantiated within the execution of rmoo and will be
returned as a result of it. All data generated during execution will be
stored in it.
}
\section{Slots}{

\describe{
\item{\code{crowdingDistance}}{Crowding-comparison approach to estimate of the
perimeter of the cuboid formed by using the nearest neighbors as the vertices.}

\item{\code{reference_points}}{R-NSGA-II uses a set of reference points defined by the user to
ensure diversity in obtained solutions.}

\item{\code{extreme_points}}{are selected using the ASF in the (\code{\link[=PerformScalarizing]{PerformScalarizing()}}).
Necessary in the  nadir point generation.}

\item{\code{smin}}{Index used to obtain the extreme points.}
}}

\examples{
showClass('rnsga2')
}

\name{rmoo_Population}
\alias{rmoo_Population}
%
\alias{rmoobin_Population}
%
\alias{rmooreal_Population}
%
\alias{rmooperm_Population}
%
\alias{rmooint_Population}

\title{Population initialization in non-dominated genetic algorithms}

\description{Functions for creating a random initial population to be used in non-dominated genetic algorithms.}

\usage{
  rmoobin_Population(object)

  rmooreal_Population(object)

  rmooperm_Population(object)

  rmooint_Population(object)
}

\arguments{

  \item{object}{An object of class \code{\link{nsga-class}}, \code{\link{nsga2-class}} or \code{\link{nsga3-class}}.}

}

\details{
  \code{rmoobin_Population} generates a random population of \code{object@nBits} binary values;

  \code{rmooreal_Population} generates a random (uniform) population of real values in the range [\code{object@lower}, \code{object@upper}];

  \code{rmooperm_Population} generates a random (uniform) population of permutation values in the range [\code{object@lower}, \code{object@upper}].

  \code{rmooint_Population} generates a random (uniform) population of integer values in the range [\code{object@lower}, \code{object@upper}].
}

\value{Return a matrix of dimension \code{object@popSize} times the number of decision variables.}

\references{
  Scrucca, L. (2017) On some extensions to 'GA' package: hybrid optimisation, parallelisation and islands evolution. The R Journal, 9/1, 187-206, doi: 10.32614/RJ-2017-008.
}

\author{Francisco Benitez}

\seealso{
  \code{\link{nsga}},
  \code{\link{nsga2}} and
  \code{\link{nsga3}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geneticoperator.R
\name{rmoo_uxMutation}
\alias{rmoo_uxMutation}
\alias{rmooint_uxMutation}
\alias{rmoobin_uxMutation}
\title{Uniform Mutation}
\usage{
rmoo_uxMutation(object, parent, indpb = 0.1)

rmooint_uxMutation(object, parent, indpb = 0.1)

rmoobin_uxMutation(object, parent, indpb = 0.1)
}
\arguments{
\item{object}{MOEA object with slots population, lower, upper.}

\item{parent}{Index of the individual to mutate.}

\item{indpb}{Per-gene mutation probability (default 0.1).}
}
\value{
An integer vector of the mutated individual.
}
\description{
Mutates an individual by randomly replacing each gene with a
new value drawn uniformly from the integer range \code{[lower, upper]},
independently at each locus with probability \code{indpb}.
}

\name{associate}
\alias{associate}
%
\alias{associate_to_niches}
\alias{compute_perpendicular_distance}
\alias{compute_niche_count}

\title{Association Operation in Non-Dominated Genetic Algorithms III}

\description{Function that associates each member of the population with a reference point. The function calculates the perpendicular distance of each individual from each of the reference lines. This code section corresponds to Algorithm 3 of the referenced paper.}

\usage{
  associate_to_niches(object, utopian_epsilon = 0)
  compute_perpendicular_distance(x, y)
  compute_niche_count(n_niches, niche_of_individuals)
}

\arguments{
  \item{object}{An object of class \code{"nsga3"}.}
  
  \item{utopian_epsilon}{The epsilon used for decrease the ideal point to get the utopian point.}
  
  \item{x}{Individuals to calculate their niche.}

  \item{y}{Reference points.}

  \item{n_niches}{Number of reference points.}

  \item{niche_of_individuals}{The niche count of individuals, except the last front.}
}

\value{Returns a list with the niche count of individuals and the distances between them.}

\references{
  J. Blank and K. Deb, "Pymoo: Multi-Objective Optimization in Python," in IEEE Access, vol. 8, pp. 89497-89509, 2020, doi: 10.1109/ACCESS.2020.2990567.

  K. Deb and H. Jain, "An Evolutionary Many-Objective Optimization Algorithm Using Reference-Point-Based Nondominated Sorting Approach, Part I: Solving Problems With Box Constraints," in IEEE Transactions on Evolutionary Computation, vol. 18, no. 4, pp. 577-601, Aug. 2014, doi: 10.1109/TEVC.2013.2281535.
}

\author{Francisco Benitez}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geneticoperator.R
\name{rmoo_uxCrossover}
\alias{rmoo_uxCrossover}
\alias{rmooint_uxCrossover}
\alias{rmoobin_uxCrossover}
\title{Uniform Crossover}
\usage{
rmoo_uxCrossover(object, parents)

rmooint_uxCrossover(object, parents)

rmoobin_uxCrossover(object, parents)
}
\arguments{
\item{object}{MOEA object with slots population and fitness.}

\item{parents}{Integer vector of length 2 with the parent row indices.}
}
\value{
A list with \code{children} (2 × n integer matrix) and
\code{fitness} (2 × nObj NA matrix).
}
\description{
Produces two children by randomly swapping genes between two
parents with equal probability at each locus. Each gene is inherited from
either parent independently, giving maximum gene-level mixing.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/niching.R
\name{niching}
\alias{niching}
\title{Niche-Preservation Operation}
\usage{
niching(pop, n_remaining, niche_count, niche_of_individuals, dist_to_niche)
}
\arguments{
\item{pop}{Last Front Population}

\item{n_remaining}{Number of points to choose}

\item{niche_count}{Niche count of individuals with the reference point}

\item{niche_of_individuals}{Count of the closest reference point to the last
front objective values}

\item{dist_to_niche}{Distance between closest reference point to last front
objective values}
}
\value{
Returns the association of reference points to each individual in the population.
}
\description{
Generation of niche, by associating reference points to population members
}
\details{
Niching procesure is a algorithms proposed by K. Deb and H. Jain in 2013.
}
\references{
K. Deb and H. Jain, 'An Evolutionary Many-Objective Optimization
Algorithm Using Reference-Point-Based Nondominated Sorting Approach, Part I:
Solving Problems With Box Constraints,' in IEEE Transactions on Evolutionary
Computation, vol. 18, no. 4, pp. 577-601, Aug. 2014,
doi: 10.1109/TEVC.2013.2281535.

Scrucca, L. (2017) On some extensions to 'GA' package: hybrid optimisation,
parallelisation and islands evolution. The R Journal, 9/1, 187-206.
doi: 10.32614/RJ-2017-008

Felix-Antoine Fortin, Francois-Michel De Rainville, Marc-André Gardner
Gardner, Marc Parizeau, and Christian Gagne. 2012. DEAP: evolutionary
algorithms made easy. J. Mach. Learn. Res. 13, 1 (January 2012), 2171–2175.
}
\seealso{
\code{\link[=associate_to_niches]{associate_to_niches()}}, \code{\link[=PerformScalarizing]{PerformScalarizing()}}
}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modified_crowding_distance.R
\name{modifiedCrowdingDistance}
\alias{modifiedCrowdingDistance}
\title{Calculation of Modified Crowding Distance}
\usage{
modifiedCrowdingDistance(
  object,
  epsilon,
  weights = NULL,
  normalization = "front",
  extreme_points_as_ref_dirs = FALSE
)
}
\arguments{
\item{object}{An object of class 'rnsga2', typically from a call to r-nsga2.
Must contain fitness, population, fronts, popSize, and reference_points.}

\item{epsilon}{Minimum allowed distance between solutions to avoid duplicates.}

\item{weights}{A numeric vector of weights for preference distance (default is equal weights).}

\item{normalization}{Type of normalization to apply: \code{"ever"}, \code{"front"}, or \code{"no"}.}

\item{extreme_points_as_ref_dirs}{Logical; whether to use extreme points as reference directions.}
}
\value{
A list with:
\describe{
\item{survivors}{Indices of selected individuals}
\item{indexmin}{Index of individuals with minimum scalarizing value (optional)}
\item{reference_points}{Updated reference points matrix}
}
}
\description{
A Crowded-comparison approach.
}
\details{
The crowded-comparison operator maintain diversity in the Pareto front
during multi-objective optimization. This version uses a reference point-based
normalization and preference distance strategy.
}
\references{
Kalyanmoy Deb and J. Sundar (2006). GECCO '06. doi:10.1145/1143997.1144112
}
\seealso{
\code{\link[=rnsga2]{rnsga2()}}
}
\author{
Francisco Benitez
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{nsga1-class}
\alias{nsga1-class}
\title{Class 'nsga1'}
\description{
The class 'nsga1' is instantiated within the execution of rmoo and will be
returned as a result of it. All data generated during execution will be
stored in it.
}
\section{Slots}{

\describe{
\item{\code{dumFitness}}{a large dummy fitness value assigned to individuals from
the nondominated front.}

\item{\code{dShare}}{the maximun phenotypic distance allowed between any two
individuals to become members of a niche.}

\item{\code{deltaDummy}}{value to decrease the dummy fitness of individuals by
non-dominated fronts.}
}}

\examples{
showClass('nsga1')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{kroC100}
\alias{kroC100}
\title{KROC100}
\format{
A data frame with 100 rows and 2 variables:
\describe{
\item{COORD}{City Coordinates}
\item{SECTION}{City Section}
}
}
\usage{
kroC100
}
\description{
A dataset containing the coord and section of 100 cities
}
\references{
Reinelt, G. (1991). TSPLIB—A traveling salesman problem library.
ORSA journal on computing, 3(4), 376-384
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/non_dominated_fronts.R
\name{non_dominated_fronts}
\alias{non_dominated_fronts}
\title{Calculate of Non-Dominated Front}
\usage{
non_dominated_fronts(object)
}
\arguments{
\item{object}{An object of class 'nsga', usually resulting from a call to
function nsga, nsga2 and nsga3.}
}
\value{
A list with 'non-dominated fronts' and 'occupied positions' on the fronts.
}
\description{
A fast approach for calculate Non-Dominated Fronts.
}
\details{
Function to determine the non-dominated fronts of a population and the
aptitude value.
}
\references{
K. Deb, A. Pratap, S. Agarwal and T. Meyarivan, 'A fast and
elitist multiobjective genetic algorithm: NSGA-II,' in IEEE Transactions on
Evolutionary Computation, vol. 6, no. 2, pp. 182-197, April 2002,
doi: 10.1109/4235.996017.
}
\seealso{
\code{\link[=nsga]{nsga()}}, \code{\link[=nsga2]{nsga2()}} and \code{\link[=nsga3]{nsga3()}}
}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

\name{rmooControl}
\alias{rmooControl}

\title{A function for setting or retrieving defaults non-dominated genetic operators}

\description{Default settings for non-dominated genetic operators used in the 'rmoo' package.}

\usage{
  rmooControl(\dots)
}

\arguments{
  \item{\dots}{
    no arguments, a single character vector, or a named list with components.
  }}

\details{
  If the function is called with no arguments returns the current default
  settings, i.e., a list with the following default components:
    \describe{
      \item{\code{"binary"}}{
        \itemize{
          \item \code{population} = \code{"rmoobin_Population"}
          \item \code{selection} = \code{"rmoobin_tourSelection"}
          \item \code{crossover} = \code{"rmoobin_spCrossover"}
          \item \code{mutation} = \code{"rmoobin_raMutation"}
        }
      }
      \item{\code{"real-valued"}}{
        \itemize{
          \item \code{population} = \code{"rmooreal_Population"}
          \item \code{selection} = \code{"rmooreal_tourSelection"}
          \item \code{crossover} = \code{"rmooreal_sbxCrossover"}
          \item \code{mutation} = \code{"rmooreal_polMutation"}
        }
      }
      \item{\code{"permutation"}}{
        \itemize{
          \item \code{population} = \code{"rmooperm_Population"}
          \item \code{selection} = \code{"rmooperm_tourSelection"}
          \item \code{crossover} = \code{"rmooperm_oxCrossover"}
          \item \code{mutation} = \code{"rmooperm_simMutation"}
        }
      }
      \item{\code{"discrete"}}{
        \itemize{
          \item \code{population} = \code{"rmooint_Population"}
          \item \code{selection} = \code{"rmooint_tourSelection"}
          \item \code{crossover} = \code{"rmooint_uxCrossover"}
          \item \code{mutation} = \code{"rmooint_uxMutation"}
        }
      }

      \item{\code{"eps"} = }{the tolerance value used by the package functions. By default set at \code{sqrt(.Machine$double.eps)}.}
    }

  The function may be called with a single string specifying the name of the component. In this case the function returns the current default settings.

  To change the default values, a named component must be followed by a single value (in case of \code{"eps"}) or a list of component(s) specifying the name of the function for a genetic operator. See the Examples section.
}

\value{
  If the argument list is empty the function returns the current list of values.
  If the argument list is not empty, the returned list is invisible.
}

\note{
  The parameter values set via a call to this function will remain in effect for the rest of the session, affecting the subsequent behaviour of the functions for which the given parameters are relevant.
}

\author{Francisco Benitez}

\references{
  Scrucca, L. (2017) On some extensions to 'GA' package: hybrid optimisation, parallelisation and islands evolution. The R Journal, 9/1, 187-206, doi: 10.32614/RJ-2017-008.
}

\seealso{
  \code{\link[=nsga2]{nsga2()}},
  \code{\link[=rnsga2]{rnsga2()}} and
  \code{\link[=nsga3]{nsga3()}}
}

\examples{
  # get and save defaults
  defaultControl <- rmooControl()
  print(defaultControl)
  # get current defaults only for real-valued search
  rmooControl("real-valued")
  # set defaults for selection operator of real-valued search
  rmooControl("real-valued" = list(selection = "rmooreal_lrSelection"))
  rmooControl("real-valued")
  # set defaults for selection and crossover operators of real-valued search
  rmooControl("real-valued" = list(selection = "rmooreal_lrSelection",
                                   crossover = "rmooreal_spCrossover"))
  rmooControl("real-valued")
  # restore defaults
  rmooControl(defaultControl)
  rmooControl()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/nsga2.R
\docType{methods}
\name{getCrowdingDistance}
\alias{getCrowdingDistance}
\alias{getCrowdingDistance,nsga2-method}
\title{Accessor methods to the crowding distance for NSGA-II results}
\usage{
getCrowdingDistance(obj)

\S4method{getCrowdingDistance}{nsga2}(obj)
}
\arguments{
\item{obj}{an object resulting from the execution of NSGA-II algorithm}
}
\value{
Returns a vector with the crowding distances of class nsga2. See \linkS4class{nsga2}
for a description of available slots information.
}
\description{
Accessor methods to the crowding distance for NSGA-II results
}
\examples{
# Where 'out' is an object resulting from the execution of the NSGA-II algorithm.
#
# getCrowdingDistance(out)
#

}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{nsga2-class}
\alias{nsga2-class}
\title{Class 'nsga2'}
\description{
The class 'nsga2' is instantiated within the execution of rmoo and will be
returned as a result of it. All data generated during execution will be
stored in it.
}
\section{Slots}{

\describe{
\item{\code{crowdingDistance}}{Crowding-comparison approach to estimate of the
perimeter of the cuboid formed by using the nearest neighbors as the vertices.}
}}

\examples{
showClass('nsga2')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nsga.R
\name{nsga}
\alias{nsga}
\title{Non-Dominated Sorting in Genetic Algorithms}
\usage{
nsga(
  type = c("binary", "real-valued", "permutation"),
  fitness,
  ...,
  lower,
  upper,
  nBits,
  population = rmooControl(type)$population,
  selection = rmooControl(type)$selection,
  crossover = rmooControl(type)$crossover,
  mutation = rmooControl(type)$mutation,
  popSize = 50,
  nObj = NULL,
  dshare,
  pcrossover = 0.8,
  pmutation = 0.1,
  maxiter = 100,
  run = maxiter,
  maxFitness = Inf,
  names = NULL,
  suggestions = NULL,
  monitor = if (interactive()) rmooMonitor else FALSE,
  summary = FALSE,
  seed = NULL
)
}
\arguments{
\item{type}{the type of genetic algorithm to be run depending on the nature
of decision variables. Possible values are:
\describe{
\item{\code{"binary"}}{for binary representations of decision variables.}
\item{\code{"real-valued"}}{for optimization problems where the decision
variables are floating-point representations of real numbers.}
\item{\code{"permutation"}}{for problems that involves reordering of a list
of objects.}
}}

\item{fitness}{the fitness function, any allowable R function which takes as
input an individual string representing a potential solution, and returns a
numerical value describing its “fitness”.}

\item{...}{additional arguments to be passed to the fitness function. This
allows to write fitness functions that keep some variables fixed during the
search.}

\item{lower}{a vector of length equal to the decision variables providing the
lower bounds of the search space in case of real-valued or permutation
encoded optimizations.}

\item{upper}{a vector of length equal to the decision variables providing the
upper bounds of the search space in case of real-valued or permutation
encoded optimizations.}

\item{nBits}{a value specifying the number of bits to be used in binary
encoded optimizations.}

\item{population}{an R function for randomly generating an initial population.
See \code{\link[=rmoo_Population]{rmoo_Population()}} for available functions.}

\item{selection}{an R function performing selection, i.e. a function which
generates a new population of individuals from the current population
probabilistically according to individual fitness.}

\item{crossover}{an R function performing crossover, i.e. a function which
forms offsprings by combining part of the genetic information from
their parents. See \code{\link[=rmoo_Crossover]{rmoo_Crossover()}} for available functions.}

\item{mutation}{an R function performing mutation, i.e. a function which
randomly alters the values of some genes in a parent chromosome.
See \code{\link[=rmoo_Mutation]{rmoo_Mutation()}} for available functions.}

\item{popSize}{the population size.}

\item{nObj}{number of objective in the fitness function.}

\item{dshare}{the maximun phenotypic distance allowed between any two
individuals to become members of a niche.}

\item{pcrossover}{the probability of crossover between pairs of chromosomes.
Typically this is a large value and by default is set to 0.8.}

\item{pmutation}{the probability of mutation in a parent chromosome. Usually
mutation occurs with a small probability, and by default is set to 0.1.}

\item{maxiter}{the maximum number of iterations to run before the NSGA search
is halted.}

\item{run}{the number of consecutive generations without any improvement in
the best fitness value before the NSGA is stopped.}

\item{maxFitness}{the upper bound on the fitness function after that the NSGA
search is interrupted.}

\item{names}{a vector of character strings providing the names of decision
variables.}

\item{suggestions}{a matrix of solutions strings to be included in the
initial population. If provided the number of columns must match the number
of decision variables.}

\item{monitor}{a logical or an R function which takes as input the current
state of the nsga-class object and show the evolution of the search. By
default, for interactive sessions the function rmooMonitor prints the average
and best fitness values at each iteration. If set to plot these information
are plotted on a graphical device. Other functions can be written by the user
and supplied as argument. In non interactive sessions, by default
monitor = FALSE so any output is suppressed.}

\item{summary}{If there will be a summary generation after generation.}

\item{seed}{an integer value containing the random number generator state.
This argument can be used to replicate the results of a NSGA search. Note
that if parallel computing is required, the doRNG package must be installed.}
}
\value{
Returns an object of class nsga1-class. See \linkS4class{nsga1} for a
description of available slots information.
}
\description{
Minimization of a fitness function using Non-Dominated Genetic algorithms
(NSGA). Local search using general-purpose optimisation algorithms can be
applied stochastically to exploit interesting regions.
}
\details{
The Non-dominated genetic algorithms is a meta-heuristic proposed by
N. Srinivas and K. Deb in 1994. The purpose of the algorithms is to find an
efficient way to optimize multi-objectives functions (two or more).
}
\examples{
#Example
#Two Objectives - Real Valued
zdt1 <- function (x) {
 if (is.null(dim(x))) {
   x <- matrix(x, nrow = 1)
 }
 n <- ncol(x)
 g <- 1 + rowSums(x[, 2:n, drop = FALSE]) * 9/(n - 1)
 return(cbind(x[, 1], g * (1 - sqrt(x[, 1]/g))))
}

#Not run:
\dontrun{
result <- nsga(type = "real-valued",
               fitness = zdt1,
               lower = c(0,0),
               upper = c(1,1),
               popSize = 100,
               nObj = 2,
               dshare = 1,
               monitor = FALSE,
               maxiter = 500)
}

}
\references{
N. Srinivas and K. Deb, "Multiobjective Optimization Using
Nondominated Sorting in Genetic Algorithms, in Evolutionary Computation,
vol. 2, no. 3, pp. 221-248, Sept. 1994, doi: 10.1162/evco.1994.2.3.221.

Scrucca, L. (2017) On some extensions to 'GA' package: hybrid optimisation,
parallelisation and islands evolution. The R Journal, 9/1, 187-206.
doi: 10.32614/RJ-2017-008
}
\seealso{
\code{\link[=nsga2]{nsga2()}}, \code{\link[=nsga3]{nsga3()}}
}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crowding_distance.R
\name{crowding_distance}
\alias{crowding_distance}
\title{Calculation of Crowding Distance}
\usage{
crowding_distance(object, nObj)
}
\arguments{
\item{object, nObj}{An object of class 'nsga2', usually resulting from a call
to function nsga2. Fitness Function Objective Numbers}
}
\value{
A vector with the crowding-distance between individuals of a population.
}
\description{
A Crowded-comparison approach.
}
\details{
The crowded-comparison operator guides the selection process at the various
stages of the algorithm toward a uniformly spread-out Pareto-optimal front
}
\references{
K. Deb, A. Pratap, S. Agarwal and T. Meyarivan, 'A fast and
elitist multiobjective genetic algorithm: NSGA-II,' in IEEE Transactions on
Evolutionary Computation, vol. 6, no. 2, pp. 182-197, April 2002,
doi: 10.1109/4235.996017.
}
\seealso{
\code{\link[=non_dominated_fronts]{non_dominated_fronts()}}
}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

\name{rmooMonitor}
\alias{rmooMonitor}

\title{Monitor the execution of rmoo}

\description{Functions to plotting fitness values at each iteration of a search for the 'rmoo' package.}

\usage{
  rmooMonitor(object, \dots)
}

\arguments{
  \item{object}{an object of class \code{nsga}, \code{nsga2} or \code{nsga3}, usually resulting from a call to function \code{\link{nsga}}, \code{\link{nsga2}} or \code{\link{nsga3}}, respectively.}
  \item{\dots}{further arguments passed to or from other methods.}
}

\value{
  These functions plot the fitness values of the current step of the nsga3 on the console. \cr
  By default, \code{rmooMonitor} is called in interactive sessions by \code{\link{nsga}}, \code{\link{nsga2}}, or \code{\link{nsga3}}. \cr
  The function can be modified by the user to plot or print the values it considers by iteration. \cr
}

\references{
  Scrucca, L. (2017) On some extensions to 'GA' package: hybrid optimisation, parallelisation and islands evolution. The R Journal, 9/1, 187-206, doi: 10.32614/RJ-2017-008.
}

\author{Francisco Benitez}

\seealso{
  \code{\link[=nsga]{nsga()}},
  \code{\link[=nsga2]{nsga2()}} and
  \code{\link[=nsga3]{nsga3()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{nsga3-class}
\alias{nsga3-class}
\title{Class 'nsga3'}
\description{
The class 'nsga3' is instantiated within the execution of rmoo and will be
returned as a result of it. All data generated during execution will be
stored in it.
}
\section{Slots}{

\describe{
\item{\code{ideal_point}}{Nadir point estimate used as lower bound in normalization.}

\item{\code{worst_point}}{Worst point generated over generations.}

\item{\code{smin}}{Index used to obtain the extreme points.}

\item{\code{extreme_points}}{are selected using the ASF in the (\code{\link[=PerformScalarizing]{PerformScalarizing()}}).
Necessary in the  nadir point generation.}

\item{\code{worst_of_population}}{The worst individuals generated by objectives in
the current generation.}

\item{\code{worst_of_front}}{The worst individuals in the first front generated by
objectives in the current generation.}

\item{\code{nadir_point}}{Nadir point estimate used as upper bound in normalization.}

\item{\code{reference_points}}{NSGA-III uses a predefined set of reference points to
ensure diversity in obtained solutions.
The chosen refenrece points can be predefined in structured manner or
supplied by the user. We use the Das and Dennis procedure.}
}}

\examples{
showClass('nsga3')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{algorithm-class}
\alias{algorithm-class}
\title{Virtual Parent Class Algorithm}
\description{
It will use when other algorithms are implemented. Equivalent to a Abstract
class in other languages.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/nsga.R
\docType{methods}
\name{getDummyFitness}
\alias{getDummyFitness}
\alias{getDummyFitness,nsga1-method}
\title{Accessor methods to the dummy fitness for NSGA-I results}
\usage{
getDummyFitness(obj)

\S4method{getDummyFitness}{nsga1}(obj)
}
\arguments{
\item{obj}{an object resulting from the execution of NSGA-I algorithm}
}
\value{
Returns a matrix with the dummy fitness of class nsga1. See \linkS4class{nsga1}
for a description of available slots information.
}
\description{
Accessor methods to the dummy fitness for NSGA-I results
}
\examples{
# Where 'out' is an object resulting from the execution of the NSGA-I algorithm.
#
# getDummyFitness(out)
#

}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sharing.R
\name{sharing}
\alias{sharing}
\title{Calculation of Dummy Fitness}
\usage{
sharing(object)
}
\arguments{
\item{object}{An object of class 'nsga', usually resulting from a call to
function nsga. Fitness Function Objective Numbers.}
}
\value{
A vector with the dummy fitness.
}
\description{
Calculate of sharing distance and dummy fitness
}
\details{
The sharing distance operator guides the selection process at the various
stages of the algorithm toward a uniformly spread-out Pareto-optimal front
}
\references{
N. Srinivas and K. Deb, 'Multiobjective Optimization Using
Nondominated Sorting in Genetic Algorithms,' in Evolutionary Computation,
vol. 2, no. 3, pp. 221-248, Sept. 1994, doi: 10.1162/evco.1994.2.3.221.
}
\seealso{
\code{\link[=non_dominated_fronts]{non_dominated_fronts()}}
}
\author{
Francisco Benitez
\email{benitezfj94@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geneticoperator.R
\name{rmoo_lrSelection}
\alias{rmoo_lrSelection}
\alias{rmoobin_lrSelection}
\alias{rmooperm_lrSelection}
\alias{rmooreal_lrSelection}
\title{Linear Rank Selection}
\usage{
rmoo_lrSelection(object, r, q)

rmoobin_lrSelection(object, r, q)

rmooperm_lrSelection(object, r, q)

rmooreal_lrSelection(object, r, q)
}
\arguments{
\item{object}{MOEA object with slots population, fitness, front, popSize.}

\item{r}{Slope parameter controlling probability spread (computed from
popSize if missing).}

\item{q}{Intercept parameter (computed from popSize if missing).}
}
\value{
A list with \code{population} and \code{fitness} of the selected
individuals.
}
\description{
Selects individuals from the population using linear rank-based
probabilities. Individuals are ranked by their Pareto front, and selection
probability is assigned linearly so better-ranked individuals are more
likely to be chosen.
}
